@@ -17,18 +17,17 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
+
+
 /**
- * File containing CAPIInfo class for API.
+ * Class containing methods for operations with API.
+ *
  * @package API
  */
-/**
- * Class containing methods for operations with APIInfo
- */
 class CAPIInfo extends CZBXAPI {
+
 	/**
-	 * Get API version
+	 * Get API version.
 	 *
 	 * @return string
 	 */
@@ -36,4 +35,3 @@ class CAPIInfo extends CZBXAPI {
 		return ZABBIX_API_VERSION;
 	}
 }
-?>
