@@ -20,18 +20,27 @@
 
 
 /**
- * Class containing methods for operations with graphs
+ * Class containing methods for operations with graph prototypes.
+ *
+ * @package API
  */
 class CGraphPrototype extends CGraphGeneral {
 
 	protected $tableName = 'graphs';
 	protected $tableAlias = 'g';
+	protected $sortColumns = array('graphid', 'name', 'graphtype');
 
 	public function __construct() {
 		parent::__construct();
 
 		$this->errorMessages = array_merge($this->errorMessages, array(
-			self::ERROR_TEMPLATE_HOST_MIX => _('Graph prototype "%1$s" with templated items cannot contain items from other hosts.')
+			self::ERROR_TEMPLATE_HOST_MIX =>
+				_('Graph prototype "%1$s" with templated items cannot contain items from other hosts.'),
+			self::ERROR_MISSING_GRAPH_NAME => _('Missing "name" field for graph prototype.'),
+			self::ERROR_MISSING_GRAPH_ITEMS => _('Missing items for graph prototype "%1$s".'),
+			self::ERROR_MISSING_REQUIRED_VALUE => _('No "%1$s" given for graph prototype.'),
+			self::ERROR_TEMPLATED_ID => _('Cannot update "templateid" for graph prototype "%1$s".'),
+			self::ERROR_GRAPH_SUM => _('Cannot add more than one item with type "Graph sum" on graph prototype "%1$s".')
 		));
 	}
 
@@ -46,16 +55,10 @@ class CGraphPrototype extends CGraphGeneral {
 		$userType = self::$userData['type'];
 		$userid = self::$userData['userid'];
 
-		// allowed columns for sorting
-		$sortColumns = array('graphid', 'name', 'graphtype');
-
-		// allowed output options for [ select_* ] params
-		$subselectsAllowedOutputs = array(API_OUTPUT_REFER, API_OUTPUT_EXTEND, API_OUTPUT_CUSTOM);
-
 		$sqlParts = array(
 			'select'	=> array('graphs' => 'g.graphid'),
 			'from'		=> array('graphs' => 'graphs g'),
-			'where'		=> array('g.flags='.ZBX_FLAG_DISCOVERY_CHILD),
+			'where'		=> array('g.flags='.ZBX_FLAG_DISCOVERY_PROTOTYPE),
 			'group'		=> array(),
 			'order'		=> array(),
 			'limit'		=> null
@@ -69,7 +72,6 @@ class CGraphPrototype extends CGraphGeneral {
 			'graphids'					=> null,
 			'itemids'					=> null,
 			'discoveryids'				=> null,
-			'type'						=> null,
 			'templated'					=> null,
 			'inherited'					=> null,
 			'editable'					=> null,
@@ -98,49 +100,64 @@ class CGraphPrototype extends CGraphGeneral {
 		);
 		$options = zbx_array_merge($defOptions, $options);
 
-		if (is_array($options['output'])) {
-			unset($sqlParts['select']['graphs']);
-
-			$dbTable = DB::getSchema('graphs');
-			foreach ($options['output'] as $field) {
-				if (isset($dbTable['fields'][$field])) {
-					$sqlParts['select'][$field] = 'g.'.$field;
-				}
-			}
-			$options['output'] = API_OUTPUT_CUSTOM;
-		}
-
 		// editable + PERMISSION CHECK
 		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
-			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ_ONLY;
+			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
 
 			$userGroups = getUserGroupsByUserId($userid);
 
-			$sqlParts['where'][] = 'EXISTS ('.
-					'SELECT NULL'.
-					' FROM graphs_items gi,items i,hosts_groups hgg'.
-						' JOIN rights r'.
-							' ON r.id=hgg.groupid'.
-								' AND '.dbConditionInt('r.groupid', $userGroups).
-					' WHERE g.graphid=gi.graphid'.
-						' AND gi.itemid=i.itemid'.
-						' AND i.hostid=hgg.hostid'.
-					' GROUP BY gi.graphid'.
-					' HAVING MIN(r.permission)>='.$permission.
-					')';
+			// check permissions by graph items
+			$sqlParts['where'][] = 'NOT EXISTS ('.
+				'SELECT NULL'.
+				' FROM graphs_items gi,items i,hosts_groups hgg'.
+					' LEFT JOIN rights r'.
+						' ON r.id=hgg.groupid'.
+							' AND '.dbConditionInt('r.groupid', $userGroups).
+				' WHERE g.graphid=gi.graphid'.
+					' AND gi.itemid=i.itemid'.
+					' AND i.hostid=hgg.hostid'.
+				' GROUP BY i.hostid'.
+				' HAVING MAX(permission)<'.$permission.
+					' OR MIN(permission) IS NULL'.
+					' OR MIN(permission)='.PERM_DENY.
+				')';
+			// check permissions by Y min item
+			$sqlParts['where'][] = 'NOT EXISTS ('.
+				'SELECT NULL'.
+				' FROM items i,hosts_groups hgg'.
+					' LEFT JOIN rights r'.
+						' ON r.id=hgg.groupid'.
+							' AND '.dbConditionInt('r.groupid', $userGroups).
+				' WHERE g.ymin_type='.GRAPH_YAXIS_TYPE_ITEM_VALUE.
+					' AND g.ymin_itemid=i.itemid'.
+					' AND i.hostid=hgg.hostid'.
+				' GROUP BY i.hostid'.
+				' HAVING MAX(permission)<'.$permission.
+					' OR MIN(permission) IS NULL'.
+					' OR MIN(permission)='.PERM_DENY.
+				')';
+			// check permissions by Y max item
+			$sqlParts['where'][] = 'NOT EXISTS ('.
+				'SELECT NULL'.
+				' FROM items i,hosts_groups hgg'.
+					' LEFT JOIN rights r'.
+						' ON r.id=hgg.groupid'.
+							' AND '.dbConditionInt('r.groupid', $userGroups).
+				' WHERE g.ymax_type='.GRAPH_YAXIS_TYPE_ITEM_VALUE.
+					' AND g.ymax_itemid=i.itemid'.
+					' AND i.hostid=hgg.hostid'.
+				' GROUP BY i.hostid'.
+				' HAVING MAX(permission)<'.$permission.
+					' OR MIN(permission) IS NULL'.
+					' OR MIN(permission)='.PERM_DENY.
+				')';
 		}
 
-		// nodeids
-		$nodeids = !is_null($options['nodeids']) ? $options['nodeids'] : get_current_nodeid();
-
 		// groupids
 		if (!is_null($options['groupids'])) {
 			zbx_value2array($options['groupids']);
 
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['groupid'] = 'hg.groupid';
-			}
-
+			$sqlParts['select']['groupid'] = 'hg.groupid';
 			$sqlParts['from']['graphs_items'] = 'graphs_items gi';
 			$sqlParts['from']['items'] = 'items i';
 			$sqlParts['from']['hosts_groups'] = 'hosts_groups hg';
@@ -171,10 +188,8 @@ class CGraphPrototype extends CGraphGeneral {
 		// hostids
 		if (!is_null($options['hostids'])) {
 			zbx_value2array($options['hostids']);
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['hostid'] = 'i.hostid';
-			}
 
+			$sqlParts['select']['hostid'] = 'i.hostid';
 			$sqlParts['from']['graphs_items'] = 'graphs_items gi';
 			$sqlParts['from']['items'] = 'items i';
 			$sqlParts['where'][] = dbConditionInt('i.hostid', $options['hostids']);
@@ -196,9 +211,8 @@ class CGraphPrototype extends CGraphGeneral {
 		// itemids
 		if (!is_null($options['itemids'])) {
 			zbx_value2array($options['itemids']);
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['itemid'] = 'gi.itemid';
-			}
+
+			$sqlParts['select']['itemid'] = 'gi.itemid';
 			$sqlParts['from']['graphs_items'] = 'graphs_items gi';
 			$sqlParts['where']['gig'] = 'gi.graphid=g.graphid';
 			$sqlParts['where'][] = dbConditionInt('gi.itemid', $options['itemids']);
@@ -211,9 +225,8 @@ class CGraphPrototype extends CGraphGeneral {
 		// discoveryids
 		if (!is_null($options['discoveryids'])) {
 			zbx_value2array($options['discoveryids']);
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['itemid'] = 'id.parent_itemid';
-			}
+
+			$sqlParts['select']['itemid'] = 'id.parent_itemid';
 			$sqlParts['from']['graphs_items'] = 'graphs_items gi';
 			$sqlParts['from']['item_discovery'] = 'item_discovery id';
 			$sqlParts['where']['gig'] = 'gi.graphid=g.graphid';
@@ -225,11 +238,6 @@ class CGraphPrototype extends CGraphGeneral {
 			}
 		}
 
-		// type
-		if (!is_null($options['type'] )) {
-			$sqlParts['where'][] = 'g.type='.$options['type'];
-		}
-
 		// templated
 		if (!is_null($options['templated'])) {
 			$sqlParts['from']['graphs_items'] = 'graphs_items gi';
@@ -257,24 +265,6 @@ class CGraphPrototype extends CGraphGeneral {
 			}
 		}
 
-		// output
-		if ($options['output'] == API_OUTPUT_EXTEND) {
-			$sqlParts['select']['graphs'] = 'g.*';
-		}
-
-		// countOutput
-		if (!is_null($options['countOutput'])) {
-			$options['sortfield'] = '';
-			$sqlParts['select'] = array('count(DISTINCT g.graphid) as rowscount');
-
-			// groupCount
-			if (!is_null($options['groupCount'])) {
-				foreach ($sqlParts['group'] as $key => $fields) {
-					$sqlParts['select'][$key] = $fields;
-				}
-			}
-		}
-
 		// search
 		if (is_array($options['search'])) {
 			zbx_db_search('graphs g', $options, $sqlParts);
@@ -307,9 +297,6 @@ class CGraphPrototype extends CGraphGeneral {
 			}
 		}
 
-		// sorting
-		zbx_db_sorting($sqlParts, $options, $sortColumns, 'g');
-
 		// limit
 		if (zbx_ctype_digit($options['limit']) && $options['limit']) {
 			$sqlParts['limit'] = $options['limit'];
@@ -317,42 +304,10 @@ class CGraphPrototype extends CGraphGeneral {
 
 		$graphids = array();
 
-		$sqlParts['select'] = array_unique($sqlParts['select']);
-		$sqlParts['from'] = array_unique($sqlParts['from']);
-		$sqlParts['where'] = array_unique($sqlParts['where']);
-		$sqlParts['group'] = array_unique($sqlParts['group']);
-		$sqlParts['order'] = array_unique($sqlParts['order']);
-
-		$sqlSelect = '';
-		$sqlFrom = '';
-		$sqlWhere = '';
-		$sqlGroup = '';
-		$sqlOrder = '';
-		if (!empty($sqlParts['select'])) {
-			$sqlSelect .= implode(',', $sqlParts['select']);
-		}
-		if (!empty($sqlParts['from'])) {
-			$sqlFrom .= implode(',', $sqlParts['from']);
-		}
-		if (!empty($sqlParts['where'])) {
-			$sqlWhere .= ' AND '.implode(' AND ', $sqlParts['where']);
-		}
-		if (!empty($sqlParts['group'])) {
-			$sqlWhere .= ' GROUP BY '.implode(',', $sqlParts['group']);
-		}
-		if (!empty($sqlParts['order'])) {
-			$sqlOrder .= ' ORDER BY '.implode(',', $sqlParts['order']);
-		}
-		$sqlLimit = $sqlParts['limit'];
-
-		$sql = 'SELECT '.zbx_db_distinct($sqlParts).' '.$sqlSelect.
-				' FROM '.$sqlFrom.
-				' WHERE '.DBin_node('g.graphid', $nodeids).
-					$sqlWhere.
-					$sqlGroup.
-					$sqlOrder;
-
-		$dbRes = DBselect($sql, $sqlLimit);
+		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$sqlParts = $this->applyQueryNodeOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($graph = DBfetch($dbRes)) {
 			if (!is_null($options['countOutput'])) {
 				if (!is_null($options['groupCount'])) {
@@ -365,49 +320,29 @@ class CGraphPrototype extends CGraphGeneral {
 			else {
 				$graphids[$graph['graphid']] = $graph['graphid'];
 
-				if ($options['output'] == API_OUTPUT_SHORTEN) {
-					$result[$graph['graphid']] = array('graphid' => $graph['graphid']);
+				if (!isset($result[$graph['graphid']])) {
+					$result[$graph['graphid']]= array();
 				}
-				else {
-					if (!isset($result[$graph['graphid']])) {
-						$result[$graph['graphid']]= array();
-					}
-					if (!is_null($options['selectHosts']) && !isset($result[$graph['graphid']]['hosts'])) {
+
+				// hostids
+				if (isset($graph['hostid']) && is_null($options['selectHosts'])) {
+					if (!isset($result[$graph['graphid']]['hosts'])) {
 						$result[$graph['graphid']]['hosts'] = array();
 					}
-					if (!is_null($options['selectGraphItems']) && !isset($result[$graph['graphid']]['gitems'])) {
-						$result[$graph['graphid']]['gitems'] = array();
-					}
-					if (!is_null($options['selectTemplates']) && !isset($result[$graph['graphid']]['templates'])) {
-						$result[$graph['graphid']]['templates'] = array();
-					}
-					if (!is_null($options['selectItems']) && !isset($result[$graph['graphid']]['items'])) {
+					$result[$graph['graphid']]['hosts'][] = array('hostid' => $graph['hostid']);
+					unset($graph['hostid']);
+				}
+
+				// itemids
+				if (isset($graph['itemid']) && is_null($options['selectItems'])) {
+					if (!isset($result[$graph['graphid']]['items'])) {
 						$result[$graph['graphid']]['items'] = array();
 					}
-					if (!is_null($options['selectDiscoveryRule']) && !isset($result[$graph['graphid']]['discoveryRule'])) {
-						$result[$graph['graphid']]['discoveryRule'] = array();
-					}
-
-					// hostids
-					if (isset($graph['hostid']) && is_null($options['selectHosts'])) {
-						if (!isset($result[$graph['graphid']]['hosts'])) {
-							$result[$graph['graphid']]['hosts'] = array();
-						}
-						$result[$graph['graphid']]['hosts'][] = array('hostid' => $graph['hostid']);
-						unset($graph['hostid']);
-					}
-
-					// itemids
-					if (isset($graph['itemid']) && is_null($options['selectItems'])) {
-						if (!isset($result[$graph['graphid']]['items'])) {
-							$result[$graph['graphid']]['items'] = array();
-						}
-						$result[$graph['graphid']]['items'][] = array('itemid' => $graph['itemid']);
-						unset($graph['itemid']);
-					}
-
-					$result[$graph['graphid']] += $graph;
+					$result[$graph['graphid']]['items'][] = array('itemid' => $graph['itemid']);
+					unset($graph['itemid']);
 				}
+
+				$result[$graph['graphid']] += $graph;
 			}
 		}
 
@@ -415,136 +350,8 @@ class CGraphPrototype extends CGraphGeneral {
 			return $result;
 		}
 
-		// adding GraphItems
-		if (!is_null($options['selectGraphItems']) && str_in_array($options['selectGraphItems'], $subselectsAllowedOutputs)) {
-			$gitems = API::GraphItem()->get(array(
-				'nodeids' => $nodeids,
-				'output' => $options['selectGraphItems'],
-				'graphids' => $graphids,
-				'nopermissions' => true,
-				'preservekeys' => true
-			));
-
-			foreach ($gitems as $gitem) {
-				$ggraphs = $gitem['graphs'];
-				unset($gitem['graphs']);
-				foreach ($ggraphs as $graph) {
-					$result[$graph['graphid']]['gitems'][$gitem['gitemid']] = $gitem;
-				}
-			}
-		}
-
-		// adding Hostgroups
-		if (!is_null($options['selectGroups'])) {
-			if (is_array($options['selectGroups']) || str_in_array($options['selectGroups'], $subselectsAllowedOutputs)) {
-				$groups = API::HostGroup()->get(array(
-					'nodeids' => $nodeids,
-					'output' => $options['selectGroups'],
-					'graphids' => $graphids,
-					'nopermissions' => true,
-					'preservekeys' => true
-				));
-
-				foreach ($groups as $group) {
-					$ggraphs = $group['graphs'];
-					unset($group['graphs']);
-					foreach ($ggraphs as $graph) {
-						$result[$graph['graphid']]['groups'][] = $group;
-					}
-				}
-			}
-		}
-
-		// adding Hosts
-		if (!is_null($options['selectHosts'])) {
-			if (is_array($options['selectHosts']) || str_in_array($options['selectHosts'], $subselectsAllowedOutputs)) {
-				$hosts = API::Host()->get(array(
-					'nodeids' => $nodeids,
-					'output' => $options['selectHosts'],
-					'graphids' => $graphids,
-					'nopermissions' => true,
-					'preservekeys' => true
-				));
-				foreach ($hosts as $host) {
-					$hgraphs = $host['graphs'];
-					unset($host['graphs']);
-					foreach ($hgraphs as $graph) {
-						$result[$graph['graphid']]['hosts'][] = $host;
-					}
-				}
-			}
-		}
-
-		// adding Templates
-		if (!is_null($options['selectTemplates']) && str_in_array($options['selectTemplates'], $subselectsAllowedOutputs)) {
-			$templates = API::Template()->get(array(
-				'nodeids' => $nodeids,
-				'output' => $options['selectTemplates'],
-				'graphids' => $graphids,
-				'nopermissions' => true,
-				'preservekeys' => true
-			));
-			foreach ($templates as $template) {
-				$tgraphs = $template['graphs'];
-				unset($template['graphs']);
-				foreach ($tgraphs as $graph) {
-					$result[$graph['graphid']]['templates'][] = $template;
-				}
-			}
-		}
-
-		// adding Items
-		if (!is_null($options['selectItems']) && str_in_array($options['selectItems'], $subselectsAllowedOutputs)) {
-			$items = API::Item()->get(array(
-				'nodeids' => $nodeids,
-				'output' => $options['selectItems'],
-				'graphids' => $graphids,
-				'nopermissions' => true,
-				'preservekeys' => true,
-				'filter' => array('flags' => null)
-			));
-
-			foreach ($items as $item) {
-				$igraphs = $item['graphs'];
-				unset($item['graphs']);
-				foreach ($igraphs as $graph) {
-					$result[$graph['graphid']]['items'][] = $item;
-				}
-			}
-		}
-
-		// adding discoveryRule
-		if (!is_null($options['selectDiscoveryRule'])) {
-			$ruleids = $ruleMap = array();
-
-			$dbRules = DBselect(
-				'SELECT id.parent_itemid,gi.graphid'.
-					' FROM item_discovery id,graphs_items gi'.
-					' WHERE '.dbConditionInt('gi.graphid', $graphids).
-						' AND gi.itemid=id.itemid'
-			);
-			while ($rule = DBfetch($dbRules)) {
-				$ruleids[$rule['parent_itemid']] = $rule['parent_itemid'];
-				$ruleMap[$rule['graphid']] = $rule['parent_itemid'];
-			}
-
-			$objParams = array(
-				'nodeids' => $nodeids,
-				'itemids' => $ruleids,
-				'nopermissions' => true,
-				'preservekeys' => true
-			);
-
-			if (is_array($options['selectDiscoveryRule']) || str_in_array($options['selectDiscoveryRule'], $subselectsAllowedOutputs)) {
-				$objParams['output'] = $options['selectDiscoveryRule'];
-				$discoveryRules = API::DiscoveryRule()->get($objParams);
-
-				foreach ($result as $graphid => $graph) {
-					if (isset($ruleMap[$graphid]) && isset($discoveryRules[$ruleMap[$graphid]])) {
-						$result[$graphid]['discoveryRule'] = $discoveryRules[$ruleMap[$graphid]];
-					}
-				}
-			}
+		if ($result) {
+			$result = $this->addRelatedObjects($options, $result);
 		}
 
 		// removing keys (hash -> array)
@@ -558,7 +365,7 @@ class CGraphPrototype extends CGraphGeneral {
 	protected function inherit($graph, $hostids = null) {
 		$graphTemplates = API::Template()->get(array(
 			'itemids' => zbx_objectValues($graph['gitems'], 'itemid'),
-			'output' => API_OUTPUT_SHORTEN,
+			'output' => array('templateid'),
 			'nopermissions' => true
 		));
 
@@ -581,7 +388,6 @@ class CGraphPrototype extends CGraphGeneral {
 			'graphids' => $graph['graphid'],
 			'nopermissions' => true,
 			'filter' => array('flags' => null),
-			'selectItems' => API_OUTPUT_EXTEND,
 			'selectGraphItems' => API_OUTPUT_EXTEND,
 			'output' => API_OUTPUT_EXTEND
 		));
@@ -591,12 +397,12 @@ class CGraphPrototype extends CGraphGeneral {
 			$tmpGraph = $graph;
 			$tmpGraph['templateid'] = $graph['graphid'];
 
-			if (!$tmpGraph['gitems'] = get_same_graphitems_for_host($tmpGraph['gitems'], $chdHost['hostid'])) {
+			if (!$tmpGraph['gitems'] = getSameGraphItemsForHost($tmpGraph['gitems'], $chdHost['hostid'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" cannot inherit. No required items on "%2$s".', $tmpGraph['name'], $chdHost['host']));
 			}
 
 			if ($tmpGraph['ymax_itemid'] > 0) {
-				$ymaxItemid = get_same_graphitems_for_host(array(array('itemid' => $tmpGraph['ymax_itemid'])), $chdHost['hostid']);
+				$ymaxItemid = getSameGraphItemsForHost(array(array('itemid' => $tmpGraph['ymax_itemid'])), $chdHost['hostid']);
 				if (!$ymaxItemid) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" cannot inherit. No required items on "%2$s" (Ymax value item).', $tmpGraph['name'], $chdHost['host']));
 				}
@@ -604,9 +410,13 @@ class CGraphPrototype extends CGraphGeneral {
 				$tmpGraph['ymax_itemid'] = $ymaxItemid['itemid'];
 			}
 			if ($tmpGraph['ymin_itemid'] > 0) {
-				$yminItemid = get_same_graphitems_for_host(array(array('itemid' => $tmpGraph['ymin_itemid'])), $chdHost['hostid']);
+				$yminItemid = getSameGraphItemsForHost(array(array('itemid' => $tmpGraph['ymin_itemid'])), $chdHost['hostid']);
 				if (!$yminItemid) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" cannot inherit. No required items on "%2$s" (Ymin value item).', $tmpGraph['name'], $chdHost['host']));
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Graph "%1$s" cannot inherit. No required items on "%2$s" (Ymin value item).',
+							$tmpGraph['name'],
+							$chdHost['host']
+					));
 				}
 				$yminItemid = reset($yminItemid);
 				$tmpGraph['ymin_itemid'] = $yminItemid['itemid'];
@@ -614,7 +424,10 @@ class CGraphPrototype extends CGraphGeneral {
 
 			// check if templated graph exists
 			$chdGraphs = $this->get(array(
-				'filter' => array('templateid' => $tmpGraph['graphid'], 'flags' => array(ZBX_FLAG_DISCOVERY_CHILD, ZBX_FLAG_DISCOVERY_NORMAL)),
+				'filter' => array(
+					'templateid' => $tmpGraph['graphid'],
+					'flags' => array(ZBX_FLAG_DISCOVERY_PROTOTYPE, ZBX_FLAG_DISCOVERY_NORMAL)
+				),
 				'output' => API_OUTPUT_EXTEND,
 				'selectGraphItems' => API_OUTPUT_EXTEND,
 				'preservekeys' => true,
@@ -624,7 +437,9 @@ class CGraphPrototype extends CGraphGeneral {
 			if ($chdGraph = reset($chdGraphs)) {
 				if (zbx_strtolower($tmpGraph['name']) != zbx_strtolower($chdGraph['name'])
 						&& $this->exists(array('name' => $tmpGraph['name'], 'hostids' => $chdHost['hostid']))) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" already exists on "%2$s".', $tmpGraph['name'], $chdHost['host']));
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Graph "%1$s" already exists on "%2$s".', $tmpGraph['name'], $chdHost['host'])
+					);
 				}
 				elseif ($chdGraph['flags'] != $tmpGraph['flags']) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _('Graph with same name but other type exist.'));
@@ -744,6 +559,7 @@ class CGraphPrototype extends CGraphGeneral {
 		}
 
 		$graphids = zbx_toArray($graphids);
+		$delGraphPrototypeIds = $graphids;
 
 		$delGraphs = $this->get(array(
 			'graphids' => $graphids,
@@ -769,10 +585,11 @@ class CGraphPrototype extends CGraphGeneral {
 			$parentGraphids = array();
 			while ($dbGraph = DBfetch($dbGraphs)) {
 				$parentGraphids[] = $dbGraph['graphid'];
-				$graphids[$dbGraph['graphid']] = $dbGraph['graphid'];
+				$graphids[] = $dbGraph['graphid'];
 			}
 		} while (!empty($parentGraphids));
 
+		$graphids = array_unique($graphids);
 		$createdGraphs = array();
 
 		$dbGraphs = DBselect('SELECT gd.graphid FROM graph_discovery gd WHERE '.dbConditionInt('gd.parent_graphid', $graphids));
@@ -792,76 +609,180 @@ class CGraphPrototype extends CGraphGeneral {
 			info(_s('Graph prototype "%s" deleted.', $graph['name']));
 		}
 
-		return array('graphids' => $graphids);
+		return array('graphids' => $delGraphPrototypeIds);
+	}
+
+	protected function createReal($graph) {
+		// mark the graph as a graph prototype
+		$graph['flags'] = ZBX_FLAG_DISCOVERY_PROTOTYPE;
+
+		return parent::createReal($graph);
+	}
+
+	protected function addRelatedObjects(array $options, array $result) {
+		$result = parent::addRelatedObjects($options, $result);
+
+		$graphids = array_keys($result);
+
+		// adding Items
+		if ($options['selectItems'] !== null && $options['selectItems'] !== API_OUTPUT_COUNT) {
+			$relationMap = $this->createRelationMap($result, 'graphid', 'itemid', 'graphs_items');
+			$items = API::Item()->get(array(
+				'nodeids' => $options['nodeids'],
+				'output' => $options['selectItems'],
+				'itemids' => $relationMap->getRelatedIds(),
+				'webitems' => true,
+				'nopermissions' => true,
+				'preservekeys' => true,
+				'filter' => array('flags' => null)
+			));
+			$result = $relationMap->mapMany($result, $items, 'items');
+		}
+
+		// adding discoveryRule
+		if (!is_null($options['selectDiscoveryRule'])) {
+			$dbRules = DBselect(
+				'SELECT id.parent_itemid,gi.graphid'.
+					' FROM item_discovery id,graphs_items gi'.
+					' WHERE '.dbConditionInt('gi.graphid', $graphids).
+						' AND gi.itemid=id.itemid'
+			);
+			$relationMap = new CRelationMap();
+			while ($relation = DBfetch($dbRules)) {
+				$relationMap->addRelation($relation['graphid'], $relation['parent_itemid']);
+			}
+
+			$discoveryRules = API::DiscoveryRule()->get(array(
+				'output' => $options['selectDiscoveryRule'],
+				'nodeids' => $options['nodeids'],
+				'itemids' => $relationMap->getRelatedIds(),
+				'nopermissions' => true,
+				'preservekeys' => true
+			));
+			$result = $relationMap->mapOne($result, $discoveryRules, 'discoveryRule');
+		}
+
+		return $result;
 	}
 
 	/**
-	 * Check graph data
+	 * Validate graph prototype specific data on Create method.
+	 * Get allowed item ID's, check permissions, check if items have at least one prototype, do all general validation,
+	 * and check for numeric item types.
 	 *
 	 * @param array $graphs
-	 * @param boolean $update
 	 *
 	 * @return void
 	 */
-	protected function checkInput($graphs, $update = false) {
-		$itemids = array();
-		foreach ($graphs as $graph) {
-			// no items
-			if (!isset($graph['gitems']) || !is_array($graph['gitems']) || empty($graph['gitems'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Missing items for graph "%1$s".', $graph['name']));
-			}
-
-			$fields = array('itemid' => null);
-			foreach ($graph['gitems'] as $gitem) {
-				if (!check_db_fields($fields, $gitem)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Missing "itemid" field for item.'));
-				}
-
-				// assigning with key preserves unique itemids
-				$itemids[$gitem['itemid']] = $gitem['itemid'];
-			}
-		}
+	protected function validateCreate(array $graphs) {
+		$itemIds = $this->validateItemsCreate($graphs);
 
 		$allowedItems = API::Item()->get(array(
 			'nodeids' => get_current_nodeid(true),
-			'itemids' => $itemids,
+			'itemids' => $itemIds,
 			'webitems' => true,
 			'editable' => true,
-			'output' => API_OUTPUT_EXTEND,
+			'output' => array('name', 'value_type', 'flags'),
 			'preservekeys' => true,
 			'filter' => array('flags' => null)
 		));
 
-		foreach ($itemids as $itemid) {
+		foreach ($itemIds as $itemid) {
 			if (!isset($allowedItems[$itemid])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('No permissions to referred object or it does not exist!'));
 			}
 		}
 
-
-		parent::checkInput($graphs, $update);
-
 		foreach ($graphs as $graph) {
-			// check if the graph has at least one prototype
 			$hasPrototype = false;
-			foreach ($graph['gitems'] as $gitem) {
-				// $allowedItems used because it is possible to make API call without full item data
-				if ($allowedItems[$gitem['itemid']]['flags'] == ZBX_FLAG_DISCOVERY_CHILD) {
-					$hasPrototype = true;
-					break;
+			if ($graph['gitems']) {
+				// check if the graph has at least one prototype
+				foreach ($graph['gitems'] as $gitem) {
+					// $allowedItems used because it is possible to make API call without full item data
+					if ($allowedItems[$gitem['itemid']]['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+						$hasPrototype = true;
+						break;
+					}
 				}
 			}
-			if (!$hasPrototype) {
+
+			if (!$graph['gitems'] || !$hasPrototype) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Graph prototype must have at least one prototype.'));
 			}
 		}
+
+		parent::validateCreate($graphs);
+
+		$allowedValueTypes = array(ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64);
+
+		foreach ($allowedItems as $item) {
+			if (!in_array($item['value_type'], $allowedValueTypes)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Cannot add a non-numeric item "%1$s" to graph prototype "%2$s".', $item['name'], $graph['name'])
+				);
+			}
+		}
 	}
 
-	protected function createReal($graph) {
-		// mark the graph as a graph prototype
-		$graph['flags'] = ZBX_FLAG_DISCOVERY_CHILD;
+	/**
+	 * Validate graph prototype specific data on Update method.
+	 * Get allowed item ID's, check permissions, check if items have at least one prototype, do all general validation,
+	 * and check for numeric item types.
+	 *
+	 * @param array $graphs
+	 * @param array $dbGraphs
+	 *
+	 * @return void
+	 */
+	protected function validateUpdate(array $graphs, array $dbGraphs) {
+		$itemIds = $this->validateItemsUpdate($graphs);
 
-		return parent::createReal($graph);
+		$allowedItems = API::Item()->get(array(
+			'nodeids' => get_current_nodeid(true),
+			'itemids' => $itemIds,
+			'webitems' => true,
+			'editable' => true,
+			'output' => array('name', 'value_type', 'flags'),
+			'preservekeys' => true,
+			'filter' => array('flags' => null)
+		));
+
+		foreach ($itemIds as $itemid) {
+			if (!isset($allowedItems[$itemid])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('No permissions to referred object or it does not exist!'));
+			}
+		}
+
+		foreach ($graphs as $graph) {
+			if (isset($graph['gitems'])) {
+				$hasPrototype = false;
+				if ($graph['gitems']) {
+					// check if the graph has at least one prototype
+					foreach ($graph['gitems'] as $gitem) {
+						// $allowedItems used because it is possible to make API call without full item data
+						if ($allowedItems[$gitem['itemid']]['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+							$hasPrototype = true;
+							break;
+						}
+					}
+				}
+
+				if (!$graph['gitems'] || !$hasPrototype) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _('Graph prototype must have at least one prototype.'));
+				}
+			}
+		}
+
+		parent::validateUpdate($graphs, $dbGraphs);
+
+		$allowedValueTypes = array(ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64);
+
+		foreach ($allowedItems as $item) {
+			if (!in_array($item['value_type'], $allowedValueTypes)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Cannot add a non-numeric item "%1$s" to graph prototype "%2$s".', $item['name'], $graph['name'])
+				);
+			}
+		}
 	}
-
 }
