@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -20,12 +20,10 @@
 
 
 /**
- * File containing CScreenItem class for API.
+ * Class containing methods for operations with screen items.
+ *
  * @package API
  */
-/**
- * Class containing methods for operations with ScreenItems
- */
 class CScreenItem extends CZBXAPI {
 
 	protected $tableName = 'screens_items';
@@ -104,10 +102,10 @@ class CScreenItem extends CZBXAPI {
 			// normal select query
 			else {
 				if ($options['preservekeys'] !== null) {
-					$result[$row['screenitemid']] = $this->unsetExtraFields($this->tableName(), $row, $options['output']);
+					$result[$row['screenitemid']] = $row;
 				}
 				else {
-					$result[] = $this->unsetExtraFields($this->tableName(), $row, $options['output']);
+					$result[] = $row;
 				}
 			}
 		}
@@ -474,13 +472,31 @@ class CScreenItem extends CZBXAPI {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _('No URL provided for screen element.'));
 				}
 			}
+
+			// check fields specific to each resource type
+			// check "Show lines" field
+			switch ($screenItem['resourcetype']) {
+				case SCREEN_RESOURCE_ACTIONS:
+				case SCREEN_RESOURCE_EVENTS:
+				case SCREEN_RESOURCE_HOSTGROUP_TRIGGERS:
+				case SCREEN_RESOURCE_HOST_TRIGGERS:
+				case SCREEN_RESOURCE_PLAIN_TEXT:
+					if ($screenItem['elements'] < 1 || $screenItem['elements'] > 100) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('Incorrect value "%1$s" for "%2$s" field: must be between %3$s and %4$s.',
+								$screenItem['elements'], 'elements', 1, 100
+							)
+						);
+					}
+					break;
+			}
 		}
 
 		// check host groups
 		if (!empty($hostgroups)) {
 			$result = API::HostGroup()->get(array(
 				'groupids' => $hostgroups,
-				'output' => API_OUTPUT_SHORTEN,
+				'output' => array('groupid'),
 				'preservekeys' => true
 			));
 			foreach ($hostgroups as $id) {
@@ -494,7 +510,7 @@ class CScreenItem extends CZBXAPI {
 		if ($hosts) {
 			$result = API::Host()->get(array(
 				'hostids' => $hosts,
-				'output' => API_OUTPUT_SHORTEN,
+				'output' => array('hostid'),
 				'preservekeys' => true
 			));
 			foreach ($hosts as $id) {
@@ -508,7 +524,7 @@ class CScreenItem extends CZBXAPI {
 		if ($graphs) {
 			$result = API::Graph()->get(array(
 				'graphids' => $graphs,
-				'output' => API_OUTPUT_SHORTEN,
+				'output' => array('graphid'),
 				'preservekeys' => true
 			));
 			foreach ($graphs as $id) {
@@ -522,7 +538,7 @@ class CScreenItem extends CZBXAPI {
 		if ($items) {
 			$result = API::Item()->get(array(
 				'itemids' => $items,
-				'output' => API_OUTPUT_SHORTEN,
+				'output' => array('itemid'),
 				'preservekeys' => true,
 				'webitems' => true
 			));
@@ -537,7 +553,7 @@ class CScreenItem extends CZBXAPI {
 		if ($maps) {
 			$result = API::Map()->get(array(
 				'sysmapids' => $maps,
-				'output' => API_OUTPUT_SHORTEN,
+				'output' => array('sysmapid'),
 				'preservekeys' => true
 			));
 			foreach ($maps as $id) {
@@ -551,7 +567,7 @@ class CScreenItem extends CZBXAPI {
 		if ($screens) {
 			$result = API::Screen()->get(array(
 				'screenids' => $screens,
-				'output' => API_OUTPUT_SHORTEN,
+				'output' => array('screenid'),
 				'preservekeys' => true
 			));
 			if (empty($result)) {
@@ -581,10 +597,14 @@ class CScreenItem extends CZBXAPI {
 	 */
 	protected function checkSpans(array $screenItem) {
 		if (zbx_empty($screenItem['rowspan']) || !zbx_is_int($screenItem['rowspan'])) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Incorrect row span provided for screen element.'));
+			self::exception(ZBX_API_ERROR_PERMISSIONS,
+				_s('Incorrect row span provided for screen element located at X - %1$s and Y - %2$s.', $screenItem['x'], $screenItem['y'])
+			);
 		}
 		if (zbx_empty($screenItem['colspan']) || !zbx_is_int($screenItem['colspan'])) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Incorrect column span provided for screen element.'));
+			self::exception(ZBX_API_ERROR_PERMISSIONS,
+				_s('Incorrect column span provided for screen element located at X - %1$s and Y - %2$s.', $screenItem['x'], $screenItem['y'])
+			);
 		}
 	}
 
@@ -600,10 +620,14 @@ class CScreenItem extends CZBXAPI {
 	 */
 	protected function checkSpansInBounds(array $screenItem, array $screen) {
 		if ($screenItem['rowspan'] > $screen['vsize'] - $screenItem['y']) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Screen elements row span is too big.'));
+			self::exception(ZBX_API_ERROR_PERMISSIONS,
+				_s('Row span of screen element located at X - %1$s and Y - %2$s is too big.', $screenItem['x'], $screenItem['y'])
+			);
 		}
 		if ($screenItem['colspan'] > $screen['hsize'] - $screenItem['x']) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Screen elements column span is too big.'));
+			self::exception(ZBX_API_ERROR_PERMISSIONS,
+				_s('Column span of screen element located at X - %1$s and Y - %2$s is too big.', $screenItem['x'], $screenItem['y'])
+			);
 		}
 	}
 
@@ -623,7 +647,7 @@ class CScreenItem extends CZBXAPI {
 		if ($options['screenids'] !== null) {
 			zbx_value2array($options['screenids']);
 			$sqlParts = $this->addQuerySelect($this->fieldId('screenid'), $sqlParts);
-			$sqlParts['where'][] = DBcondition($this->fieldId('screenid'), $options['screenids']);
+			$sqlParts['where'][] = dbConditionInt($this->fieldId('screenid'), $options['screenids']);
 		}
 
 		return $sqlParts;
