@@ -19,13 +19,19 @@
 **/
 
 
+/**
+ * Class containing methods for operations with trigger prototypes.
+ *
+ * @package API
+ */
 class CTriggerPrototype extends CTriggerGeneral {
 
 	protected $tableName = 'triggers';
 	protected $tableAlias = 't';
+	protected $sortColumns = array('triggerid', 'description', 'status', 'priority');
 
 	/**
-	 * Get TriggerPrototypes data
+	 * Get TriggerPrototypes data.
 	 *
 	 * @param array $options
 	 * @param array $options['itemids']
@@ -39,6 +45,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 	 * @param array $options['pattern']
 	 * @param array $options['limit']
 	 * @param array $options['order']
+	 *
 	 * @return array|int item data as array or false if error
 	 */
 	public function get(array $options = array()) {
@@ -46,16 +53,10 @@ class CTriggerPrototype extends CTriggerGeneral {
 		$userType = self::$userData['type'];
 		$userid = self::$userData['userid'];
 
-		// allowed columns for sorting
-		$sortColumns = array('triggerid', 'description', 'status', 'priority');
-
-		// allowed output options for [ select_* ] params
-		$subselectsAllowedOutputs = array(API_OUTPUT_REFER, API_OUTPUT_EXTEND);
-
 		$sqlParts = array(
 			'select'	=> array('triggers' => 't.triggerid'),
 			'from'		=> array('t' => 'triggers t'),
-			'where'		=> array('t.flags='.ZBX_FLAG_DISCOVERY_CHILD),
+			'where'		=> array('t.flags='.ZBX_FLAG_DISCOVERY_PROTOTYPE),
 			'group'		=> array(),
 			'order'		=> array(),
 			'limit'		=> null
@@ -89,7 +90,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 			'excludeSearch'					=> null,
 			'searchWildcardsEnabled'		=> null,
 			// output
-			'expandExpression'              => null,
+			'expandExpression'				=> null,
 			'expandData'					=> null,
 			'output'						=> API_OUTPUT_REFER,
 			'selectGroups'					=> null,
@@ -107,56 +108,33 @@ class CTriggerPrototype extends CTriggerGeneral {
 		);
 		$options = zbx_array_merge($defOptions, $options);
 
-		if (is_array($options['output'])) {
-			unset($sqlParts['select']['triggers']);
-
-			$dbTable = DB::getSchema('triggers');
-			$sqlParts['select']['triggerid'] = 't.triggerid';
-			foreach ($options['output'] as $field) {
-				if (isset($dbTable['fields'][$field])) {
-					$sqlParts['select'][$field] = 't.'.$field;
-				}
-			}
-
-			// ignore the "expandExpression" parameter if the expression is not requested
-			if ($options['expandExpression'] !== null && !str_in_array('expression', $options['output'])) {
-				$options['expandExpression'] = null;
-			}
-
-			$options['output'] = API_OUTPUT_CUSTOM;
-		}
-
 		// editable + permission check
 		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
-			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ_ONLY;
+			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
 
 			$userGroups = getUserGroupsByUserId($userid);
 
-			$sqlParts['where'][] = 'EXISTS ('.
-					'SELECT NULL'.
-					' FROM functions f,items i,hosts_groups hgg'.
-						' JOIN rights r'.
-							' ON r.id=hgg.groupid'.
-								' AND '.dbConditionInt('r.groupid', $userGroups).
-					' WHERE t.triggerid=f.triggerid'.
-						' AND f.itemid=i.itemid'.
-						' AND i.hostid=hgg.hostid'.
-					' GROUP BY f.triggerid'.
-					' HAVING MIN(r.permission)>='.$permission.
-					')';
+			$sqlParts['where'][] = 'NOT EXISTS ('.
+				'SELECT NULL'.
+				' FROM functions f,items i,hosts_groups hgg'.
+					' LEFT JOIN rights r'.
+						' ON r.id=hgg.groupid'.
+							' AND '.dbConditionInt('r.groupid', $userGroups).
+				' WHERE t.triggerid=f.triggerid'.
+					' AND f.itemid=i.itemid'.
+					' AND i.hostid=hgg.hostid'.
+				' GROUP BY i.hostid'.
+				' HAVING MAX(permission)<'.$permission.
+					' OR MIN(permission) IS NULL'.
+					' OR MIN(permission)='.PERM_DENY.
+			')';
 		}
 
-		// nodeids
-		$nodeids = !is_null($options['nodeids']) ? $options['nodeids'] : get_current_nodeid();
-
 		// groupids
 		if (!is_null($options['groupids'])) {
 			zbx_value2array($options['groupids']);
 
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['groupid'] = 'hg.groupid';
-			}
-
+			$sqlParts['select']['groupid'] = 'hg.groupid';
 			$sqlParts['from']['functions'] = 'functions f';
 			$sqlParts['from']['items'] = 'items i';
 			$sqlParts['from']['hosts_groups'] = 'hosts_groups hg';
@@ -187,10 +165,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 		if (!is_null($options['hostids'])) {
 			zbx_value2array($options['hostids']);
 
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['hostid'] = 'i.hostid';
-			}
-
+			$sqlParts['select']['hostid'] = 'i.hostid';
 			$sqlParts['from']['functions'] = 'functions f';
 			$sqlParts['from']['items'] = 'items i';
 			$sqlParts['where']['hostid'] = dbConditionInt('i.hostid', $options['hostids']);
@@ -213,10 +188,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 		if (!is_null($options['itemids'])) {
 			zbx_value2array($options['itemids']);
 
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['itemid'] = 'f.itemid';
-			}
-
+			$sqlParts['select']['itemid'] = 'f.itemid';
 			$sqlParts['from']['functions'] = 'functions f';
 			$sqlParts['where']['itemid'] = dbConditionInt('f.itemid', $options['itemids']);
 			$sqlParts['where']['ft'] = 'f.triggerid=t.triggerid';
@@ -230,10 +202,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 		if (!is_null($options['applicationids'])) {
 			zbx_value2array($options['applicationids']);
 
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['applicationid'] = 'a.applicationid';
-			}
-
+			$sqlParts['select']['applicationid'] = 'a.applicationid';
 			$sqlParts['from']['functions'] = 'functions f';
 			$sqlParts['from']['items'] = 'items i';
 			$sqlParts['from']['applications'] = 'applications a';
@@ -247,9 +216,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 		if (!is_null($options['discoveryids'])) {
 			zbx_value2array($options['discoveryids']);
 
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['itemid'] = 'id.parent_itemid';
-			}
+			$sqlParts['select']['itemid'] = 'id.parent_itemid';
 			$sqlParts['from']['functions'] = 'functions f';
 			$sqlParts['from']['item_discovery'] = 'item_discovery id';
 			$sqlParts['where']['fid'] = 'f.itemid=id.itemid';
@@ -272,14 +239,14 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		// monitored
 		if (!is_null($options['monitored'])) {
-			$sqlParts['where']['monitored'] = ''.
+			$sqlParts['where']['monitored'] =
 				' NOT EXISTS ('.
 					' SELECT NULL'.
 					' FROM functions ff'.
 					' WHERE ff.triggerid=t.triggerid'.
 						' AND EXISTS ('.
 								' SELECT NULL'.
-								' FROM items ii, hosts hh'.
+								' FROM items ii,hosts hh'.
 								' WHERE ff.itemid=ii.itemid'.
 									' AND hh.hostid=ii.hostid'.
 									' AND ('.
@@ -293,14 +260,14 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		// active
 		if (!is_null($options['active'])) {
-			$sqlParts['where']['active'] = ''.
+			$sqlParts['where']['active'] =
 				' NOT EXISTS ('.
 					' SELECT NULL'.
 					' FROM functions ff'.
 					' WHERE ff.triggerid=t.triggerid'.
 						' AND EXISTS ('.
 							' SELECT NULL'.
-							' FROM items ii, hosts hh'.
+							' FROM items ii,hosts hh'.
 							' WHERE ff.itemid=ii.itemid'.
 								' AND hh.hostid=ii.hostid'.
 								' AND  hh.status<>'.HOST_STATUS_MONITORED.
@@ -311,14 +278,14 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		// maintenance
 		if (!is_null($options['maintenance'])) {
-			$sqlParts['where'][] = (($options['maintenance'] == 0) ? ' NOT ':'').
+			$sqlParts['where'][] = (($options['maintenance'] == 0) ? ' NOT ' : '').
 				' EXISTS ('.
 					' SELECT NULL'.
 					' FROM functions ff'.
 					' WHERE ff.triggerid=t.triggerid'.
 						' AND EXISTS ('.
 								' SELECT NULL'.
-								' FROM items ii, hosts hh'.
+								' FROM items ii,hosts hh'.
 								' WHERE ff.itemid=ii.itemid'.
 									' AND hh.hostid=ii.hostid'.
 									' AND hh.maintenance_status=1'.
@@ -390,10 +357,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		// group
 		if (!is_null($options['group'])) {
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['name'] = 'g.name';
-			}
-
+			$sqlParts['select']['name'] = 'g.name';
 			$sqlParts['from']['functions'] = 'functions f';
 			$sqlParts['from']['items'] = 'items i';
 			$sqlParts['from']['hosts_groups'] = 'hosts_groups hg';
@@ -401,16 +365,13 @@ class CTriggerPrototype extends CTriggerGeneral {
 			$sqlParts['where']['ft'] = 'f.triggerid=t.triggerid';
 			$sqlParts['where']['fi'] = 'f.itemid=i.itemid';
 			$sqlParts['where']['hgi'] = 'hg.hostid=i.hostid';
-			$sqlParts['where']['ghg'] = 'g.groupid = hg.groupid';
+			$sqlParts['where']['ghg'] = 'g.groupid=hg.groupid';
 			$sqlParts['where']['group'] = ' g.name='.zbx_dbstr($options['group']);
 		}
 
 		// host
 		if (!is_null($options['host'])) {
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['host'] = 'h.host';
-			}
-
+			$sqlParts['select']['host'] = 'h.host';
 			$sqlParts['from']['functions'] = 'functions f';
 			$sqlParts['from']['items'] = 'items i';
 			$sqlParts['from']['hosts'] = 'hosts h';
@@ -423,85 +384,19 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		// min_severity
 		if (!is_null($options['min_severity'])) {
-			$sqlParts['where'][] = 't.priority>='.$options['min_severity'];
+			$sqlParts['where'][] = 't.priority>='.zbx_dbstr($options['min_severity']);
 		}
 
-		// output
-		if ($options['output'] == API_OUTPUT_EXTEND) {
-			$sqlParts['select']['triggers'] = 't.*';
-		}
-
-		// expandData
-		if (!is_null($options['expandData'])) {
-			$sqlParts['select']['host'] = 'h.host';
-			$sqlParts['select']['hostid'] = 'h.hostid';
-			$sqlParts['from']['functions'] = 'functions f';
-			$sqlParts['from']['items'] = 'items i';
-			$sqlParts['from']['hosts'] = 'hosts h';
-			$sqlParts['where']['ft'] = 'f.triggerid=t.triggerid';
-			$sqlParts['where']['fi'] = 'f.itemid=i.itemid';
-			$sqlParts['where']['hi'] = 'h.hostid=i.hostid';
-		}
-
-		// countOutput
-		if (!is_null($options['countOutput'])) {
-			$options['sortfield'] = '';
-			$sqlParts['select'] = array('COUNT(DISTINCT t.triggerid) as rowscount');
-
-			// groupCount
-			if (!is_null($options['groupCount'])) {
-				foreach ($sqlParts['group'] as $key => $fields) {
-					$sqlParts['select'][$key] = $fields;
-				}
-			}
-		}
-
-		// sorting
-		zbx_db_sorting($sqlParts, $options, $sortColumns, 't');
-
 		// limit
 		if (zbx_ctype_digit($options['limit']) && $options['limit']) {
 			$sqlParts['limit'] = $options['limit'];
 		}
 
 		$triggerids = array();
-
-		$sqlParts['select'] = array_unique($sqlParts['select']);
-		$sqlParts['from'] = array_unique($sqlParts['from']);
-		$sqlParts['where'] = array_unique($sqlParts['where']);
-		$sqlParts['group'] = array_unique($sqlParts['group']);
-		$sqlParts['order'] = array_unique($sqlParts['order']);
-
-		$sqlSelect = '';
-		$sqlFrom = '';
-		$sqlWhere = '';
-		$sqlGroup = '';
-		$sqlOrder = '';
-		if (!empty($sqlParts['select'])) {
-			$sqlSelect .= implode(',', $sqlParts['select']);
-		}
-		if (!empty($sqlParts['from'])) {
-			$sqlFrom .= implode(',', $sqlParts['from']);
-		}
-		if (!empty($sqlParts['where'])) {
-			$sqlWhere .= ' AND '.implode(' AND ', $sqlParts['where']);
-		}
-		if (!empty($sqlParts['group'])) {
-			$sqlWhere .= ' GROUP BY '.implode(',', $sqlParts['group']);
-		}
-		if (!empty($sqlParts['order'])) {
-			$sqlOrder .= ' ORDER BY '.implode(',', $sqlParts['order']);
-		}
-		$sqlLimit = $sqlParts['limit'];
-
-		$sql = 'SELECT '.zbx_db_distinct($sqlParts).' '.$sqlSelect.
-				' FROM '.$sqlFrom.
-				' WHERE '.DBin_node('t.triggerid', $nodeids).
-					$sqlWhere.
-					$sqlGroup.
-					$sqlOrder;
-
-		$dbRes = DBselect($sql, $sqlLimit);
+		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$sqlParts = $this->applyQueryNodeOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($trigger = DBfetch($dbRes)) {
 			if (!is_null($options['countOutput'])) {
 				if (!is_null($options['groupCount'])) {
@@ -514,66 +409,49 @@ class CTriggerPrototype extends CTriggerGeneral {
 			else {
 				$triggerids[$trigger['triggerid']] = $trigger['triggerid'];
 
-				if ($options['output'] == API_OUTPUT_SHORTEN) {
-					$result[$trigger['triggerid']] = array('triggerid' => $trigger['triggerid']);
+				if (!isset($result[$trigger['triggerid']])) {
+					$result[$trigger['triggerid']] = array();
 				}
-				else {
-					if (!isset($result[$trigger['triggerid']])) {
-						$result[$trigger['triggerid']] = array();
+
+				// groups
+				if (isset($trigger['groupid']) && is_null($options['selectGroups'])) {
+					if (!isset($result[$trigger['triggerid']]['groups'])) {
+						$result[$trigger['triggerid']]['groups'] = array();
 					}
-					if (!is_null($options['selectHosts']) && !isset($result[$trigger['triggerid']]['hosts'])) {
+
+					$result[$trigger['triggerid']]['groups'][] = array('groupid' => $trigger['groupid']);
+					unset($trigger['groupid']);
+				}
+
+				// hostids
+				if (isset($trigger['hostid']) && is_null($options['selectHosts'])) {
+					if (!isset($result[$trigger['triggerid']]['hosts'])) {
 						$result[$trigger['triggerid']]['hosts'] = array();
 					}
-					if (!is_null($options['selectItems']) && !isset($result[$trigger['triggerid']]['items'])) {
+
+					$result[$trigger['triggerid']]['hosts'][] = array('hostid' => $trigger['hostid']);
+
+					if (is_null($options['expandData'])) {
+						unset($trigger['hostid']);
+					}
+				}
+
+				// itemids
+				if (isset($trigger['itemid']) && is_null($options['selectItems'])) {
+					if (!isset($result[$trigger['triggerid']]['items'])) {
 						$result[$trigger['triggerid']]['items'] = array();
 					}
-					if (!is_null($options['selectFunctions']) && !isset($result[$trigger['triggerid']]['functions'])) {
-						$result[$trigger['triggerid']]['functions'] = array();
-					}
-					if (!is_null($options['selectDiscoveryRule']) && !isset($result[$trigger['triggerid']]['discoveryRule'])) {
-						$result[$trigger['triggerid']]['discoveryRule'] = array();
-					}
 
-					// groups
-					if (isset($trigger['groupid']) && is_null($options['selectGroups'])) {
-						if (!isset($result[$trigger['triggerid']]['groups'])) {
-							$result[$trigger['triggerid']]['groups'] = array();
-						}
-
-						$result[$trigger['triggerid']]['groups'][] = array('groupid' => $trigger['groupid']);
-						unset($trigger['groupid']);
-					}
-
-					// hostids
-					if (isset($trigger['hostid']) && is_null($options['selectHosts'])) {
-						if (!isset($result[$trigger['triggerid']]['hosts'])) {
-							$result[$trigger['triggerid']]['hosts'] = array();
-						}
-
-						$result[$trigger['triggerid']]['hosts'][] = array('hostid' => $trigger['hostid']);
-
-						if (is_null($options['expandData'])) {
-							unset($trigger['hostid']);
-						}
-					}
-
-					// itemids
-					if (isset($trigger['itemid']) && is_null($options['selectItems'])) {
-						if (!isset($result[$trigger['triggerid']]['items'])) {
-							$result[$trigger['triggerid']]['items'] = array();
-						}
-
-						$result[$trigger['triggerid']]['items'][] = array('itemid' => $trigger['itemid']);
-						unset($trigger['itemid']);
-					}
-
-					// expand expression
-					if ($options['expandExpression'] !== null && $trigger['expression']) {
-						$trigger['expression'] = explode_exp($trigger['expression'], false, true);
-					}
-
-					$result[$trigger['triggerid']] += $trigger;
+					$result[$trigger['triggerid']]['items'][] = array('itemid' => $trigger['itemid']);
+					unset($trigger['itemid']);
 				}
+
+				// expand expression
+				if ($options['expandExpression'] !== null && isset($trigger['expression'])) {
+					$trigger['expression'] = explode_exp($trigger['expression'], false, true);
+				}
+
+				$result[$trigger['triggerid']] += $trigger;
 			}
 		}
 
@@ -581,146 +459,8 @@ class CTriggerPrototype extends CTriggerGeneral {
 			return $result;
 		}
 
-		/*
-		 * Adding objects
-		 */
-		// adding groups
-		if (!is_null($options['selectGroups']) && str_in_array($options['selectGroups'], $subselectsAllowedOutputs)) {
-			$groups = API::HostGroup()->get(array(
-				'nodeids' => $nodeids,
-				'output' => $options['selectGroups'],
-				'triggerids' => $triggerids,
-				'preservekeys' => true
-			));
-			foreach ($groups as $group) {
-				$gtriggers = $group['triggers'];
-				unset($group['triggers']);
-
-				foreach ($gtriggers as $trigger) {
-					$result[$trigger['triggerid']]['groups'][] = $group;
-				}
-			}
-		}
-
-		// adding hosts
-		if (!is_null($options['selectHosts'])) {
-			$objParams = array(
-				'nodeids' => $nodeids,
-				'triggerids' => $triggerids,
-				'templated_hosts' => 1,
-				'nopermissions' => true,
-				'preservekeys' => true
-			);
-
-			if (is_array($options['selectHosts']) || str_in_array($options['selectHosts'], $subselectsAllowedOutputs)) {
-				$objParams['output'] = $options['selectHosts'];
-				$hosts = API::Host()->get($objParams);
-
-				if (!is_null($options['limitSelects'])) {
-					order_result($hosts, 'host');
-				}
-				foreach ($hosts as $hostid => $host) {
-					unset($hosts[$hostid]['triggers']);
-
-					$count = array();
-					foreach ($host['triggers'] as $trigger) {
-						if (!is_null($options['limitSelects'])) {
-							if (!isset($count[$trigger['triggerid']])) {
-								$count[$trigger['triggerid']] = 0;
-							}
-							$count[$trigger['triggerid']]++;
-
-							if ($count[$trigger['triggerid']] > $options['limitSelects']) {
-								continue;
-							}
-						}
-
-						$result[$trigger['triggerid']]['hosts'][] = &$hosts[$hostid];
-					}
-				}
-			}
-			elseif (API_OUTPUT_COUNT == $options['selectHosts']) {
-				$objParams['countOutput'] = 1;
-				$objParams['groupCount'] = 1;
-
-				$hosts = API::Host()->get($objParams);
-				$hosts = zbx_toHash($hosts, 'hostid');
-				foreach ($result as $triggerid => $trigger) {
-					$result[$triggerid]['hosts'] = isset($hosts[$triggerid]) ? $hosts[$triggerid]['rowscount'] : 0;
-				}
-			}
-		}
-
-		// adding functions
-		if (!is_null($options['selectFunctions']) && str_in_array($options['selectFunctions'], $subselectsAllowedOutputs)) {
-			$sqlSelect = ($options['selectFunctions'] == API_OUTPUT_EXTEND) ? 'f.*' : 'f.functionid, f.triggerid';
-
-			$res = DBselect(
-				'SELECT '.$sqlSelect.
-				' FROM functions f'.
-				' WHERE '.dbConditionInt('f.triggerid', $triggerids)
-			);
-			while ($function = DBfetch($res)) {
-				$triggerid = $function['triggerid'];
-				unset($function['triggerid']);
-
-				$result[$triggerid]['functions'][] = $function;
-			}
-		}
-
-		// adding items
-		if (!is_null($options['selectItems']) && (is_array($options['selectItems']) || str_in_array($options['selectItems'], $subselectsAllowedOutputs))) {
-			$items = API::Item()->get(array(
-				'nodeids' => $nodeids,
-				'output' => $options['selectItems'],
-				'triggerids' => $triggerids,
-				'webitems' => true,
-				'nopermissions' => true,
-				'preservekeys' => true,
-				'filter' => array('flags' => null)
-			));
-
-			foreach ($items as $item) {
-				$itriggers = $item['triggers'];
-				unset($item['triggers']);
-				foreach ($itriggers as $trigger) {
-					$result[$trigger['triggerid']]['items'][] = $item;
-				}
-			}
-		}
-
-		// adding discoveryrule
-		if (!is_null($options['selectDiscoveryRule'])) {
-			$ruleids = $ruleMap = array();
-
-			$dbRules = DBselect(
-				'SELECT id.parent_itemid,f.triggerid'.
-				' FROM item_discovery id,functions f'.
-				' WHERE '.dbConditionInt('f.triggerid', $triggerids).
-					' AND f.itemid=id.itemid'
-			);
-			while ($rule = DBfetch($dbRules)) {
-				$ruleids[$rule['parent_itemid']] = $rule['parent_itemid'];
-				$ruleMap[$rule['triggerid']] = $rule['parent_itemid'];
-			}
-
-			$objParams = array(
-				'nodeids' => $nodeids,
-				'itemids' => $ruleids,
-				'nopermissions' => true,
-				'preservekeys' => true
-			);
-
-			if (is_array($options['selectDiscoveryRule']) || str_in_array($options['selectDiscoveryRule'], $subselectsAllowedOutputs)) {
-				$objParams['output'] = $options['selectDiscoveryRule'];
-				$discoveryRules = API::DiscoveryRule()->get($objParams);
-
-				foreach ($result as $triggerid => $trigger) {
-					if (isset($ruleMap[$triggerid]) && isset($discoveryRules[$ruleMap[$triggerid]])) {
-						$result[$triggerid]['discoveryRule'] = $discoveryRules[$ruleMap[$triggerid]];
-					}
-				}
-			}
+		if ($result) {
+			$result = $this->addRelatedObjects($options, $result);
 		}
 
 		// removing keys (hash -> array)
@@ -732,23 +472,30 @@ class CTriggerPrototype extends CTriggerGeneral {
 	}
 
 	/**
-	 * Add triggers
+	 * Create triggers.
 	 *
-	 * Trigger params: expression, description, type, priority, status, comments, url, templateid
+	 * @param array  $triggers
+	 * @param string $triggers['expression']
+	 * @param string $triggers['description']
+	 * @param int    $triggers['type']
+	 * @param int    $triggers['priority']
+	 * @param int    $triggers['status']
+	 * @param string $triggers['comments']
+	 * @param string $triggers['url']
+	 * @param string $triggers['flags']
+	 * @param int    $triggers['templateid']
 	 *
-	 * @param array $triggers
 	 * @return boolean
 	 */
-	public function create($triggers) {
+	public function create(array $triggers) {
 		$triggers = zbx_toArray($triggers);
-		$triggerids = array();
+		$triggerIds = array();
 
 		foreach ($triggers as $trigger) {
 			$triggerDbFields = array(
 				'description' => null,
 				'expression' => null,
-				'error' => _('Trigger just added. No status update so far.'),
-				'value' => TRIGGER_VALUE_UNKNOWN
+				'error' => _('Trigger just added. No status update so far.')
 			);
 			if (!check_db_fields($triggerDbFields, $trigger)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Wrong fields for trigger.'));
@@ -756,60 +503,57 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 			// check for "templateid", because it is not allowed
 			if (array_key_exists('templateid', $trigger)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Cannot set "templateid" for trigger prototype "%1$s".', $trigger['description']));
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Cannot set "templateid" for trigger prototype "%1$s".', $trigger['description']));
 			}
 
-			$expressionData = new CTriggerExpression();
-			if (!$expressionData->parse($trigger['expression'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, $expressionData->error);
+			$triggerExpression = new CTriggerExpression();
+			if (!$triggerExpression->parse($trigger['expression'])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, $triggerExpression->error);
 			}
 
 			$this->checkIfExistsOnHost($trigger);
-		}
 
-		$this->createReal($triggers);
+			// check item prototypes
+			$items = getExpressionItems($triggerExpression);
 
-		$triggerids = zbx_objectValues($triggers, 'triggerid');
-		$createdTriggers = $this->get(array(
-			'triggerids' => $triggerids,
-			'output' => array('description', 'expression', 'flags'),
-			'selectItems' => API_OUTPUT_EXTEND,
-			'selectHosts' => array('name')
-		));
-		foreach ($createdTriggers as $createdTrigger) {
 			$hasPrototype = false;
-
-			foreach ($createdTrigger['items'] as $titem) {
-				if ($titem['flags'] == ZBX_FLAG_DISCOVERY_CHILD) {
+			foreach ($items as $item) {
+				if ($item['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
 					$hasPrototype = true;
 					break;
 				}
 			}
+
 			if (!$hasPrototype) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Trigger "%1$s:%2$s" does not have item prototype.', $createdTrigger['description'], $createdTrigger['expression']));
+					_s('Trigger prototype expression "%1$s" must contain at least one item prototype.',
+						$trigger['expression']));
 			}
 		}
 
+		$this->createReal($triggers);
+
 		foreach ($triggers as $trigger) {
 			$this->inherit($trigger);
 		}
 
-		return array('triggerids' => $triggerids);
+		return array('triggerids' => zbx_objectValues($triggers, 'triggerid'));
 	}
 
 	/**
-	 * Update triggers
+	 * Update triggers.
 	 *
 	 * @param array $triggers
-	 * @return boolean
+	 *
+	 * @return array
 	 */
-	public function update($triggers) {
+	public function update(array $triggers) {
 		$triggers = zbx_toArray($triggers);
-		$triggerids = zbx_objectValues($triggers, 'triggerid');
+		$triggerIds = zbx_objectValues($triggers, 'triggerid');
 
 		$dbTriggers = $this->get(array(
-			'triggerids' => $triggerids,
+			'triggerids' => $triggerIds,
 			'editable' => true,
 			'output' => API_OUTPUT_EXTEND,
 			'preservekeys' => true
@@ -817,18 +561,19 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		$triggers = $this->extendObjects($this->tableName(), $triggers, array('description'));
 
-		foreach ($triggers as $tnum => $trigger) {
+		foreach ($triggers as $key => $trigger) {
 			if (!isset($dbTriggers[$trigger['triggerid']])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('No permissions to referred object or it does not exist!'));
 			}
 
 			if (!isset($trigger['triggerid'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Wrong fields for trigger.'));
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Wrong fields for trigger.'));
 			}
 
 			// check for "templateid", because it is not allowed
 			if (array_key_exists('templateid', $trigger)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Cannot update "templateid" for trigger prototype "%1$s".', $trigger['description']));
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Cannot update "templateid" for trigger prototype "%1$s".', $trigger['description']));
 			}
 
 			$dbTrigger = $dbTriggers[$trigger['triggerid']];
@@ -836,27 +581,49 @@ class CTriggerPrototype extends CTriggerGeneral {
 			if (isset($trigger['expression'])) {
 				$expressionFull = explode_exp($dbTrigger['expression']);
 				if (strcmp($trigger['expression'], $expressionFull) == 0) {
-					unset($triggers[$tnum]['expression']);
+					unset($triggers[$key]['expression']);
+				}
+
+				// check item prototypes
+				$triggerExpression = new CTriggerExpression();
+				if (!$triggerExpression->parse($trigger['expression'])) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, $triggerExpression->error);
+				}
+
+				$items = getExpressionItems($triggerExpression);
+
+				$hasPrototype = false;
+				foreach ($items as $item) {
+					if ($item['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+						$hasPrototype = true;
+						break;
+					}
+				}
+
+				if (!$hasPrototype) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Trigger prototype expression "%1$s" must contain at least one item prototype.',
+							$trigger['expression']));
 				}
 			}
 
 			if (isset($trigger['description']) && strcmp($trigger['description'], $dbTrigger['comments']) == 0) {
-				unset($triggers[$tnum]['description']);
+				unset($triggers[$key]['description']);
 			}
 			if (isset($trigger['priority']) && $trigger['priority'] == $dbTrigger['priority']) {
-				unset($triggers[$tnum]['priority']);
+				unset($triggers[$key]['priority']);
 			}
 			if (isset($trigger['type']) && $trigger['type'] == $dbTrigger['type']) {
-				unset($triggers[$tnum]['type']);
+				unset($triggers[$key]['type']);
 			}
 			if (isset($trigger['comments']) && strcmp($trigger['comments'], $dbTrigger['comments']) == 0) {
-				unset($triggers[$tnum]['comments']);
+				unset($triggers[$key]['comments']);
 			}
 			if (isset($trigger['url']) && strcmp($trigger['url'], $dbTrigger['url']) == 0) {
-				unset($triggers[$tnum]['url']);
+				unset($triggers[$key]['url']);
 			}
 			if (isset($trigger['status']) && $trigger['status'] == $dbTrigger['status']) {
-				unset($triggers[$tnum]['status']);
+				unset($triggers[$key]['status']);
 			}
 
 			$this->checkIfExistsOnHost($trigger);
@@ -864,36 +631,16 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		$this->updateReal($triggers);
 
-		$updatedTriggers = $this->get(array(
-			'triggerids' => zbx_objectValues($triggers, 'triggerid'),
-			'output' => API_OUTPUT_REFER,
-			'selectItems' => API_OUTPUT_EXTEND
-		));
-		foreach ($updatedTriggers as $updatedTrigger) {
-			$hasPrototype = false;
-
-			foreach ($updatedTrigger['items'] as $titem) {
-				if ($titem['flags'] == ZBX_FLAG_DISCOVERY_CHILD) {
-					$hasPrototype = true;
-					break;
-				}
-			}
-			if (!$hasPrototype) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-						sprintf(_('Trigger "%1$s" does not have item prototype.'), $trigger['description']));
-			}
-		}
-
 		foreach ($triggers as $trigger) {
-			$trigger['flags'] = ZBX_FLAG_DISCOVERY_CHILD;
+			$trigger['flags'] = ZBX_FLAG_DISCOVERY_PROTOTYPE;
 			$this->inherit($trigger);
 		}
 
-		return array('triggerids' => $triggerids);
+		return array('triggerids' => $triggerIds);
 	}
 
 	/**
-	 * Delete triggers
+	 * Delete triggers.
 	 *
 	 * @param int|string|array $triggerIds array with trigger ids
 	 * @param bool             $nopermissions
@@ -901,18 +648,20 @@ class CTriggerPrototype extends CTriggerGeneral {
 	 * @return array
 	 */
 	public function delete($triggerIds, $nopermissions = false) {
-		$triggerIds = zbx_toArray($triggerIds);
-
 		if (empty($triggerIds)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
 		}
 
+		$triggerIds = zbx_toArray($triggerIds);
+		$triggerPrototypeIds = $triggerIds;
+
 		$delTriggers = $this->get(array(
 			'triggerids' => $triggerIds,
 			'output' => API_OUTPUT_EXTEND,
 			'editable' => true,
 			'preservekeys' => true
 		));
+
 		// TODO: remove $nopermissions hack
 		if (!$nopermissions) {
 			foreach ($triggerIds as $triggerId) {
@@ -975,14 +724,14 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		DB::delete('triggers', array('triggerid' => $triggerIds));
 
-		return array('triggerids' => $triggerIds);
+		return array('triggerids' => $triggerPrototypeIds);
 	}
 
 	protected function createReal(array &$triggers) {
 		$triggers = zbx_toArray($triggers);
 
-		foreach ($triggers as $num => $trigger) {
-			$triggers[$num]['flags'] = ZBX_FLAG_DISCOVERY_CHILD;
+		foreach ($triggers as $key => $trigger) {
+			$triggers[$key]['flags'] = ZBX_FLAG_DISCOVERY_PROTOTYPE;
 		}
 
 		// insert triggers without expression
@@ -991,23 +740,24 @@ class CTriggerPrototype extends CTriggerGeneral {
 			unset($triggersCopy[$i]['expression']);
 		}
 
-		$triggerids = DB::insert('triggers', $triggersCopy);
+		$triggerIds = DB::insert('triggers', $triggersCopy);
 		unset($triggersCopy);
 
-		foreach ($triggers as $tnum => $trigger) {
-			$triggerid = $triggers[$tnum]['triggerid'] = $triggerids[$tnum];
-
+		foreach ($triggers as $key => $trigger) {
+			$triggerId = $triggers[$key]['triggerid'] = $triggerIds[$key];
 			$hosts = array();
+
 			try {
-				$expression = implode_exp($trigger['expression'], $triggerid, $hosts);
+				$expression = implode_exp($trigger['expression'], $triggerId, $hosts);
 			}
 			catch (Exception $e) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
 						_s('Cannot implode expression "%s".', $trigger['expression']).' '.$e->getMessage());
 			}
+
 			DB::update('triggers', array(
 				'values' => array('expression' => $expression),
-				'where' => array('triggerid' => $triggerid)
+				'where' => array('triggerid' => $triggerId)
 			));
 
 			info(_s('Created: Trigger prototype "%1$s" on "%2$s".', $trigger['description'], implode(', ', $hosts)));
@@ -1038,7 +788,6 @@ class CTriggerPrototype extends CTriggerGeneral {
 			if (isset($trigger['expression']) && strcmp($expressionFull, $trigger['expression']) != 0) {
 				$expressionChanged = true;
 				$expressionFull = $trigger['expression'];
-				$trigger['error'] = 'Trigger expression updated. No status update so far.';
 			}
 
 			if ($descriptionChanged || $expressionChanged) {
@@ -1049,7 +798,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 				if (!isset($expressionData->expressions[0])) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
-							_('Trigger expression must contain at least one host:key reference.'));
+						_('Trigger expression must contain at least one host:key reference.'));
 				}
 			}
 
@@ -1061,7 +810,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 				}
 				catch (Exception $e) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
-							_s('Cannot implode expression "%s".', $expressionFull).' '.$e->getMessage());
+						_s('Cannot implode expression "%s".', $expressionFull).' '.$e->getMessage());
 				}
 			}
 
@@ -1073,6 +822,14 @@ class CTriggerPrototype extends CTriggerGeneral {
 				unset($triggerUpdate['expression']);
 			}
 
+			// skip updating read only values
+			unset(
+				$triggerUpdate['state'],
+				$triggerUpdate['value'],
+				$triggerUpdate['lastchange'],
+				$triggerUpdate['error']
+			);
+
 			DB::update('triggers', array(
 				'values' => $triggerUpdate,
 				'where' => array('triggerid' => $trigger['triggerid'])
@@ -1085,15 +842,16 @@ class CTriggerPrototype extends CTriggerGeneral {
 		unset($trigger);
 	}
 
-	public function syncTemplates($data) {
+	public function syncTemplates(array $data) {
 		$data['templateids'] = zbx_toArray($data['templateids']);
 		$data['hostids'] = zbx_toArray($data['hostids']);
 
 		$triggers = $this->get(array(
 			'hostids' => $data['templateids'],
 			'preservekeys' => true,
-			'output' => API_OUTPUT_EXTEND,
-			'selectDependencies' => true
+			'output' => array(
+				'triggerid', 'expression', 'description', 'url', 'status', 'priority', 'comments', 'type'
+			)
 		));
 
 		foreach ($triggers as $trigger) {
@@ -1103,4 +861,70 @@ class CTriggerPrototype extends CTriggerGeneral {
 
 		return true;
 	}
+
+	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
+		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
+
+		if (!$options['countOutput'] !== null) {
+			// expandData
+			if (!is_null($options['expandData'])) {
+				$sqlParts['select']['host'] = 'h.host';
+				$sqlParts['select']['hostid'] = 'h.hostid';
+				$sqlParts['from']['functions'] = 'functions f';
+				$sqlParts['from']['items'] = 'items i';
+				$sqlParts['from']['hosts'] = 'hosts h';
+				$sqlParts['where']['ft'] = 'f.triggerid=t.triggerid';
+				$sqlParts['where']['fi'] = 'f.itemid=i.itemid';
+				$sqlParts['where']['hi'] = 'h.hostid=i.hostid';
+			}
+		}
+
+		return $sqlParts;
+	}
+
+	protected function addRelatedObjects(array $options, array $result) {
+		$result = parent::addRelatedObjects($options, $result);
+
+		$triggerids = array_keys($result);
+
+		// adding items
+		if ($options['selectItems'] !== null && $options['selectItems'] != API_OUTPUT_COUNT) {
+			$relationMap = $this->createRelationMap($result, 'triggerid', 'itemid', 'functions');
+			$items = API::Item()->get(array(
+				'nodeids' => $options['nodeids'],
+				'output' => $options['selectItems'],
+				'itemids' => $relationMap->getRelatedIds(),
+				'webitems' => true,
+				'nopermissions' => true,
+				'preservekeys' => true,
+				'filter' => array('flags' => null)
+			));
+			$result = $relationMap->mapMany($result, $items, 'items');
+		}
+
+		// adding discoveryrule
+		if ($options['selectDiscoveryRule'] !== null && $options['selectDiscoveryRule'] != API_OUTPUT_COUNT) {
+			$dbRules = DBselect(
+				'SELECT id.parent_itemid,f.triggerid'.
+					' FROM item_discovery id,functions f'.
+					' WHERE '.dbConditionInt('f.triggerid', $triggerids).
+					' AND f.itemid=id.itemid'
+			);
+			$relationMap = new CRelationMap();
+			while ($rule = DBfetch($dbRules)) {
+				$relationMap->addRelation($rule['triggerid'], $rule['parent_itemid']);
+			}
+
+			$discoveryRules = API::DiscoveryRule()->get(array(
+				'output' => $options['selectDiscoveryRule'],
+				'nodeids' => $options['nodeids'],
+				'itemids' => $relationMap->getRelatedIds(),
+				'nopermissions' => true,
+				'preservekeys' => true,
+			));
+			$result = $relationMap->mapOne($result, $discoveryRules, 'discoveryRule');
+		}
+
+		return $result;
+	}
 }
