@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,27 +10,25 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
+
+
 /**
- * File containing CUserGroup class for API.
+ * Class containing methods for operations with user groups.
+ *
  * @package API
  */
-/**
- * Class containing methods for operations with UserGroups.
- */
 class CUserGroup extends CZBXAPI {
 
 	protected $tableName = 'usrgrp';
-
 	protected $tableAlias = 'g';
+	protected $sortColumns = array('usrgrpid', 'name');
 
 	/**
 	 * Get UserGroups
@@ -51,13 +49,6 @@ class CUserGroup extends CZBXAPI {
 	public function get($options = array()) {
 		$result = array();
 		$userType = self::$userData['type'];
-		$userid = self::$userData['userid'];
-
-		// allowed columns for sorting
-		$sortColumns = array('usrgrpid', 'name');
-
-		// allowed output options for [ select_* ] params
-		$subselectsAllowedOutputs = array(API_OUTPUT_REFER, API_OUTPUT_EXTEND);
 
 		$sqlParts = array(
 			'select'	=> array('usrgrp' => 'g.usrgrpid'),
@@ -93,58 +84,40 @@ class CUserGroup extends CZBXAPI {
 
 		$options = zbx_array_merge($defOptions, $options);
 
-		if (is_array($options['output'])) {
-			unset($sqlParts['select']['usrgrp']);
-
-			$dbTable = DB::getSchema('usrgrp');
-			$sqlParts['select']['usrgrpid'] = 'g.usrgrpid';
-			foreach ($options['output'] as $field) {
-				if (isset($dbTable['fields'][$field])) {
-					$sqlParts['select'][$field] = 'g.'.$field;
-				}
-			}
-			$options['output'] = API_OUTPUT_CUSTOM;
-		}
-
 		// permission check
 		if (USER_TYPE_SUPER_ADMIN == $userType) {
 		}
 		elseif (is_null($options['editable']) && (self::$userData['type'] == USER_TYPE_ZABBIX_ADMIN)) {
 			$sqlParts['where'][] = 'g.usrgrpid IN ('.
-				' SELECT uug.usrgrpid'.
+				'SELECT uug.usrgrpid'.
 				' FROM users_groups uug'.
 				' WHERE uug.userid='.self::$userData['userid'].
-				' )';
+				')';
 		}
 		elseif (!is_null($options['editable']) && (self::$userData['type'] != USER_TYPE_SUPER_ADMIN)) {
 			return array();
 		}
 
-		// nodeids
-		$nodeids = !is_null($options['nodeids']) ? $options['nodeids'] : get_current_nodeid();
-
 		// usrgrpids
 		if (!is_null($options['usrgrpids'])) {
 			zbx_value2array($options['usrgrpids']);
 
-			$sqlParts['where'][] = DBcondition('g.usrgrpid', $options['usrgrpids']);
+			$sqlParts['where'][] = dbConditionInt('g.usrgrpid', $options['usrgrpids']);
 		}
 
 		// userids
 		if (!is_null($options['userids'])) {
 			zbx_value2array($options['userids']);
 
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['userid'] = 'ug.userid';
-			}
+			$sqlParts['select']['userid'] = 'ug.userid';
 			$sqlParts['from']['users_groups'] = 'users_groups ug';
-			$sqlParts['where'][] = DBcondition('ug.userid', $options['userids']);
+			$sqlParts['where'][] = dbConditionInt('ug.userid', $options['userids']);
 			$sqlParts['where']['gug'] = 'g.usrgrpid=ug.usrgrpid';
 		}
 
 		// status
 		if (!is_null($options['status'])) {
-			$sqlParts['where'][] = 'g.users_status='.$options['status'];
+			$sqlParts['where'][] = 'g.users_status='.zbx_dbstr($options['status']);
 		}
 
 		// with_gui_access
@@ -152,20 +125,9 @@ class CUserGroup extends CZBXAPI {
 			$sqlParts['where'][] = 'g.gui_access='.GROUP_GUI_ACCESS_ENABLED;
 		}
 
-		// output
-		if ($options['output'] == API_OUTPUT_EXTEND) {
-			$sqlParts['select']['usrgrp'] = 'g.*';
-		}
-
-		// countOutput
-		if (!is_null($options['countOutput'])) {
-			$options['sortfield'] = '';
-			$sqlParts['select'] = array('count(g.usrgrpid) as rowscount');
-		}
-
 		// filter
 		if (is_array($options['filter'])) {
-			zbx_db_filter('usrgrp g', $options, $sqlParts);
+			$this->dbFilter('usrgrp g', $options, $sqlParts);
 		}
 
 		// search
@@ -173,73 +135,33 @@ class CUserGroup extends CZBXAPI {
 			zbx_db_search('usrgrp g', $options, $sqlParts);
 		}
 
-		// sorting
-		zbx_db_sorting($sqlParts, $options, $sortColumns, 'g');
-
 		// limit
 		if (zbx_ctype_digit($options['limit']) && $options['limit']) {
 			$sqlParts['limit'] = $options['limit'];
 		}
 
-		$usrgrpids = array();
-
-		$sqlParts['select'] = array_unique($sqlParts['select']);
-		$sqlParts['from'] = array_unique($sqlParts['from']);
-		$sqlParts['where'] = array_unique($sqlParts['where']);
-		$sqlParts['order'] = array_unique($sqlParts['order']);
-
-		$sqlSelect = '';
-		$sqlFrom = '';
-		$sqlWhere = '';
-		$sqlOrder = '';
-		if (!empty($sqlParts['select'])) {
-			$sqlSelect .= implode(',', $sqlParts['select']);
-		}
-		if (!empty($sqlParts['from'])) {
-			$sqlFrom .= implode(',', $sqlParts['from']);
-		}
-		if (!empty($sqlParts['where'])) {
-			$sqlWhere .= ' AND '.implode(' AND ', $sqlParts['where']);
-		}
-		if (!empty($sqlParts['order'])) {
-			$sqlOrder .= ' ORDER BY '.implode(',', $sqlParts['order']);
-		}
-		$sqlLimit = $sqlParts['limit'];
-
-		$sql = 'SELECT '.zbx_db_distinct($sqlParts).' '.$sqlSelect.'
-				FROM '.$sqlFrom.'
-				WHERE '.DBin_node('g.usrgrpid', $nodeids).
-			$sqlWhere.
-			$sqlOrder;
-		$res = DBselect($sql, $sqlLimit);
+		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$sqlParts = $this->applyQueryNodeOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
+		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($usrgrp = DBfetch($res)) {
 			if ($options['countOutput']) {
 				$result = $usrgrp['rowscount'];
 			}
 			else {
-				$usrgrpids[$usrgrp['usrgrpid']] = $usrgrp['usrgrpid'];
-
-				if ($options['output'] == API_OUTPUT_SHORTEN) {
-					$result[$usrgrp['usrgrpid']] = array('usrgrpid' => $usrgrp['usrgrpid']);
+				if (!isset($result[$usrgrp['usrgrpid']])) {
+					$result[$usrgrp['usrgrpid']]= array();
 				}
-				else {
-					if (!isset($result[$usrgrp['usrgrpid']])) {
-						$result[$usrgrp['usrgrpid']]= array();
-					}
-					if (!is_null($options['selectUsers']) && !isset($result[$usrgrp['usrgrpid']]['users'])) {
+
+				// groupids
+				if (isset($usrgrp['userid']) && is_null($options['selectUsers'])) {
+					if (!isset($result[$usrgrp['usrgrpid']]['users'])) {
 						$result[$usrgrp['usrgrpid']]['users'] = array();
 					}
-
-					// groupids
-					if (isset($usrgrp['userid']) && is_null($options['selectUsers'])) {
-						if (!isset($result[$usrgrp['usrgrpid']]['users'])) {
-							$result[$usrgrp['usrgrpid']]['users'] = array();
-						}
-						$result[$usrgrp['usrgrpid']]['users'][] = array('userid' => $usrgrp['userid']);
-						unset($usrgrp['userid']);
-					}
-					$result[$usrgrp['usrgrpid']] += $usrgrp;
+					$result[$usrgrp['usrgrpid']]['users'][] = array('userid' => $usrgrp['userid']);
+					unset($usrgrp['userid']);
 				}
+				$result[$usrgrp['usrgrpid']] += $usrgrp;
 			}
 		}
 
@@ -247,25 +169,8 @@ class CUserGroup extends CZBXAPI {
 			return $result;
 		}
 
-		/*
-		 * Adding objects
-		 */
-		// adding users
-		if (!is_null($options['selectUsers']) && str_in_array($options['selectUsers'], $subselectsAllowedOutputs)) {
-			$objParams = array(
-				'output' => $options['selectUsers'],
-				'usrgrpids' => $usrgrpids,
-				'getAccess' => $options['selectUsers'] == API_OUTPUT_EXTEND ? true : null,
-				'preservekeys' => true
-			);
-			$users = API::User()->get($objParams);
-			foreach ($users as $user) {
-				$uusrgrps = $user['usrgrps'];
-				unset($user['usrgrps']);
-				foreach ($uusrgrps as $usrgrp) {
-					$result[$usrgrp['usrgrpid']]['users'][] = $user;
-				}
-			}
+		if ($result) {
+			$result = $this->addRelatedObjects($options, $result);
 		}
 
 		// removing keys (hash -> array)
@@ -285,17 +190,19 @@ class CUserGroup extends CZBXAPI {
 		$result = array();
 		$usrgrpids = array();
 
-		$sql = 'SELECT g.usrgrpid '.
-			' FROM usrgrp g '.
-			' WHERE g.name='.zbx_dbstr($groupData['name']).
-			' AND '.DBin_node('g.usrgrpid', false);
-		$res = DBselect($sql);
+		$res = DBselect(
+				'SELECT g.usrgrpid'.
+				' FROM usrgrp g'.
+				' WHERE g.name='.zbx_dbstr($groupData['name']).
+					andDbNode('g.usrgrpid', false)
+		);
 		while ($group = DBfetch($res)) {
 			$usrgrpids[$group['usrgrpid']] = $group['usrgrpid'];
 		}
 
-		if (!empty($usrgrpids))
-			$result = $this->get(array('usrgrpids'=>$usrgrpids, 'output' => API_OUTPUT_EXTEND));
+		if (!empty($usrgrpids)) {
+			$result = $this->get(array('usrgrpids' => $usrgrpids, 'output' => API_OUTPUT_EXTEND));
+		}
 
 		return $result;
 	}
@@ -303,7 +210,7 @@ class CUserGroup extends CZBXAPI {
 	public function exists($object) {
 		$options = array(
 			'filter' => array('name' => $object['name']),
-			'output' => API_OUTPUT_SHORTEN,
+			'output' => array('usrgrpid'),
 			'nopermissions' => 1,
 			'limit' => 1,
 		);
@@ -369,7 +276,7 @@ class CUserGroup extends CZBXAPI {
 
 	/**
 	 * Update UserGroups.
-	 * Checks permissions - only superadmins can update usergroups.
+	 * Checks permissions - only super admins can update usergroups.
 	 * Formats data to be used in massUpdate() method.
 	 *
 	 * @param array $usrgrps
@@ -428,8 +335,8 @@ class CUserGroup extends CZBXAPI {
 			$linkedUsers = array();
 			$sql = 'SELECT usrgrpid, userid'.
 				' FROM users_groups'.
-				' WHERE '.DBcondition('usrgrpid', $usrgrpids).
-				' AND '.DBcondition('userid', $userids);
+				' WHERE '.dbConditionInt('usrgrpid', $usrgrpids).
+				' AND '.dbConditionInt('userid', $userids);
 			$linkedUsersDb = DBselect($sql);
 			while ($link = DBfetch($linkedUsersDb)) {
 				if (!isset($linkedUsers[$link['usrgrpid']])) $linkedUsers[$link['usrgrpid']] = array();
@@ -452,10 +359,10 @@ class CUserGroup extends CZBXAPI {
 
 		if (!is_null($rights)) {
 			$linkedRights = array();
-			$sql = 'SELECT groupid, id'.
+			$sql = 'SELECT groupid,id'.
 				' FROM rights'.
-				' WHERE '.DBcondition('groupid', $usrgrpids);
-			' AND '.DBcondition('id', zbx_objectValues($rights, 'id'));
+				' WHERE '.dbConditionInt('groupid', $usrgrpids);
+			' AND '.dbConditionInt('id', zbx_objectValues($rights, 'id'));
 			$linkedRightsDb = DBselect($sql);
 			while ($link = DBfetch($linkedRightsDb)) {
 				if (!isset($linkedRights[$link['groupid']])) $linkedRights[$link['groupid']] = array();
@@ -480,7 +387,6 @@ class CUserGroup extends CZBXAPI {
 		return array('usrgrpids' => $usrgrpids);
 	}
 
-
 	/**
 	 * Mass update user group.
 	 * Checks for permissions - only super admins can change user groups.
@@ -520,7 +426,7 @@ class CUserGroup extends CZBXAPI {
 				// check if there already is hostgroup with this name, except current hostgroup
 				$groupExists = $this->get(array(
 					'filter' => array('name' => $data['name']),
-					'output' => API_OUTPUT_SHORTEN,
+					'output' => array('usrgrpid'),
 					'limit' => 1
 				));
 				$groupExists = reset($groupExists);
@@ -564,9 +470,9 @@ class CUserGroup extends CZBXAPI {
 
 			// get already linked users
 			$linkedUsers = array();
-			$sql = 'SELECT usrgrpid, userid'.
+			$sql = 'SELECT usrgrpid,userid'.
 				' FROM users_groups'.
-				' WHERE '.DBcondition('usrgrpid', $usrgrpids);
+				' WHERE '.dbConditionInt('usrgrpid', $usrgrpids);
 			$linkedUsersDb = DBselect($sql);
 			while ($link = DBfetch($linkedUsersDb)) {
 				if (!isset($linkedUsers[$link['usrgrpid']])) {
@@ -616,9 +522,9 @@ class CUserGroup extends CZBXAPI {
 
 			// get already linked rights
 			$linkedRights = array();
-			$sql = 'SELECT groupid, permission, id'.
+			$sql = 'SELECT groupid,permission,id'.
 				' FROM rights'.
-				' WHERE '.DBcondition('groupid', $usrgrpids);
+				' WHERE '.dbConditionInt('groupid', $usrgrpids);
 			$linkedRightsDb = DBselect($sql);
 			while ($link = DBfetch($linkedRightsDb)) {
 				if (!isset($linkedRights[$link['groupid']])) {
@@ -675,14 +581,9 @@ class CUserGroup extends CZBXAPI {
 			}
 		}
 
-
 		return array('usrgrpids' => $usrgrpids);
 	}
 
-	public function massRemove($data) {
-
-	}
-
 	/**
 	 * Delete UserGroups.
 	 *
@@ -727,9 +628,9 @@ class CUserGroup extends CZBXAPI {
 
 		// delete action operation msg
 		$operationids = array();
-		$sql = 'SELECT DISTINCT om.operationid '.
-			' FROM opmessage_grp om '.
-			' WHERE '.DBcondition('om.usrgrpid', $usrgrpids);
+		$sql = 'SELECT DISTINCT om.operationid'.
+			' FROM opmessage_grp om'.
+			' WHERE '.dbConditionInt('om.usrgrpid', $usrgrpids);
 		$dbOperations = DBselect($sql);
 		while ($dbOperation = DBfetch($dbOperations))
 			$operationids[$dbOperation['operationid']] = $dbOperation['operationid'];
@@ -738,9 +639,9 @@ class CUserGroup extends CZBXAPI {
 
 		// delete empty operations
 		$delOperationids = array();
-		$sql = 'SELECT DISTINCT o.operationid '.
-			' FROM operations o '.
-			' WHERE '.DBcondition('o.operationid', $operationids).
+		$sql = 'SELECT DISTINCT o.operationid'.
+			' FROM operations o'.
+			' WHERE '.dbConditionInt('o.operationid', $operationids).
 			' AND NOT EXISTS(SELECT om.opmessage_grpid FROM opmessage_grp om WHERE om.operationid=o.operationid)';
 		$dbOperations = DBselect($sql);
 		while ($dbOperation = DBfetch($dbOperations))
@@ -763,7 +664,6 @@ class CUserGroup extends CZBXAPI {
 		$count = $this->get(array(
 			'nodeids' => get_current_nodeid(true),
 			'usrgrpids' => $ids,
-			'output' => API_OUTPUT_SHORTEN,
 			'countOutput' => true
 		));
 
@@ -779,7 +679,6 @@ class CUserGroup extends CZBXAPI {
 		$count = $this->get(array(
 			'nodeids' => get_current_nodeid(true),
 			'usrgrpids' => $ids,
-			'output' => API_OUTPUT_SHORTEN,
 			'editable' => true,
 			'countOutput' => true
 		));
@@ -787,5 +686,21 @@ class CUserGroup extends CZBXAPI {
 		return (count($ids) == $count);
 	}
 
+	protected function addRelatedObjects(array $options, array $result) {
+		$result = parent::addRelatedObjects($options, $result);
+
+		// adding users
+		if ($options['selectUsers'] !== null && $options['selectUsers'] != API_OUTPUT_COUNT) {
+			$relationMap = $this->createRelationMap($result, 'usrgrpid', 'userid', 'users_groups');
+			$users = API::User()->get(array(
+				'output' => $options['selectUsers'],
+				'userids' => $relationMap->getRelatedIds(),
+				'getAccess' => $options['selectUsers'] == API_OUTPUT_EXTEND ? true : null,
+				'preservekeys' => true
+			));
+			$result = $relationMap->mapMany($result, $users, 'users');
+		}
+
+		return $result;
+	}
 }
-?>
