@@ -19,30 +19,32 @@
 **/
 
 
+/**
+ * Class containing methods for operations with user macro.
+ *
+ * @package API
+ */
 class CUserMacro extends CZBXAPI {
 
 	protected $tableName = 'hostmacro';
 	protected $tableAlias = 'hm';
+	protected $sortColumns = array('macro');
 
 	/**
-	 * Get UserMacros data
+	 * Get UserMacros data.
 	 *
 	 * @param array $options
-	 * @param array $options['nodeids'] Node IDs
-	 * @param array $options['groupids'] UserMacrosGroup IDs
-	 * @param array $options['macroids'] UserMacros IDs
-	 * @param boolean $options['monitored_macros'] only monitored UserMacros
-	 * @param boolean $options['templated_macros'] include templates in result
-	 * @param boolean $options['with_items'] only with items
-	 * @param boolean $options['with_historical_items'] only with historical items
-	 * @param boolean $options['with_triggers'] only with triggers
-	 * @param boolean $options['with_httptests'] only with http tests
-	 * @param boolean $options['with_graphs'] only with graphs
-	 * @param boolean $options['editable'] only with read-write permission. Ignored for SuperAdmins
-	 * @param int $options['count'] count UserMacros, returned column name is rowscount
-	 * @param string $options['pattern'] search macros by pattern in macro names
-	 * @param int $options['limit'] limit selection
-	 * @param string $options['order'] deprecated parameter (for now)
+	 * @param array $options['nodeids'] node ids
+	 * @param array $options['groupids'] usermacrosgroup ids
+	 * @param array $options['hostids'] host ids
+	 * @param array $options['hostmacroids'] host macros ids
+	 * @param array $options['globalmacroids'] global macros ids
+	 * @param array $options['templateids'] tempalate ids
+	 * @param boolean $options['globalmacro'] only global macros
+	 * @param boolean $options['selectGroups'] select groups
+	 * @param boolean $options['selectHosts'] select hosts
+	 * @param boolean $options['selectTemplates'] select templates
+	 *
 	 * @return array|boolean UserMacros data as array or false if error
 	 */
 	public function get($options = array()) {
@@ -50,12 +52,6 @@ class CUserMacro extends CZBXAPI {
 		$userType = self::$userData['type'];
 		$userid = self::$userData['userid'];
 
-		// allowed columns for sorting
-		$sortColumns = array('macro');
-
-		// allowed output options for [ select_* ] params
-		$subselectsAllowedOutputs = array(API_OUTPUT_REFER, API_OUTPUT_EXTEND);
-
 		$sqlParts = array(
 			'select'	=> array('macros' => 'hm.hostmacroid'),
 			'from'		=> array('hostmacro hm'),
@@ -108,7 +104,7 @@ class CUserMacro extends CZBXAPI {
 				return array();
 			}
 			else {
-				$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ_ONLY;
+				$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
 
 				$userGroups = getUserGroupsByUserId($userid);
 
@@ -120,7 +116,8 @@ class CUserMacro extends CZBXAPI {
 									' AND '.dbConditionInt('r.groupid', $userGroups).
 						' WHERE hm.hostid=hgg.hostid'.
 						' GROUP BY hgg.hostid'.
-						' HAVING MIN(r.permission)>='.$permission.
+						' HAVING MIN(r.permission)>'.PERM_DENY.
+							' AND MAX(r.permission)>='.$permission.
 						')';
 			}
 		}
@@ -130,7 +127,7 @@ class CUserMacro extends CZBXAPI {
 
 		// global macro
 		if (!is_null($options['globalmacro'])) {
-			$sqlPartsGlobal['where'][] = DBin_node('gm.globalmacroid', $nodeids);
+			$sqlPartsGlobal['where'] = sqlPartDbNode($sqlPartsGlobal['where'], 'gm.globalmacroid', $nodeids);
 			$options['groupids'] = null;
 			$options['hostmacroids'] = null;
 			$options['triggerids'] = null;
@@ -141,7 +138,7 @@ class CUserMacro extends CZBXAPI {
 			$options['selectHosts'] = null;
 		}
 		else {
-			$sqlParts['where'][] = DBin_node('hm.hostmacroid', $nodeids);
+			$sqlParts['where'] = sqlPartDbNode($sqlParts['where'], 'hm.hostmacroid', $nodeids);
 		}
 
 		// globalmacroids
@@ -159,9 +156,8 @@ class CUserMacro extends CZBXAPI {
 		// groupids
 		if (!is_null($options['groupids'])) {
 			zbx_value2array($options['groupids']);
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['groupid'] = 'hg.groupid';
-			}
+
+			$sqlParts['select']['groupid'] = 'hg.groupid';
 			$sqlParts['from']['hosts_groups'] = 'hosts_groups hg';
 			$sqlParts['where'][] = dbConditionInt('hg.groupid', $options['groupids']);
 			$sqlParts['where']['hgh'] = 'hg.hostid=hm.hostid';
@@ -170,15 +166,16 @@ class CUserMacro extends CZBXAPI {
 		// hostids
 		if (!is_null($options['hostids'])) {
 			zbx_value2array($options['hostids']);
+
+			$sqlParts['select']['hostid'] = 'hm.hostid';
 			$sqlParts['where'][] = dbConditionInt('hm.hostid', $options['hostids']);
 		}
 
 		// templateids
 		if (!is_null($options['templateids'])) {
 			zbx_value2array($options['templateids']);
-			if ($options['output'] != API_OUTPUT_SHORTEN) {
-				$sqlParts['select']['templateid'] = 'ht.templateid';
-			}
+
+			$sqlParts['select']['templateid'] = 'ht.templateid';
 			$sqlParts['from']['macros_templates'] = 'hosts_templates ht';
 			$sqlParts['where'][] = dbConditionInt('ht.templateid', $options['templateids']);
 			$sqlParts['where']['hht'] = 'hm.hostid=ht.hostid';
@@ -200,23 +197,9 @@ class CUserMacro extends CZBXAPI {
 			}
 		}
 
-		// output
-		if ($options['output'] == API_OUTPUT_EXTEND) {
-			$sqlParts['select']['macros'] = 'hm.*';
-			$sqlPartsGlobal['select']['macros'] = 'gm.*';
-		}
-
-		// countOutput
-		if (!is_null($options['countOutput'])) {
-			$options['sortfield'] = '';
-
-			$sqlParts['select'] = array('count(DISTINCT hm.hostmacroid) as rowscount');
-			$sqlPartsGlobal['select'] = array('count(DISTINCT gm.globalmacroid) as rowscount');
-		}
-
 		// sorting
-		zbx_db_sorting($sqlParts, $options, $sortColumns, 'hm');
-		zbx_db_sorting($sqlPartsGlobal, $options, $sortColumns, 'gm');
+		$sqlParts = $this->applyQuerySortOptions('hostmacro', 'hm', $options, $sqlParts);
+		$sqlPartsGlobal = $this->applyQuerySortOptions('globalmacro', 'gm', $options, $sqlPartsGlobal);
 
 		// limit
 		if (zbx_ctype_digit($options['limit']) && $options['limit']) {
@@ -226,90 +209,62 @@ class CUserMacro extends CZBXAPI {
 
 		// init GLOBALS
 		if (!is_null($options['globalmacro'])) {
+			$sqlPartsGlobal = $this->applyQueryOutputOptions('globalmacro', 'gm', $options, $sqlPartsGlobal);
 			$res = DBselect($this->createSelectQueryFromParts($sqlPartsGlobal), $sqlPartsGlobal['limit']);
 			while ($macro = DBfetch($res)) {
 				if ($options['countOutput']) {
 					$result = $macro['rowscount'];
 				}
 				else {
-					$globalmacroids[$macro['globalmacroid']] = $macro['globalmacroid'];
+					if (!isset($result[$macro['globalmacroid']])) {
+						$result[$macro['globalmacroid']] = array();
+					}
 
-					if ($options['output'] == API_OUTPUT_SHORTEN) {
-						$result[$macro['globalmacroid']] = array('globalmacroid' => $macro['globalmacroid']);
-					}
-					else {
-						if (!isset($result[$macro['globalmacroid']])) {
-							$result[$macro['globalmacroid']] = array();
-						}
-						$result[$macro['globalmacroid']] += $macro;
-					}
+					$result[$macro['globalmacroid']] += $macro;
 				}
 			}
 		}
 		// init HOSTS
 		else {
-			$hostids = array();
-
+			$sqlParts = $this->applyQueryOutputOptions('hostmacro', 'hm', $options, $sqlParts);
 			$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 			while ($macro = DBfetch($res)) {
+
 				if ($options['countOutput']) {
 					$result = $macro['rowscount'];
 				}
 				else {
-					$hostmacroids[$macro['hostmacroid']] = $macro['hostmacroid'];
-
-					if ($options['output'] == API_OUTPUT_SHORTEN) {
-						$result[$macro['hostmacroid']] = $macro['hostmacroid'];
+					if (!isset($result[$macro['hostmacroid']])) {
+						$result[$macro['hostmacroid']] = array();
 					}
-					else {
-						$hostids[$macro['hostid']] = $macro['hostid'];
 
-						if (!isset($result[$macro['hostmacroid']])) {
-							$result[$macro['hostmacroid']]= array();
-						}
-
-						// groups
-						if ($options['selectGroups'] && !isset($result[$macro['hostmacroid']]['groups'])) {
+					// groupids
+					if (isset($macro['groupid'])) {
+						if (!isset($result[$macro['hostmacroid']]['groups'])) {
 							$result[$macro['hostmacroid']]['groups'] = array();
 						}
+						$result[$macro['hostmacroid']]['groups'][] = array('groupid' => $macro['groupid']);
+						unset($macro['groupid']);
+					}
 
-						// templates
-						if ($options['selectTemplates'] && !isset($result[$macro['hostmacroid']]['templates'])) {
+					// templateids
+					if (isset($macro['templateid'])) {
+						if (!isset($result[$macro['hostmacroid']]['templates'])) {
 							$result[$macro['hostmacroid']]['templates'] = array();
 						}
+						$result[$macro['hostmacroid']]['templates'][] = array('templateid' => $macro['templateid']);
+						unset($macro['templateid']);
+					}
 
-						// hosts
-						if ($options['selectHosts'] && !isset($result[$macro['hostmacroid']]['hosts'])) {
+					// hostids
+					if (isset($macro['hostid'])) {
+						if (!isset($result[$macro['hostmacroid']]['hosts'])) {
 							$result[$macro['hostmacroid']]['hosts'] = array();
 						}
-
-						// groupids
-						if (isset($macro['groupid'])) {
-							if (!isset($result[$macro['hostmacroid']]['groups'])) {
-								$result[$macro['hostmacroid']]['groups'] = array();
-							}
-							$result[$macro['hostmacroid']]['groups'][] = array('groupid' => $macro['groupid']);
-							unset($macro['groupid']);
-						}
-
-						// templateids
-						if (isset($macro['templateid'])) {
-							if (!isset($result[$macro['hostmacroid']]['templates'])) {
-								$result[$macro['hostmacroid']]['templates'] = array();
-							}
-							$result[$macro['hostmacroid']]['templates'][] = array('templateid' => $macro['templateid']);
-							unset($macro['templateid']);
-						}
-
-						// hostids
-						if (isset($macro['hostid'])) {
-							if (!isset($result[$macro['hostmacroid']]['hosts'])) {
-								$result[$macro['hostmacroid']]['hosts'] = array();
-							}
-							$result[$macro['hostmacroid']]['hosts'][] = array('hostid' => $macro['hostid']);
-						}
-						$result[$macro['hostmacroid']] += $macro;
+						$result[$macro['hostmacroid']]['hosts'][] = array('hostid' => $macro['hostid']);
 					}
+
+					$result[$macro['hostmacroid']] += $macro;
 				}
 			}
 		}
@@ -318,72 +273,16 @@ class CUserMacro extends CZBXAPI {
 			return $result;
 		}
 
-		/*
-		 * Adding objects
-		 */
-		// adding groups
-		if (!is_null($options['selectGroups']) && str_in_array($options['selectGroups'], $subselectsAllowedOutputs)) {
-			$objParams = array(
-				'output' => $options['selectGroups'],
-				'hostids' => $hostids,
-				'preservekeys' => true
-			);
-			$groups = API::HostGroup()->get($objParams);
-			foreach ($groups as $group) {
-				$ghosts = $group['hosts'];
-				unset($group['hosts']);
-				foreach ($ghosts as $host) {
-					foreach ($result as $macroid => $macro) {
-						if (bccomp($macro['hostid'], $host['hostid']) == 0) {
-							$result[$macroid]['groups'][] = $group;
-						}
-					}
-				}
-			}
-		}
-
-		// adding templates
-		if (!is_null($options['selectTemplates']) && str_in_array($options['selectTemplates'], $subselectsAllowedOutputs)) {
-			$objParams = array(
-				'output' => $options['selectTemplates'],
-				'hostids' => $hostids,
-				'preservekeys' => true
-			);
-			$templates = API::Template()->get($objParams);
-			foreach ($templates as $template) {
-				$thosts = $template['hosts'];
-				unset($template['hosts']);
-				foreach ($thosts as $host) {
-					foreach ($result as $macroid => $macro) {
-						if (bccomp($macro['hostid'], $host['hostid']) == 0) {
-							$result[$macroid]['templates'][] = $template;
-						}
-					}
-				}
-			}
-		}
-
-		// adding hosts
-		if (!is_null($options['selectHosts']) && str_in_array($options['selectHosts'], $subselectsAllowedOutputs)) {
-			$objParams = array(
-				'output' => $options['selectHosts'],
-				'hostids' => $hostids,
-				'preservekeys' => true
-			);
-			$hosts = API::Host()->get($objParams);
-			foreach ($hosts as $hostid => $host) {
-				foreach ($result as $macroid => $macro) {
-					if (bccomp($macro['hostid'], $hostid) == 0) {
-						$result[$macroid]['hosts'][] = $host;
-					}
-				}
-			}
+		if ($result) {
+			$result = $this->addRelatedObjects($options, $result);
+			$result = $this->unsetExtraFields($result, array('hostid'), $options['output']);
 		}
 
 		// removing keys (hash -> array)
 		if (is_null($options['preservekeys'])) {
 			$result = zbx_cleanHashes($result);
 		}
+
 		return $result;
 	}
 
@@ -685,27 +584,30 @@ class CUserMacro extends CZBXAPI {
 	 */
 	public function getMacros($data) {
 		$macros = $data['macros'];
-		$itemid = isset($data['itemid']) ? $data['itemid'] : null;
-		$triggerid = isset($data['triggerid']) ? $data['triggerid'] : null;
+		$hostIds = isset($data['hostid']) ? zbx_toArray($data['hostid']) : null;
 
+		$result = array();
 		zbx_value2array($macros);
 		$macros = array_unique($macros);
 
-		$result = array();
+		if (!$hostIds) {
+			$itemid = isset($data['itemid']) ? $data['itemid'] : null;
+			$triggerid = isset($data['triggerid']) ? $data['triggerid'] : null;
 
-		$hosts = API::Host()->get(array(
-			'itemids' => $itemid,
-			'triggerids' => $triggerid,
-			'nopermissions' => true,
-			'preservekeys' => true,
-			'output' => API_OUTPUT_SHORTEN,
-			'templated_hosts' => true
-		));
-		$hostids = array_keys($hosts);
+			$hosts = API::Host()->get(array(
+				'itemids' => $itemid,
+				'triggerids' => $triggerid,
+				'nopermissions' => true,
+				'preservekeys' => true,
+				'output' => array('hostid'),
+				'templated_hosts' => true
+			));
+			$hostIds = array_keys($hosts);
+		}
 
 		do {
 			$hostMacros = $this->get(array(
-				'hostids' => $hostids,
+				'hostids' => $hostIds,
 				'macros' => $macros,
 				'output' => API_OUTPUT_EXTEND,
 				'nopermissions' => true,
@@ -725,14 +627,14 @@ class CUserMacro extends CZBXAPI {
 
 			if (!empty($macros)) {
 				$hosts = API::Template()->get(array(
-					'hostids' => $hostids,
+					'hostids' => $hostIds,
 					'nopermissions' => true,
 					'preservekeys' => true,
-					'output' => API_OUTPUT_SHORTEN
+					'output' => array('templateid')
 				));
-				$hostids = array_keys($hosts);
+				$hostIds = array_keys($hosts);
 			}
-		} while (!empty($macros) && !empty($hostids));
+		} while (!empty($macros) && !empty($hostIds));
 
 
 		if (!empty($macros)) {
@@ -837,7 +739,7 @@ class CUserMacro extends CZBXAPI {
 		$macrosToAdd = array();
 
 		foreach ($macros as $hostid => $hostMacros) {
-			$dbHostMacros = $dbMacros[$hostid]['macros'];
+			$dbHostMacros = zbx_toHash($dbMacros[$hostid]['macros'], 'hostmacroid');
 
 			// look for db macros which hostmacroids are not in list of new macros
 			// if there are any, they should be deleted
@@ -1063,7 +965,7 @@ class CUserMacro extends CZBXAPI {
 	 */
 	protected function checkIfGlobalMacrosExist(array $globalMacroIds) {
 		$globalMacros = API::getApi()->select('globalmacro', array(
-			'output' => API_OUTPUT_SHORTEN,
+			'output' => array('globalmacroid'),
 			'globalmacroids' => $globalMacroIds
 		));
 		$globalMacros = zbx_toHash($globalMacros, 'globalmacroid');
@@ -1073,4 +975,72 @@ class CUserMacro extends CZBXAPI {
 			}
 		}
 	}
+
+	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
+		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
+
+		if ($options['output'] != API_OUTPUT_COUNT && $options['globalmacro'] === null) {
+			if ($options['selectGroups'] !== null || $options['selectHosts'] !== null || $options['selectTemplates'] !== null) {
+				$sqlParts = $this->addQuerySelect($this->fieldId('hostid'), $sqlParts);
+			}
+		}
+
+		return $sqlParts;
+	}
+
+	protected function addRelatedObjects(array $options, array $result) {
+		$result = parent::addRelatedObjects($options, $result);
+
+		if ($options['globalmacro'] === null) {
+			$hostMacroIds = array_keys($result);
+
+			/*
+			 * Adding objects
+			 */
+			// adding groups
+			if ($options['selectGroups'] !== null && $options['selectGroups'] != API_OUTPUT_COUNT) {
+				$res = DBselect(
+					'SELECT hm.hostmacroid,hg.groupid'.
+						' FROM hostmacro hm,hosts_groups hg'.
+						' WHERE '.dbConditionInt('hm.hostmacroid', $hostMacroIds).
+						' AND hm.hostid=hg.hostid'
+				);
+				$relationMap = new CRelationMap();
+				while ($relation = DBfetch($res)) {
+					$relationMap->addRelation($relation['hostmacroid'], $relation['groupid']);
+				}
+
+				$groups = API::HostGroup()->get(array(
+					'output' => $options['selectGroups'],
+					'groupids' => $relationMap->getRelatedIds(),
+					'preservekeys' => true
+				));
+				$result = $relationMap->mapMany($result, $groups, 'groups');
+			}
+
+			// adding templates
+			if ($options['selectTemplates'] !== null && $options['selectTemplates'] != API_OUTPUT_COUNT) {
+				$relationMap = $this->createRelationMap($result, 'hostmacroid', 'hostid');
+				$templates = API::Template()->get(array(
+					'output' => $options['selectTemplates'],
+					'templateids' => $relationMap->getRelatedIds(),
+					'preservekeys' => true
+				));
+				$result = $relationMap->mapMany($result, $templates, 'templates');
+			}
+
+			// adding templates
+			if ($options['selectHosts'] !== null && $options['selectHosts'] != API_OUTPUT_COUNT) {
+				$relationMap = $this->createRelationMap($result, 'hostmacroid', 'hostid');
+				$templates = API::Host()->get(array(
+					'output' => $options['selectHosts'],
+					'hostids' => $relationMap->getRelatedIds(),
+					'preservekeys' => true
+				));
+				$result = $relationMap->mapMany($result, $templates, 'hosts');
+			}
+		}
+
+		return $result;
+	}
 }
