@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -31,6 +31,8 @@ require_once dirname(__FILE__).'/include/page_header.php';
 $fields = array(
 	'period' =>			array(T_ZBX_INT, O_OPT, P_NZERO,	BETWEEN(ZBX_MIN_PERIOD, ZBX_MAX_PERIOD), null),
 	'stime' =>			array(T_ZBX_INT, O_OPT, P_NZERO,	null,				null),
+	'profileIdx' =>		array(T_ZBX_STR, O_OPT, null,		null,				null),
+	'profileIdx2' =>	array(T_ZBX_STR, O_OPT, null,		null,				null),
 	'httptestid' =>		array(T_ZBX_INT, O_OPT, P_NZERO,	null,				null),
 	'http_item_type' =>	array(T_ZBX_INT, O_OPT, null,		null,				null),
 	'name' =>			array(T_ZBX_STR, O_OPT, null,		null,				null),
@@ -52,7 +54,11 @@ $fields = array(
 );
 $isDataValid = check_fields($fields);
 
-if ($httptestid = get_request('httptestid', false)) {
+if ($httptestid = getRequest('httptestid', false)) {
+	if (!API::HttpTest()->isReadable(array($_REQUEST['httptestid']))) {
+		access_deny();
+	}
+
 	$color = array(
 		'current' => 0,
 		0 => array('next' => '1'),
@@ -75,9 +81,9 @@ if ($httptestid = get_request('httptestid', false)) {
 		'SELECT i.itemid'.
 		' FROM httpstepitem hi,items i,httpstep hs'.
 		' WHERE i.itemid=hi.itemid'.
-			' AND hs.httptestid='.$httptestid.
+			' AND hs.httptestid='.zbx_dbstr($httptestid).
 			' AND hs.httpstepid=hi.httpstepid'.
-			' AND hi.type='.get_request('http_item_type', HTTPSTEP_ITEM_TYPE_TIME).
+			' AND hi.type='.zbx_dbstr(getRequest('http_item_type', HTTPSTEP_ITEM_TYPE_TIME)).
 		' ORDER BY hs.no DESC'
 	);
 	while ($item = DBfetch($dbItems)) {
@@ -86,20 +92,19 @@ if ($httptestid = get_request('httptestid', false)) {
 		$items[] = array('itemid' => $item['itemid'], 'color' => $itemColor);
 	}
 
-	$httptest = get_httptest_by_httptestid($httptestid);
-	$name = $httptest['name'];
+	$httpTest = get_httptest_by_httptestid($httptestid);
+
+	$name = CMacrosResolverHelper::resolveHttpTestName($httpTest['hostid'], $httpTest['name']);
 }
-else {
-	$items = get_request('items', array());
+elseif ($items = getRequest('items', array())) {
 	asort_by_key($items, 'sortorder');
 
 	$dbItems = API::Item()->get(array(
 		'webitems' => true,
 		'itemids' => zbx_objectValues($items, 'itemid'),
 		'nodeids' => get_current_nodeid(true),
-		'output' => API_OUTPUT_SHORTEN,
-		'preservekeys' => true,
-		'filter' => array('flags' => null)
+		'output' => array('itemid'),
+		'preservekeys' => true
 	));
 
 	$dbItems = zbx_toHash($dbItems, 'itemid');
@@ -108,35 +113,48 @@ else {
 			access_deny();
 		}
 	}
-	$name = get_request('name', '');
+	$name = getRequest('name', '');
+}
+else {
+	show_error_message(_('No items defined.'));
+	exit;
 }
 
 /*
  * Display
  */
 if ($isDataValid) {
-	$graph = new CChart(get_request('graphtype', GRAPH_TYPE_NORMAL));
+	$profileIdx = getRequest('profileIdx', 'web.httptest');
+	$profileIdx2 = getRequest('httptestid', getRequest('profileIdx2'));
+
+	$timeline = CScreenBase::calculateTime(array(
+		'profileIdx' => $profileIdx,
+		'profileIdx2' => $profileIdx2,
+		'period' => getRequest('period'),
+		'stime' => getRequest('stime')
+	));
+
+	CProfile::update($profileIdx.'.httptestid', $profileIdx2, PROFILE_TYPE_ID);
+
+	$graph = new CLineGraphDraw(getRequest('graphtype', GRAPH_TYPE_NORMAL));
 	$graph->setHeader($name);
+	$graph->setPeriod($timeline['period']);
+	$graph->setSTime($timeline['stime']);
+	$graph->setWidth(getRequest('width', 900));
+	$graph->setHeight(getRequest('height', 200));
+	$graph->showLegend(getRequest('legend', 1));
+	$graph->showWorkPeriod(getRequest('showworkperiod', 1));
+	$graph->showTriggers(getRequest('showtriggers', 1));
+	$graph->setYMinAxisType(getRequest('ymin_type', GRAPH_YAXIS_TYPE_CALCULATED));
+	$graph->setYMaxAxisType(getRequest('ymax_type', GRAPH_YAXIS_TYPE_CALCULATED));
+	$graph->setYAxisMin(getRequest('yaxismin', 0.00));
+	$graph->setYAxisMax(getRequest('yaxismax', 100.00));
+	$graph->setYMinItemId(getRequest('ymin_itemid', 0));
+	$graph->setYMaxItemId(getRequest('ymax_itemid', 0));
+	$graph->setLeftPercentage(getRequest('percent_left', 0));
+	$graph->setRightPercentage(getRequest('percent_right', 0));
 
-	navigation_bar_calc();
-
-	$graph->setPeriod($_REQUEST['period']);
-	$graph->setSTime($_REQUEST['stime']);
-	$graph->setWidth(get_request('width', 900));
-	$graph->setHeight(get_request('height', 200));
-	$graph->showLegend(get_request('legend', 1));
-	$graph->showWorkPeriod(get_request('showworkperiod', 1));
-	$graph->showTriggers(get_request('showtriggers', 1));
-	$graph->setYMinAxisType(get_request('ymin_type', GRAPH_YAXIS_TYPE_CALCULATED));
-	$graph->setYMaxAxisType(get_request('ymax_type', GRAPH_YAXIS_TYPE_CALCULATED));
-	$graph->setYAxisMin(get_request('yaxismin', 0.00));
-	$graph->setYAxisMax(get_request('yaxismax', 100.00));
-	$graph->setYMinItemId(get_request('ymin_itemid', 0));
-	$graph->setYMaxItemId(get_request('ymax_itemid', 0));
-	$graph->setLeftPercentage(get_request('percent_left', 0));
-	$graph->setRightPercentage(get_request('percent_right', 0));
-
-	foreach ($items as $inum => $item) {
+	foreach ($items as $item) {
 		$graph->addItem(
 			$item['itemid'],
 			isset($item['yaxisside']) ? $item['yaxisside'] : null,
@@ -145,8 +163,8 @@ if ($isDataValid) {
 			isset($item['drawtype']) ? $item['drawtype'] : null,
 			isset($item['type']) ? $item['type'] : null
 		);
-		unset($items[$inum]);
 	}
+
 	$graph->draw();
 }
 
