@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -24,10 +24,9 @@ require_once dirname(__FILE__).'/include/hosts.inc.php';
 require_once dirname(__FILE__).'/include/graphs.inc.php';
 require_once dirname(__FILE__).'/include/forms.inc.php';
 
-$page['title'] = _('Configuration of graphs');
+$page['title'] = isset($_REQUEST['parent_discoveryid']) ? _('Configuration of graph prototypes') : _('Configuration of graphs');
 $page['file'] = 'graphs.php';
-$page['hist_arg'] = array();
-$page['scripts'] = array();
+$page['hist_arg'] = array('hostid', 'parent_discoveryid');
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -47,16 +46,16 @@ $fields = array(
 	'graphtype' =>			array(T_ZBX_INT, O_OPT, null,		IN('0,1,2,3'),	'isset({save})'),
 	'yaxismin' =>			array(T_ZBX_DBL, O_OPT, null,		null,			'isset({save})&&(({graphtype}==0)||({graphtype}==1))'),
 	'yaxismax' =>			array(T_ZBX_DBL, O_OPT, null,		null,			'isset({save})&&(({graphtype}==0)||({graphtype}==1))'),
-	'graph3d' =>			array(T_ZBX_INT, O_OPT, P_NZERO,	IN('0,1'),		null),
-	'legend' =>				array(T_ZBX_INT, O_OPT, P_NZERO,	IN('0,1'),		null),
+	'show_3d' =>			array(T_ZBX_INT, O_OPT, P_NZERO,	IN('0,1'),		null),
+	'show_legend' =>		array(T_ZBX_INT, O_OPT, P_NZERO,	IN('0,1'),		null),
 	'ymin_itemid' =>		array(T_ZBX_INT, O_OPT, null,		DB_ID,			'isset({save})&&isset({ymin_type})&&({ymin_type}==3)'),
 	'ymax_itemid' =>		array(T_ZBX_INT, O_OPT, null,		DB_ID,			'isset({save})&&isset({ymax_type})&&({ymax_type}==3)'),
-	'percent_left' =>		array(T_ZBX_DBL, O_OPT, null,		BETWEEN(0, 100), null, _('Percentile line (Left)')),
-	'percent_right' =>		array(T_ZBX_DBL, O_OPT, null,		BETWEEN(0, 100), null, _('Percentile line (Right)')),
+	'percent_left' =>		array(T_ZBX_DBL, O_OPT, null,		BETWEEN(0, 100), null, _('Percentile line (left)')),
+	'percent_right' =>		array(T_ZBX_DBL, O_OPT, null,		BETWEEN(0, 100), null, _('Percentile line (right)')),
 	'visible' =>			array(T_ZBX_INT, O_OPT, null,		BETWEEN(0, 1),	null),
 	'items' =>				array(T_ZBX_STR, O_OPT, null,		null,			null),
-	'showworkperiod' =>		array(T_ZBX_INT, O_OPT, null,		IN('1'),		null),
-	'showtriggers' =>		array(T_ZBX_INT, O_OPT, null,		IN('1'),		null),
+	'show_work_period' =>	array(T_ZBX_INT, O_OPT, null,		IN('1'),		null),
+	'show_triggers' =>		array(T_ZBX_INT, O_OPT, null,		IN('1'),		null),
 	'group_graphid' =>		array(T_ZBX_INT, O_OPT, null,		DB_ID,			null),
 	'copy_targetid' =>		array(T_ZBX_INT, O_OPT, null,		DB_ID,			null),
 	'filter_groupid' =>		array(T_ZBX_INT, O_OPT, P_SYS,		DB_ID,			'isset({copy})&&(isset({copy_type})&&({copy_type}==0))'),
@@ -83,14 +82,13 @@ if (isset($_REQUEST['yaxismin']) && zbx_empty($_REQUEST['yaxismin'])) {
 if (isset($_REQUEST['yaxismax']) && zbx_empty($_REQUEST['yaxismax'])) {
 	unset($_REQUEST['yaxismax']);
 }
-
 check_fields($fields);
 validate_sort_and_sortorder('name', ZBX_SORT_UP);
 
 $_REQUEST['go'] = get_request('go', 'none');
 $_REQUEST['items'] = get_request('items', array());
-$_REQUEST['graph3d'] = get_request('graph3d', 0);
-$_REQUEST['legend'] = get_request('legend', 0);
+$_REQUEST['show_3d'] = get_request('show_3d', 0);
+$_REQUEST['show_legend'] = get_request('show_legend', 0);
 
 /*
  * Permissions
@@ -119,7 +117,7 @@ if (CUser::$userData['type'] !== USER_TYPE_SUPER_ADMIN) {
 		if (isset($_REQUEST['graphid'])) {
 			$graphPrototype = API::GraphPrototype()->get(array(
 				'graphids' => array($_REQUEST['graphid']),
-				'output' => API_OUTPUT_SHORTEN,
+				'output' => array('graphid'),
 				'editable' => true,
 				'preservekeys' => true
 			));
@@ -160,92 +158,151 @@ if (CUser::$userData['type'] !== USER_TYPE_SUPER_ADMIN) {
  * Actions
  */
 if (isset($_REQUEST['clone']) && isset($_REQUEST['graphid'])) {
+	// graph
+	$options = array(
+		'graphids' => $_REQUEST['graphid'],
+		'output' => API_OUTPUT_EXTEND
+	);
+	$graph = empty($_REQUEST['parent_discoveryid'])
+		? API::Graph()->get($options)
+		: API::GraphPrototype()->get($options);
+	$graph = reset($graph);
+
+	$_REQUEST = array_merge($_REQUEST, $graph);
+
+	// graph items
+	$_REQUEST['items'] = API::GraphItem()->get(array(
+		'graphids' => $_REQUEST['graphid'],
+		'sortfield' => 'gitemid',
+		'output' => API_OUTPUT_EXTEND,
+		'expandData' => true
+	));
+
 	unset($_REQUEST['graphid']);
+
 	$_REQUEST['form'] = 'clone';
 }
-elseif (isset($_REQUEST['save'])) {
-	$result = true;
+elseif (hasRequest('save')) {
+	$items = getRequest('items', array());
 
-	$items = get_request('items', array());
-
-	if ($result) {
-		$graph = array(
-			'name' => $_REQUEST['name'],
-			'width' => $_REQUEST['width'],
-			'height' => $_REQUEST['height'],
-			'ymin_type' => get_request('ymin_type', 0),
-			'ymax_type' => get_request('ymax_type', 0),
-			'yaxismin' => get_request('yaxismin', 0),
-			'yaxismax' => get_request('yaxismax', 0),
-			'ymin_itemid' => $_REQUEST['ymin_itemid'],
-			'ymax_itemid' => $_REQUEST['ymax_itemid'],
-			'show_work_period' => get_request('showworkperiod', 0),
-			'show_triggers' => get_request('showtriggers', 0),
-			'graphtype' => $_REQUEST['graphtype'],
-			'show_legend' => get_request('legend', 1),
-			'show_3d' => get_request('graph3d', 0),
-			'percent_left' => get_request('percent_left', 0),
-			'percent_right' => get_request('percent_right', 0),
-			'gitems' => $items
-		);
-
-		if (!empty($_REQUEST['parent_discoveryid'])) {
-			$graph['flags'] = ZBX_FLAG_DISCOVERY_CHILD;
+	// remove passing "gitemid" to API if new items added via pop-up
+	foreach ($items as &$item) {
+		if (!$item['gitemid']) {
+			unset($item['gitemid']);
 		}
+	}
+	unset($item);
 
-		if (isset($_REQUEST['graphid'])) {
-			$graph['graphid'] = $_REQUEST['graphid'];
+	$graph = array(
+		'name' => $_REQUEST['name'],
+		'width' => $_REQUEST['width'],
+		'height' => $_REQUEST['height'],
+		'ymin_type' => get_request('ymin_type', 0),
+		'ymax_type' => get_request('ymax_type', 0),
+		'yaxismin' => get_request('yaxismin', 0),
+		'yaxismax' => get_request('yaxismax', 0),
+		'ymin_itemid' => $_REQUEST['ymin_itemid'],
+		'ymax_itemid' => $_REQUEST['ymax_itemid'],
+		'show_work_period' => get_request('show_work_period', 0),
+		'show_triggers' => get_request('show_triggers', 0),
+		'graphtype' => $_REQUEST['graphtype'],
+		'show_legend' => get_request('show_legend', 1),
+		'show_3d' => get_request('show_3d', 0),
+		'percent_left' => get_request('percent_left', 0),
+		'percent_right' => get_request('percent_right', 0),
+		'gitems' => $items
+	);
 
-			$result = !empty($_REQUEST['parent_discoveryid'])
-				? API::GraphPrototype()->update($graph)
-				: API::Graph()->update($graph);
+	// create and update graph prototypes
+	if (hasRequest('parent_discoveryid')) {
+		$graph['flags'] = ZBX_FLAG_DISCOVERY_PROTOTYPE;
+
+		if (hasRequest('graphid')) {
+			$graph['graphid'] = getRequest('graphid');
+			$result = API::GraphPrototype()->update($graph);
+
+			show_messages($result, _('Graph prototype updated'), _('Cannot update graph prototype'));
 
-			if ($result) {
-				add_audit(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_GRAPH, 'Graph ID ['.$_REQUEST['graphid'].'] Graph ['.$_REQUEST['name'].']');
-			}
 		}
 		else {
-			$result = !empty($_REQUEST['parent_discoveryid'])
-				? API::GraphPrototype()->create($graph)
-				: API::Graph()->create($graph);
+			$result = API::GraphPrototype()->create($graph);
 
-			if ($result) {
-				add_audit(AUDIT_ACTION_ADD, AUDIT_RESOURCE_GRAPH, 'Graph ['.$_REQUEST['name'].']');
-			}
+			show_messages($result, _('Graph prototype added'), _('Cannot add graph prototype'));
 		}
 
-		if ($result) {
-			unset($_REQUEST['form']);
-		}
-	}
-
-	if (isset($_REQUEST['graphid'])) {
-		show_messages($result, _('Graph updated'), _('Cannot update graph'));
+		clearCookies($result, getRequest('parent_discoveryid'));
 	}
+	// create and update graphs
 	else {
-		show_messages($result, _('Graph added'), _('Cannot add graph'));
+		if (hasRequest('graphid')) {
+			$graph['graphid'] = getRequest('graphid');
+			$result = API::Graph()->update($graph);
+
+			show_messages($result, _('Graph updated'), _('Cannot update graph'));
+		}
+		else {
+			$result = API::Graph()->create($graph);
+
+			show_messages($result, _('Graph added'), _('Cannot add graph'));
+		}
+
+		clearCookies($result, getRequest('hostid'));
+	}
+
+	if ($result) {
+		if (hasRequest('graphid')) {
+			add_audit(
+				AUDIT_ACTION_UPDATE,
+				AUDIT_RESOURCE_GRAPH,
+				'Graph ID ['.$graph['graphid'].'] Graph ['.getRequest('name').']'
+			);
+		}
+		else {
+			add_audit(AUDIT_ACTION_ADD, AUDIT_RESOURCE_GRAPH, 'Graph ['.getRequest('name').']');
+		}
+
+		unset($_REQUEST['form']);
 	}
 }
-elseif (isset($_REQUEST['delete']) && isset($_REQUEST['graphid'])) {
-	$result = !empty($_REQUEST['parent_discoveryid'])
-		? API::GraphPrototype()->delete($_REQUEST['graphid'])
-		: API::Graph()->delete($_REQUEST['graphid']);
+elseif (hasRequest('delete') && hasRequest('graphid')) {
+	$graphId = getRequest('graphid');
+
+	if (hasRequest('parent_discoveryid')) {
+		$result = API::GraphPrototype()->delete($graphId);
+
+		show_messages($result, _('Graph prototype deleted'), _('Cannot delete graph prototype'));
+		clearCookies($result, getRequest('parent_discoveryid'));
+	}
+	else {
+		$result = API::Graph()->delete($graphId);
+
+		show_messages($result, _('Graph deleted'), _('Cannot delete graph'));
+		clearCookies($result, getRequest('hostid'));
+	}
 
 	if ($result) {
 		unset($_REQUEST['form']);
 	}
-	show_messages($result, _('Graph deleted'), _('Cannot delete graph'));
 }
-elseif ($_REQUEST['go'] == 'delete' && isset($_REQUEST['group_graphid'])) {
-	$go_result = !empty($_REQUEST['parent_discoveryid'])
-		? API::GraphPrototype()->delete($_REQUEST['group_graphid'])
-		: API::Graph()->delete($_REQUEST['group_graphid']);
+elseif (getRequest('go') == 'delete' && hasRequest('group_graphid')) {
+	$graphIds = getRequest('group_graphid');
 
-	show_messages($go_result, _('Graphs deleted'), _('Cannot delete graphs'));
+	if (hasRequest('parent_discoveryid')) {
+		$result = API::GraphPrototype()->delete($graphIds);
+
+		show_messages($result, _('Graph prototypes deleted'), _('Cannot delete graph prototypes'));
+		clearCookies($result, getRequest('parent_discoveryid'));
+	}
+	else {
+		$result = API::Graph()->delete($graphIds);
+
+		show_messages($result, _('Graphs deleted'), _('Cannot delete graphs'));
+		clearCookies($result, getRequest('hostid'));
+	}
 }
 elseif ($_REQUEST['go'] == 'copy_to' && isset($_REQUEST['copy']) && isset($_REQUEST['group_graphid'])) {
 	if (!empty($_REQUEST['copy_targetid']) && isset($_REQUEST['copy_type'])) {
-		$go_result = true;
+		$goResult = true;
 
 		$options = array(
 			'editable' => true,
@@ -282,12 +339,16 @@ elseif ($_REQUEST['go'] == 'copy_to' && isset($_REQUEST['copy']) && isset($_REQU
 		DBstart();
 		foreach ($_REQUEST['group_graphid'] as $graphid) {
 			foreach ($dbHosts as $host) {
-				$go_result &= (bool) copy_graph_to_host($graphid, $host['hostid']);
+				$goResult &= (bool) copyGraphToHost($graphid, $host['hostid']);
 			}
 		}
-		$go_result = DBend($go_result);
+		$goResult = DBend($goResult);
+
+		show_messages($goResult, _('Graphs copied'), _('Cannot copy graphs'));
+		clearCookies($goResult,
+			empty($_REQUEST['parent_discoveryid']) ? $_REQUEST['hostid'] : $_REQUEST['parent_discoveryid']
+		);
 
-		show_messages($go_result, _('Graphs copied'), _('Cannot copy graphs'));
 		$_REQUEST['go'] = 'none2';
 	}
 	else {
@@ -295,11 +356,6 @@ elseif ($_REQUEST['go'] == 'copy_to' && isset($_REQUEST['copy']) && isset($_REQU
 	}
 	show_messages();
 }
-if ($_REQUEST['go'] != 'none' && isset($go_result) && $go_result) {
-	$url = new CUrl();
-	$path = $url->getPath();
-	insert_js('cookie.eraseArray("'.$path.'")');
-}
 
 /*
  * Display
@@ -316,9 +372,14 @@ $pageFilter = new CPageFilter(array(
 	'groupid' => get_request('groupid', null),
 	'hostid' => get_request('hostid', null)
 ));
+
 if (empty($_REQUEST['parent_discoveryid'])) {
-	$_REQUEST['groupid'] = $pageFilter->groupid;
-	$_REQUEST['hostid'] = $pageFilter->hostid;
+	if ($pageFilter->groupid > 0) {
+		$_REQUEST['groupid'] = $pageFilter->groupid;
+	}
+	if ($pageFilter->hostid > 0) {
+		$_REQUEST['hostid'] = $pageFilter->hostid;
+	}
 }
 
 if ($_REQUEST['go'] == 'copy_to' && isset($_REQUEST['group_graphid'])) {
@@ -335,7 +396,6 @@ elseif (isset($_REQUEST['form'])) {
 		'parent_discoveryid' => get_request('parent_discoveryid'),
 		'group_gid' => get_request('group_gid', array()),
 		'hostid' => get_request('hostid', 0),
-		'is_template' => isTemplate(get_request('hostid', 0)),
 		'normal_only' => get_request('normal_only')
 	);
 
@@ -345,7 +405,7 @@ elseif (isset($_REQUEST['form'])) {
 			'output' => API_OUTPUT_EXTEND,
 			'selectHosts' => array('hostid')
 		);
-		$graph = !empty($data['parent_discoveryid']) ? API::GraphPrototype()->get($options) : API::Graph()->get($options);
+		$graph = empty($data['parent_discoveryid']) ? API::Graph()->get($options) : API::GraphPrototype()->get($options);
 		$graph = reset($graph);
 
 		$data['name'] = $graph['name'];
@@ -357,18 +417,18 @@ elseif (isset($_REQUEST['form'])) {
 		$data['yaxismax'] = $graph['yaxismax'];
 		$data['ymin_itemid'] = $graph['ymin_itemid'];
 		$data['ymax_itemid'] = $graph['ymax_itemid'];
-		$data['showworkperiod'] = $graph['show_work_period'];
-		$data['showtriggers'] = $graph['show_triggers'];
+		$data['show_work_period'] = $graph['show_work_period'];
+		$data['show_triggers'] = $graph['show_triggers'];
 		$data['graphtype'] = $graph['graphtype'];
-		$data['legend'] = $graph['show_legend'];
-		$data['graph3d'] = $graph['show_3d'];
+		$data['show_legend'] = $graph['show_legend'];
+		$data['show_3d'] = $graph['show_3d'];
 		$data['percent_left'] = $graph['percent_left'];
 		$data['percent_right'] = $graph['percent_right'];
 		$data['templateid'] = $graph['templateid'];
 		$data['templates'] = array();
 
 		// if no host has been selected for the navigation panel, use the first graph host
-		if (!$data['hostid']) {
+		if (empty($data['hostid'])) {
 			$host = reset($graph['hosts']);
 			$data['hostid'] = $host['hostid'];
 		}
@@ -391,7 +451,7 @@ elseif (isset($_REQUEST['form'])) {
 						$parentTemplate = reset($parentGraphPrototype['templates']);
 
 						$link = new CLink($parentTemplate['name'],
-							'graphs.php?form=update&graphid='.$parentGraphPrototype['graphid'].'&hostid='.$parentTemplate['hostid'].'&parent_discoveryid='.$parentGraphPrototype['discoveryRule']['itemid']
+							'graphs.php?form=update&graphid='.$parentGraphPrototype['graphid'].'&hostid='.$parentTemplate['templateid'].'&parent_discoveryid='.$parentGraphPrototype['discoveryRule']['itemid']
 						);
 					}
 				}
@@ -441,10 +501,10 @@ elseif (isset($_REQUEST['form'])) {
 		$data['yaxismax'] = get_request('yaxismax', '100.00');
 		$data['ymin_itemid'] = get_request('ymin_itemid', 0);
 		$data['ymax_itemid'] = get_request('ymax_itemid', 0);
-		$data['showworkperiod'] = get_request('showworkperiod', 0);
-		$data['showtriggers'] = get_request('showtriggers', 0);
-		$data['legend'] = get_request('legend', 0);
-		$data['graph3d'] = get_request('graph3d', 0);
+		$data['show_work_period'] = get_request('show_work_period', 0);
+		$data['show_triggers'] = get_request('show_triggers', 0);
+		$data['show_legend'] = get_request('show_legend', 0);
+		$data['show_3d'] = get_request('show_3d', 0);
 		$data['visible'] = get_request('visible');
 		$data['percent_left'] = 0;
 		$data['percent_right'] = 0;
@@ -460,9 +520,9 @@ elseif (isset($_REQUEST['form'])) {
 	}
 
 	if (empty($data['graphid']) && !isset($_REQUEST['form_refresh'])) {
-		$data['legend'] = $_REQUEST['legend'] = 1;
-		$data['showworkperiod'] = $_REQUEST['showworkperiod'] = 1;
-		$data['showtriggers'] = $_REQUEST['showtriggers'] = 1;
+		$data['show_legend'] = $_REQUEST['show_legend'] = 1;
+		$data['show_work_period'] = $_REQUEST['show_work_period'] = 1;
+		$data['show_triggers'] = $_REQUEST['show_triggers'] = 1;
 	}
 
 	$_REQUEST['items'] = $data['items'];
@@ -475,11 +535,11 @@ elseif (isset($_REQUEST['form'])) {
 	$_REQUEST['yaxismax'] = $data['yaxismax'];
 	$_REQUEST['ymin_itemid'] = $data['ymin_itemid'];
 	$_REQUEST['ymax_itemid'] = $data['ymax_itemid'];
-	$_REQUEST['showworkperiod'] = $data['showworkperiod'];
-	$_REQUEST['showtriggers'] = $data['showtriggers'];
+	$_REQUEST['show_work_period'] = $data['show_work_period'];
+	$_REQUEST['show_triggers'] = $data['show_triggers'];
 	$_REQUEST['graphtype'] = $data['graphtype'];
-	$_REQUEST['legend'] = $data['legend'];
-	$_REQUEST['graph3d'] = $data['graph3d'];
+	$_REQUEST['show_legend'] = $data['show_legend'];
+	$_REQUEST['show_3d'] = $data['show_3d'];
 	$_REQUEST['percent_left'] = $data['percent_left'];
 	$_REQUEST['percent_right'] = $data['percent_right'];
 
@@ -505,76 +565,82 @@ elseif (isset($_REQUEST['form'])) {
 	asort_by_key($data['items'], 'sortorder');
 	$data['items'] = array_values($data['items']);
 
+	// is template
+	$data['is_template'] = isTemplate($data['hostid']);
+
 	// render view
 	$graphView = new CView('configuration.graph.edit', $data);
 	$graphView->render();
 	$graphView->show();
 }
 else {
-	if (isset($_REQUEST['graphid']) && $_REQUEST['graphid'] == 0) {
-		unset($_REQUEST['graphid']);
-	}
-
 	$data = array(
 		'pageFilter' => $pageFilter,
-		'hostid' => get_request('hostid'),
+		'hostid' => ($pageFilter->hostid > 0) ? $pageFilter->hostid : get_request('hostid'),
 		'parent_discoveryid' => get_request('parent_discoveryid'),
-		'graphs' => array()
+		'graphs' => array(),
+		'discovery_rule' => empty($_REQUEST['parent_discoveryid']) ? null : $discovery_rule,
+		'displayNodes' => (is_array(get_current_nodeid()) && $pageFilter->groupid == 0 && $pageFilter->hostid == 0)
 	);
-	if (!empty($_REQUEST['parent_discoveryid'])) {
-		$data['discovery_rule'] = $discovery_rule;
-	}
 
 	$sortfield = getPageSortField('name');
 	$sortorder = getPageSortOrder();
 
-	if ($pageFilter->hostsSelected) {
-		$options = array(
-			'editable' => true,
-			'output' => array('graphid', 'name', 'graphtype'),
-			'limit' => $config['search_limit'] + 1
-		);
-		// get real graphs
-		if (empty($_REQUEST['parent_discoveryid'])) {
-			if ($pageFilter->hostid > 0) {
-				$options['hostids'] = $pageFilter->hostid;
-			}
-			elseif ($pageFilter->groupid > 0) {
-				$options['groupids'] = $pageFilter->groupid;
-			}
-			$data['graphs'] = API::Graph()->get($options);
-		}
-		// get graph prototypes
-		else {
-			$options['discoveryids'] = $_REQUEST['parent_discoveryid'];
-			$data['graphs'] = API::GraphPrototype()->get($options);
-		}
-	}
+	// get graphs
+	$options = array(
+		'hostids' => $data['hostid'] ? $data['hostid'] : null,
+		'groupids' => (!$data['hostid'] && $pageFilter->groupid > 0) ? $pageFilter->groupid : null,
+		'discoveryids' => empty($_REQUEST['parent_discoveryid']) ? null : get_request('parent_discoveryid'),
+		'editable' => true,
+		'output' => array('graphid', 'name', 'graphtype'),
+		'limit' => $config['search_limit'] + 1
+	);
+
+	$data['graphs'] = empty($_REQUEST['parent_discoveryid'])
+		? API::Graph()->get($options)
+		: API::GraphPrototype()->get($options);
 
 	if ($sortfield == 'graphtype') {
 		foreach ($data['graphs'] as $gnum => $graph) {
 			$data['graphs'][$gnum]['graphtype'] = graphType($graph['graphtype']);
 		}
 	}
-	order_result($data['graphs'], $sortfield, $sortorder);
-	$data['paging'] = getPagingLine($data['graphs']);
 
+	order_result($data['graphs'], $sortfield, $sortorder);
+
+	$data['paging'] = getPagingLine(
+		$data['graphs'],
+		array('graphid'),
+		array(
+			'hostid' => get_request('hostid'),
+			'parent_discoveryid' => get_request('parent_discoveryid')
+		)
+	);
+
+	// get graphs after paging
 	$options = array(
 		'graphids' => zbx_objectValues($data['graphs'], 'graphid'),
 		'output' => array('graphid', 'name', 'templateid', 'graphtype', 'width', 'height'),
 		'selectDiscoveryRule' => array('itemid', 'name'),
+		'selectHosts' => $data['hostid'] ? null : array('name'),
+		'selectTemplates' => $data['hostid'] ? null : array('name')
 	);
-	if ($pageFilter->hostid == 0) {
-		$options['selectHosts'] = array('name');
-		$options['selectTemplates'] = array('name');
-	}
-	$data['graphs'] = !empty($_REQUEST['parent_discoveryid'])
-		? API::GraphPrototype()->get($options)
-		: API::Graph()->get($options);
+
+	$data['graphs'] = empty($_REQUEST['parent_discoveryid'])
+		? API::Graph()->get($options)
+		: API::GraphPrototype()->get($options);
 
 	foreach ($data['graphs'] as $gnum => $graph) {
 		$data['graphs'][$gnum]['graphtype'] = graphType($graph['graphtype']);
 	}
+
+	// nodes
+	if ($data['displayNodes']) {
+		foreach ($data['graphs'] as $key => $graph) {
+			$data['graphs'][$key]['nodename'] = get_node_name_by_elid($graph['graphid'], true);
+		}
+	}
+
 	order_result($data['graphs'], $sortfield, $sortorder);
 
 	// render view
