@@ -58,10 +58,6 @@ check_fields($fields);
  * Ajax
  */
 if (isset($_REQUEST['favobj'])) {
-	if ($_REQUEST['favobj'] == 'hat') {
-		CProfile::update('web.hostscreen.hats.'.$_REQUEST['favref'].'.state', $_REQUEST['favstate'], PROFILE_TYPE_INT);
-	}
-
 	if ($_REQUEST['favobj'] == 'filter') {
 		CProfile::update('web.hostscreen.filter.state', $_REQUEST['favstate'], PROFILE_TYPE_INT);
 	}
@@ -75,14 +71,14 @@ if (isset($_REQUEST['favobj'])) {
 	if (str_in_array($_REQUEST['favobj'], array('screenid', 'slideshowid'))) {
 		$result = false;
 		if ($_REQUEST['favaction'] == 'add') {
-			$result = add2favorites('web.favorite.screenids',$_REQUEST['favid'], $_REQUEST['favobj']);
+			$result = CFavorite::add('web.favorite.screenids',$_REQUEST['favid'], $_REQUEST['favobj']);
 			if ($result) {
 				echo '$("addrm_fav").title = "'._('Remove from favourites').'";'."\n";
 				echo '$("addrm_fav").onclick = function() { rm4favorites("'.$_REQUEST['favobj'].'", "'.$_REQUEST['favid'].'", 0); }'."\n";
 			}
 		}
 		elseif ($_REQUEST['favaction'] == 'remove') {
-			$result = rm4favorites('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
+			$result = CFavorite::remove('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
 
 			if ($result) {
 				echo '$("addrm_fav").title = "'._('Add to favourites').'";'."\n";
@@ -106,7 +102,7 @@ if ($page['type'] == PAGE_TYPE_JS || $page['type'] == PAGE_TYPE_HTML_BLOCK) {
  */
 $data = array(
 	'hostid' => get_request('hostid', 0),
-	'fullscreen' => get_request('fullscreen', 0),
+	'fullscreen' => $_REQUEST['fullscreen'],
 	'screenid' => get_request('screenid', CProfile::get('web.hostscreen.screenid', null)),
 	'period' => get_request('period'),
 	'stime' => get_request('stime')
