@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -18,6 +18,7 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+
 class API {
 
 	const RETURN_TYPE_API = 'api';
@@ -49,6 +50,7 @@ class API {
 		'graphprototype' => 'CGraphPrototype',
 		'host' => 'CHost',
 		'hostgroup' => 'CHostGroup',
+		'hostprototype' => 'CHostPrototype',
 		'history' => 'CHistory',
 		'hostinterface' => 'CHostInterface',
 		'image' => 'CImage',
@@ -72,7 +74,8 @@ class API {
 		'usergroup' => 'CUserGroup',
 		'usermacro' => 'CUserMacro',
 		'usermedia' => 'CUserMedia',
-		'webcheck' => 'CWebCheck'
+		'httptest' => 'CHttpTest',
+		'webcheck' => 'CHttpTest'
 	);
 
 	/**
@@ -133,7 +136,7 @@ class API {
 	}
 
 	public static function getObject($className) {
-		return self::$return == self::RETURN_TYPE_API ? self::getApi($className) : self::getRpc($className);
+		return (self::$return == self::RETURN_TYPE_API) ? self::getApi($className) : self::getRpc($className);
 	}
 
 	/**
@@ -248,6 +251,13 @@ class API {
 		return self::getObject('host');
 	}
 
+	/**
+	 * @return CHostPrototype
+	 */
+	public static function HostPrototype() {
+		return self::getObject('hostprototype');
+	}
+
 	/**
 	 * @return CHostGroup
 	 */
@@ -410,9 +420,9 @@ class API {
 	}
 
 	/**
-	 * @return CWebCheck
+	 * @return CHttpTest
 	 */
-	public static function WebCheck() {
-		return self::getObject('webcheck');
+	public static function HttpTest() {
+		return self::getObject('httptest');
 	}
 }
