@@ -1,7 +1,7 @@
 <?php
 /*
-** ZABBIX
-** Copyright (C) 2000-2009 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,30 +10,23 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
-class CColorCell extends CDiv{
-	public function __construct($name, $value, $action=null){
+
+
+class CColorCell extends CDiv {
+
+	public function __construct($name, $value, $action = null) {
 		parent::__construct(SPACE.SPACE.SPACE, 'pointer');
 		$this->setName($name);
-		$this->setAttribute('id', $name);
-		$this->setAttribute('title', '#'.$value);
-		$this->setAttribute('style', 'display: inline; width: 10px; height: 10px; text-decoration: none; border: 1px solid black; background-color: #'.$value);
-
-		$this->setAction($action);
-	}
-
-	public function setAction($action=null){
-		if(!isset($action)) return false;
-
-		return $this->addAction('onclick', $action);
+		$this->attr('id', zbx_formatDomId($name));
+		$this->attr('title', '#'.$value);
+		$this->attr('style', 'display: inline; width: 10px; height: 10px; text-decoration: none; border: 1px solid black; background-color: #'.$value);
+		$this->attr('onclick', $action);
 	}
 }
-?>
