@@ -1,7 +1,7 @@
 <?php
 /*
-** ZABBIX
-** Copyright (C) 2000-2009 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,26 +10,26 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
-class CSpan extends CTag{
-	public function __construct($items=NULL,$class=NULL){
-		parent::__construct('span','yes');
-		$this->setClass($class);
-		$this->addItem($items);
 
+
+class CSpan extends CTag {
+
+	public function __construct($items = null, $class = null, $id = null) {
+		parent::__construct('span', 'yes');
+		$this->attr('class', $class);
+		$this->attr('id', $id);
+		$this->addItem($items);
 		$this->tag_body_start = '';
-		$this->tag_start= '';
+		$this->tag_start = '';
 		$this->tag_end = '';
 		$this->tag_body_start = '';
 		$this->tag_body_end = '';
 	}
 }
-?>
