@@ -21,7 +21,7 @@
 
 class DB {
 
-	const SCHEMA_FILE = 'include/schema.inc.php';
+	const SCHEMA_FILE = 'schema.inc.php';
 
 	const DBEXECUTE_ERROR = 1;
 	const RESERVEIDS_ERROR = 2;
@@ -44,6 +44,42 @@ class DB {
 	private static $maxNodeId = null;
 	private static $minNodeId = null;
 
+	/**
+	 * @var DbBackend
+	 */
+	private static $dbBackend;
+
+	/**
+	 * Get necessary DB class.
+	 *
+	 * @return DbBackend
+	 */
+	public static function getDbBackend() {
+		global $DB;
+
+		if (!self::$dbBackend) {
+			switch ($DB['TYPE']) {
+				case ZBX_DB_MYSQL:
+					self::$dbBackend = new MysqlDbBackend();
+					break;
+				case ZBX_DB_POSTGRESQL:
+					self::$dbBackend = new PostgresqlDbBackend();
+					break;
+				case ZBX_DB_ORACLE:
+					self::$dbBackend = new OracleDbBackend();
+					break;
+				case ZBX_DB_DB2:
+					self::$dbBackend = new Db2DbBackend();
+					break;
+				case ZBX_DB_SQLITE3:
+					self::$dbBackend = new SqliteDbBackend();
+					break;
+			}
+		}
+
+		return self::$dbBackend;
+	}
+
 	private static function exception($code, $error) {
 		throw new DBException($error, $code);
 	}
@@ -69,6 +105,7 @@ class DB {
 	 * using maximum id from table or minimum allowed value.
 	 *
 	 * @throw APIException
+	 *
 	 * @static
 	 *
 	 * @param string $table table name
@@ -116,6 +153,7 @@ class DB {
 		else {
 			$nextid = self::refreshIds($table, $count);
 		}
+
 		return $nextid;
 	}
 
@@ -165,6 +203,7 @@ class DB {
 		}
 
 		$nextid = bcadd($nextid, 1, 0);
+
 		return $nextid;
 	}
 
@@ -184,8 +223,7 @@ class DB {
 	 */
 	public static function getSchema($table = null) {
 		if (is_null(self::$schema)) {
-
-			self::$schema = include(Z::getRootDir().'/'.self::SCHEMA_FILE);
+			self::$schema = include(dirname(__FILE__).'/../../'.self::SCHEMA_FILE);
 		}
 
 		if (is_null($table)) {
@@ -271,27 +309,23 @@ class DB {
 				continue;
 			}
 
+			if (isset($tableSchema['fields'][$field]['ref_table'])) {
+				if ($tableSchema['fields'][$field]['null']) {
+					$values[$field] = ($values[$field] == '0') ? NULL : $values[$field];
+				}
+			}
+
 			if (is_null($values[$field])) {
 				if ($tableSchema['fields'][$field]['null']) {
 					$values[$field] = 'NULL';
 				}
 				elseif (isset($tableSchema['fields'][$field]['default'])) {
-					$values[$field] = $tableSchema['fields'][$field]['default'];
+					$values[$field] = zbx_dbstr($tableSchema['fields'][$field]['default']);
 				}
 				else {
-					self::exception(self::DBEXECUTE_ERROR, _s('Mandatory field "%1$s" is missing in table "%2$s".', $field, $table));
-				}
-			}
-
-			if (isset($tableSchema['fields'][$field]['ref_table'])) {
-				if ($tableSchema['fields'][$field]['null']) {
-					$values[$field] = zero2null($values[$field]);
-				}
-			}
-
-			if ($values[$field] === 'NULL') {
-				if (!$tableSchema['fields'][$field]['null']) {
-					self::exception(self::DBEXECUTE_ERROR, _s('Incorrect value "NULL" for NOT NULL field "%1$s".', $field));
+					self::exception(self::DBEXECUTE_ERROR,
+						_s('Field "%1$s" cannot be set to NULL.', $field)
+					);
 				}
 			}
 			else {
@@ -373,7 +407,7 @@ class DB {
 	}
 
 	/**
-	 * Insert data into DB
+	 * Insert data into DB.
 	 *
 	 * @param string $table
 	 * @param array  $values pair of fieldname => fieldvalue
@@ -385,6 +419,7 @@ class DB {
 		if (empty($values)) {
 			return true;
 		}
+
 		$resultIds = array();
 
 		if ($getids) {
@@ -411,22 +446,71 @@ class DB {
 				self::exception(self::DBEXECUTE_ERROR, _s('SQL statement execution has failed "%1$s".', $sql));
 			}
 		}
+
 		return $resultIds;
 	}
 
 	/**
-	 * Update data in DB
+	 * Insert batch data into DB.
+	 *
+	 * @param string $table
+	 * @param array  $values pair of fieldname => fieldvalue
+	 * @param bool   $getids
+	 *
+	 * @return array    an array of ids with the keys preserved
+	 */
+	public static function insertBatch($table, $values, $getids = true) {
+		if (empty($values)) {
+			return true;
+		}
+
+		$resultIds = array();
+
+		$tableSchema = self::getSchema($table);
+		$values = self::addMissingFields($tableSchema, $values);
+		$fields = array_keys(reset($values));
+
+		if ($getids) {
+			$id = self::reserveIds($table, count($values));
+			$fields[] = $tableSchema['key'];
+		}
+
+		$newValues = array();
+		foreach ($values as $key => $row) {
+			if ($getids) {
+				$resultIds[$key] = $id;
+				$row[$tableSchema['key']] = $id;
+				$values[$key][$tableSchema['key']] = $id;
+				$id = bcadd($id, 1, 0);
+			}
+			self::checkValueTypes($table, $row);
+			$newValues[] = $row;
+		}
+
+		$sql = self::getDbBackend()->createInsertQuery($table, $fields, $newValues);
+
+		if (!DBexecute($sql)) {
+			self::exception(self::DBEXECUTE_ERROR, _s('SQL statement execution has failed "%1$s".', $sql));
+		}
+
+		return $resultIds;
+	}
+
+	/**
+	 * Update data in DB.
 	 *
 	 * @param string $table
 	 * @param array $data
 	 * @param array $data[...]['values'] pair of fieldname => fieldvalue for SET clause
 	 * @param array $data[...]['where'] pair of fieldname => fieldvalue for WHERE clause
+	 *
 	 * @return array of ids
 	 */
 	public static function update($table, $data) {
 		if (empty($data)) {
 			return true;
 		}
+
 		$tableSchema = self::getSchema($table);
 
 		$data = zbx_toArray($data);
@@ -466,6 +550,7 @@ class DB {
 				self::exception(self::DBEXECUTE_ERROR, _s('SQL statement execution has failed "%1$s".', $sql));
 			}
 		}
+
 		return true;
 	}
 
@@ -594,7 +679,7 @@ class DB {
 	}
 
 	/**
-	 * Delete data from DB
+	 * Delete data from DB.
 	 *
 	 * Example:
 	 * DB::delete('applications', array('applicationid'=>array(1, 8, 6)));
@@ -630,6 +715,7 @@ class DB {
 		if (!DBexecute($sql)) {
 			self::exception(self::DBEXECUTE_ERROR, _s('SQL statement execution has failed "%1$s"', $sql));
 		}
+
 		return true;
 	}
 
@@ -647,6 +733,7 @@ class DB {
 			case self::FIELD_TYPE_UINT:
 				return true;
 		}
+
 		return false;
 	}
 }
