@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -195,6 +195,7 @@ class CConfigurationImport {
 		$screensRefs = array();
 		$macrosRefs = array();
 		$proxyRefs = array();
+		$hostPrototypeRefs = array();
 
 		foreach ($this->getFormattedGroups() as $group) {
 			$groupsRefs[$group['name']] = $group['name'];
@@ -292,6 +293,20 @@ class CConfigurationImport {
 						$itemsRefs[$gitemItem['host']][$gitemItem['key']] = $gitemItem['key'];
 					}
 				}
+
+				foreach ($discoveryRule['host_prototypes'] as $hostPrototype) {
+					$hostPrototypeRefs[$host][$discoveryRule['key_']][$hostPrototype['host']] = $hostPrototype['host'];
+
+					foreach ($hostPrototype['group_prototypes'] as $groupPrototype) {
+						if (isset($groupPrototype['group'])) {
+							$groupsRefs[$groupPrototype['group']['name']] = $groupPrototype['group']['name'];
+						}
+					}
+
+					foreach ($hostPrototype['templates'] as $template) {
+						$templatesRefs[$template['name']] = $template['name'];
+					}
+				}
 			}
 		}
 
@@ -442,6 +457,7 @@ class CConfigurationImport {
 		$this->referencer->addScreens($screensRefs);
 		$this->referencer->addMacros($macrosRefs);
 		$this->referencer->addProxies($proxyRefs);
+		$this->referencer->addHostPrototypes($hostPrototypeRefs);
 	}
 
 	/**
@@ -570,8 +586,17 @@ class CConfigurationImport {
 					$item['interfaceid'] = $this->referencer->interfacesCache[$hostid][$item['interface_ref']];
 				}
 
-				if (!empty($item['valuemap'])) {
-					$item['valuemapid'] = $this->referencer->resolveValueMap($item['valuemap']['name']);
+				if ($item['valuemap']) {
+					$valueMapId = $this->referencer->resolveValueMap($item['valuemap']['name']);
+					if (!$valueMapId) {
+						throw new Exception(_s(
+							'Cannot find value map "%1$s" used for item "%2$s" on "%3$s".',
+							$item['valuemap']['name'],
+							$item['name'],
+							$host
+						));
+					}
+					$item['valuemapid'] = $valueMapId;
 				}
 
 				$itemsId = $this->referencer->resolveItem($hostid, $item['key_']);
@@ -632,6 +657,7 @@ class CConfigurationImport {
 				unset($item['item_prototypes']);
 				unset($item['trigger_prototypes']);
 				unset($item['graph_prototypes']);
+				unset($item['host_prototypes']);
 
 				$itemId = $this->referencer->resolveItem($hostid, $item['key_']);
 				if ($itemId) {
@@ -670,6 +696,8 @@ class CConfigurationImport {
 		// process prototypes
 		$prototypesToUpdate = array();
 		$prototypesToCreate = array();
+		$hostPrototypesToUpdate = array();
+		$hostPrototypesToCreate = array();
 		foreach ($allDiscoveryRules as $host => $discoveryRules) {
 			$hostid = $this->referencer->resolveHostOrTemplate($host);
 			foreach ($discoveryRules as $item) {
@@ -679,6 +707,7 @@ class CConfigurationImport {
 				}
 
 				$item['hostid'] = $hostid;
+				$itemId = $this->referencer->resolveItem($hostid, $item['key_']);
 
 				// prototypes
 				foreach ($item['item_prototypes'] as $prototype) {
@@ -694,8 +723,18 @@ class CConfigurationImport {
 						$prototype['interfaceid'] = $this->referencer->interfacesCache[$hostid][$prototype['interface_ref']];
 					}
 
-					if (!empty($prototype['valuemap'])) {
-						$prototype['valuemapid'] = $this->referencer->resolveValueMap($prototype['valuemap']['name']);
+					if ($prototype['valuemap']) {
+						$valueMapId = $this->referencer->resolveValueMap($prototype['valuemap']['name']);
+						if (!$valueMapId) {
+							throw new Exception(_s(
+								'Cannot find value map "%1$s" used for item prototype "%2$s" of discovery rule "%3$s" on "%4$s".',
+								$prototype['valuemap']['name'],
+								$prototype['name'],
+								$item['name'],
+								$host
+							));
+						}
+						$prototype['valuemapid'] = $valueMapId;
 					}
 
 					$prototypeId = $this->referencer->resolveItem($hostid, $prototype['key_']);
@@ -709,6 +748,54 @@ class CConfigurationImport {
 					}
 				}
 
+				// host prototype
+				foreach ($item['host_prototypes'] as $hostPrototype) {
+					// resolve group prototypes
+					$groupLinks = array();
+					foreach ($hostPrototype['group_links'] as $groupLink) {
+						$groupId = $this->referencer->resolveGroup($groupLink['group']['name']);
+						if (!$groupId) {
+							throw new Exception(_s(
+								'Cannot find host group "%1$s" for host prototype "%2$s" of discovery rule "%3$s" on "%4$s".',
+								$groupLink['group']['name'],
+								$hostPrototype['name'],
+								$item['name'],
+								$host
+							));
+						}
+						$groupLinks[] = array('groupid' => $groupId);
+					}
+					$hostPrototype['groupLinks'] = $groupLinks;
+					$hostPrototype['groupPrototypes'] = $hostPrototype['group_prototypes'];
+					unset($hostPrototype['group_links'], $hostPrototype['group_prototypes']);
+
+					// resolve templates
+					$templates = array();
+					foreach ($hostPrototype['templates'] as $template) {
+						$templateId = $this->referencer->resolveTemplate($template['name']);
+						if (!$templateId) {
+							throw new Exception(_s(
+								'Cannot find template "%1$s" for host prototype "%2$s" of discovery rule "%3$s" on "%4$s".',
+								$template['name'],
+								$hostPrototype['name'],
+								$item['name'],
+								$host
+							));
+						}
+						$templates[] = array('templateid' => $templateId);
+					}
+					$hostPrototype['templates'] = $templates;
+
+					$hostPrototypeId = $this->referencer->resolveHostPrototype($hostid, $itemId, $hostPrototype['host']);
+					if ($hostPrototypeId) {
+						$hostPrototype['hostid'] = $hostPrototypeId;
+						$hostPrototypesToUpdate[] = $hostPrototype;
+					}
+					else {
+						$hostPrototype['ruleid'] = $itemId;
+						$hostPrototypesToCreate[] = $hostPrototype;
+					}
+				}
 
 				if (isset($item['interface_ref'])) {
 					$item['interfaceid'] = $this->referencer->interfacesCache[$hostid][$item['interface_ref']];
@@ -716,6 +803,7 @@ class CConfigurationImport {
 				unset($item['item_prototypes']);
 				unset($item['trigger_prototypes']);
 				unset($item['graph_prototypes']);
+				unset($item['host_prototypes']);
 
 				$itemsId = $this->referencer->resolveItem($hostid, $item['key_']);
 				if ($itemsId) {
@@ -728,7 +816,6 @@ class CConfigurationImport {
 			}
 		}
 
-
 		if ($prototypesToCreate) {
 			foreach ($prototypesToCreate as &$prototype) {
 				$prototype['ruleid'] = $this->referencer->resolveItem($prototype['rule']['hostid'], $prototype['rule']['key']);
@@ -749,6 +836,13 @@ class CConfigurationImport {
 			API::ItemPrototype()->update($prototypesToUpdate);
 		}
 
+		if ($hostPrototypesToCreate) {
+			API::HostPrototype()->create($hostPrototypesToCreate);
+		}
+		if ($hostPrototypesToUpdate) {
+			API::HostPrototype()->update($hostPrototypesToUpdate);
+		}
+
 		// refresh prototypes because templated ones can be inherited to host and used in triggers prototypes or graph prototypes
 		$this->referencer->refreshItems();
 
@@ -781,43 +875,78 @@ class CConfigurationImport {
 				// graph prototypes
 				foreach ($item['graph_prototypes'] as $graph) {
 					$graphHostIds = array();
-					if (!empty($graph['ymin_item_1'])) {
+
+					if ($graph['ymin_item_1']) {
 						$hostId = $this->referencer->resolveHostOrTemplate($graph['ymin_item_1']['host']);
-						$graph['ymin_itemid'] = $this->referencer->resolveItem($hostId, $graph['ymin_item_1']['key']);
+						$itemId = ($hostId)
+							? $this->referencer->resolveItem($hostId, $graph['ymin_item_1']['key'])
+							: false;
+
+						if (!$itemId) {
+							throw new Exception(_s(
+								'Cannot find item "%1$s" on "%2$s" used as the Y axis MIN value for graph prototype "%3$s" of discovery rule "%4$s" on "%5$s".',
+								$graph['ymin_item_1']['key'],
+								$graph['ymin_item_1']['host'],
+								$graph['name'],
+								$item['name'],
+								$host
+							));
+						}
+
+						$graph['ymin_itemid'] = $itemId;
 					}
-					if (!empty($graph['ymax_item_1'])) {
+
+					if ($graph['ymax_item_1']) {
 						$hostId = $this->referencer->resolveHostOrTemplate($graph['ymax_item_1']['host']);
-						$graph['ymax_itemid'] = $this->referencer->resolveItem($hostId, $graph['ymax_item_1']['key']);
+						$itemId = ($hostId)
+							? $this->referencer->resolveItem($hostId, $graph['ymax_item_1']['key'])
+							: false;
+
+						if (!$itemId) {
+							throw new Exception(_s(
+								'Cannot find item "%1$s" on "%2$s" used as the Y axis MAX value for graph prototype "%3$s" of discovery rule "%4$s" on "%5$s".',
+								$graph['ymax_item_1']['key'],
+								$graph['ymax_item_1']['host'],
+								$graph['name'],
+								$item['name'],
+								$host
+							));
+						}
+
+						$graph['ymax_itemid'] = $itemId;
 					}
 
 
 					foreach ($graph['gitems'] as &$gitem) {
-						$gitemhostId = $this->referencer->resolveHostOrTemplate($gitem['item']['host']);
+						if (!$gitemHostId = $this->referencer->resolveHostOrTemplate($gitem['item']['host'])) {
+							throw new Exception(_s('Cannot find host or template "%1$s" used in graph "%2$s".',
+								$gitem['item']['host'], $graph['name']));
+						}
 
-						$gitem['itemid'] = $this->referencer->resolveItem($gitemhostId, $gitem['item']['key']);
+						$gitem['itemid'] = $this->referencer->resolveItem($gitemHostId, $gitem['item']['key']);
 
-						$graphHostIds[$gitemhostId] = $gitemhostId;
+						$graphHostIds[$gitemHostId] = $gitemHostId;
 					}
 					unset($gitem);
 
 
 					// TODO: do this for all graphs at once
-					$sql = 'SELECT g.graphid
-							FROM graphs g,graphs_items gi,items i
-							WHERE g.graphid=gi.graphid
-								AND gi.itemid=i.itemid
-								AND g.name='.zbx_dbstr($graph['name']).'
-								AND '.DBcondition('i.hostid', $graphHostIds);
+					$sql = 'SELECT g.graphid'.
+							' FROM graphs g,graphs_items gi,items i'.
+							' WHERE g.graphid=gi.graphid'.
+								' AND gi.itemid=i.itemid'.
+								' AND g.name='.zbx_dbstr($graph['name']).
+								' AND '.dbConditionInt('i.hostid', $graphHostIds);
 					$graphExists = DBfetch(DBselect($sql));
 
 					if ($graphExists) {
 						$dbGraph = API::GraphPrototype()->get(array(
 							'graphids' => $graphExists['graphid'],
-							'output' => API_OUTPUT_SHORTEN,
+							'output' => array('graphid'),
 							'editable' => true
 						));
 						if (empty($dbGraph)) {
-							throw new Exception(_s('No permission for Graph "%1$s".', $graph['name']));
+							throw new Exception(_s('No permission for graph "%1$s".', $graph['name']));
 						}
 						$graph['graphid'] = $graphExists['graphid'];
 						$graphsToUpdate[] = $graph;
@@ -860,17 +989,47 @@ class CConfigurationImport {
 		foreach ($allGraphs as $graph) {
 			$graphHostIds = array();
 
-			if (!empty($graph['ymin_item_1'])) {
+			if ($graph['ymin_item_1']) {
 				$hostId = $this->referencer->resolveHostOrTemplate($graph['ymin_item_1']['host']);
-				$graph['ymin_itemid'] = $this->referencer->resolveItem($hostId, $graph['ymin_item_1']['key']);
+				$itemId = ($hostId)
+					? $this->referencer->resolveItem($hostId, $graph['ymin_item_1']['key'])
+					: false;
+
+				if (!$itemId) {
+					throw new Exception(_s(
+						'Cannot find item "%1$s" on "%2$s" used as the Y axis MIN value for graph "%3$s".',
+						$graph['ymin_item_1']['key'],
+						$graph['ymin_item_1']['host'],
+						$graph['name']
+					));
+				}
+
+				$graph['ymin_itemid'] = $itemId;
 			}
-			if (!empty($graph['ymax_item_1'])) {
+
+			if ($graph['ymax_item_1']) {
 				$hostId = $this->referencer->resolveHostOrTemplate($graph['ymax_item_1']['host']);
-				$graph['ymax_itemid'] = $this->referencer->resolveItem($hostId, $graph['ymax_item_1']['key']);
+				$itemId = ($hostId)
+					? $this->referencer->resolveItem($hostId, $graph['ymax_item_1']['key'])
+					: false;
+
+				if (!$itemId) {
+					throw new Exception(_s(
+						'Cannot find item "%1$s" on "%2$s" used as the Y axis MAX value for graph "%3$s".',
+						$graph['ymax_item_1']['key'],
+						$graph['ymax_item_1']['host'],
+						$graph['name']
+					));
+				}
+
+				$graph['ymax_itemid'] = $itemId;
 			}
 
 			foreach ($graph['gitems'] as &$gitem) {
-				$gitemHostId = $this->referencer->resolveHostOrTemplate($gitem['item']['host']);
+				if (!$gitemHostId = $this->referencer->resolveHostOrTemplate($gitem['item']['host'])) {
+					throw new Exception(_s('Cannot find host or template "%1$s" used in graph "%2$s".',
+						$gitem['item']['host'], $graph['name']));
+				}
 
 				$gitem['itemid'] = $this->referencer->resolveItem($gitemHostId, $gitem['item']['key']);
 
@@ -879,22 +1038,22 @@ class CConfigurationImport {
 			unset($gitem);
 
 			// TODO: do this for all graphs at once
-			$sql = 'SELECT g.graphid
-					FROM graphs g,graphs_items gi,items i
-					WHERE g.graphid=gi.graphid
-						AND gi.itemid=i.itemid
-						AND g.name='.zbx_dbstr($graph['name']).'
-						AND '.DBcondition('i.hostid', $graphHostIds);
+			$sql = 'SELECT g.graphid'.
+					' FROM graphs g,graphs_items gi,items i'.
+					' WHERE g.graphid=gi.graphid'.
+						' AND gi.itemid=i.itemid'.
+						' AND g.name='.zbx_dbstr($graph['name']).
+						' AND '.dbConditionInt('i.hostid', $graphHostIds);
 			$graphExists = DBfetch(DBselect($sql));
 
 			if ($graphExists) {
 				$dbGraph = API::Graph()->get(array(
 					'graphids' => $graphExists['graphid'],
-					'output' => API_OUTPUT_SHORTEN,
+					'output' => array('graphid'),
 					'editable' => true
 				));
 				if (empty($dbGraph)) {
-					throw new Exception(_s('No permission for Graph "%1$s".', $graph['name']));
+					throw new Exception(_s('No permission for graph "%1$s".', $graph['name']));
 				}
 				$graph['graphid'] = $graphExists['graphid'];
 				$graphsToUpdate[] = $graph;
@@ -1008,7 +1167,7 @@ class CConfigurationImport {
 		$imagesToUpdate = array();
 		$allImages = zbx_toHash($allImages, 'name');
 
-		$dbImages = DBselect('SELECT i.imageid,i.name FROM images i WHERE '.DBcondition('i.name', array_keys($allImages)));
+		$dbImages = DBselect('SELECT i.imageid,i.name FROM images i WHERE '.dbConditionString('i.name', array_keys($allImages)));
 		while ($dbImage = DBfetch($dbImages)) {
 			$dbImage['image'] = $allImages[$dbImage['name']]['image'];
 			$imagesToUpdate[] = $dbImage;
