@@ -331,13 +331,13 @@ class CScreenBase {
 		}
 		else {
 			if ($options['period'] < ZBX_MIN_PERIOD) {
-				show_message(_n('Warning. Minimum time period to display is %1$s hour.',
-						'Warning. Minimum time period to display is %1$s hours.', (int) ZBX_MIN_PERIOD / SEC_PER_HOUR));
+				show_message(_n('Minimum time period to display is %1$s hour.',
+						'Minimum time period to display is %1$s hours.', (int) ZBX_MIN_PERIOD / SEC_PER_HOUR));
 				$options['period'] = ZBX_MIN_PERIOD;
 			}
 			elseif ($options['period'] > ZBX_MAX_PERIOD) {
-				show_message(_n('Warning. Maximum time period to display is %1$s day.',
-						'Warning. Maximum time period to display is %1$s days.', (int) ZBX_MAX_PERIOD / SEC_PER_DAY));
+				show_message(_n('Maximum time period to display is %1$s day.',
+						'Maximum time period to display is %1$s days.', (int) ZBX_MAX_PERIOD / SEC_PER_DAY));
 				$options['period'] = ZBX_MAX_PERIOD;
 			}
 		}
@@ -349,34 +349,34 @@ class CScreenBase {
 		$time = time();
 		$usertime = null;
 		$stimeNow = null;
-		$isNow = false;
+		$isNow = 0;
 
 		if (!empty($options['stime'])) {
 			$stimeUnix = zbxDateToTime($options['stime']);
 
 			if ($stimeUnix > $time || zbxAddSecondsToUnixtime($options['period'], $stimeUnix) > $time) {
 				$stimeNow = $options['stime'];
-				$options['stime'] = date('YmdHis', $time - $options['period']);
-				$usertime = date('YmdHis', $time);
-				$isNow = true;
+				$options['stime'] = date(TIMESTAMP_FORMAT, $time - $options['period']);
+				$usertime = date(TIMESTAMP_FORMAT, $time);
+				$isNow = 1;
 			}
 			else {
-				$usertime = date('YmdHis', zbxAddSecondsToUnixtime($options['period'], $stimeUnix));
-				$isNow = false;
+				$usertime = date(TIMESTAMP_FORMAT, zbxAddSecondsToUnixtime($options['period'], $stimeUnix));
+				$isNow = 0;
 			}
 
 			if ($options['updateProfile'] && !empty($options['profileIdx'])) {
 				CProfile::update($options['profileIdx'].'.stime', $options['stime'], PROFILE_TYPE_STR, $options['profileIdx2']);
-				CProfile::update($options['profileIdx'].'.isnow', $isNow, PROFILE_TYPE_STR, $options['profileIdx2']);
+				CProfile::update($options['profileIdx'].'.isnow', $isNow, PROFILE_TYPE_INT, $options['profileIdx2']);
 			}
 		}
 		else {
 			if (!empty($options['profileIdx'])) {
 				$isNow = CProfile::get($options['profileIdx'].'.isnow', null, $options['profileIdx2']);
 				if ($isNow) {
-					$options['stime'] = date('YmdHis', $time - $options['period']);
-					$usertime = date('YmdHis', $time);
-					$stimeNow = date('YmdHis', zbxAddSecondsToUnixtime(31536000, $options['stime'])); // 31536000 = 86400 * 365 = 1 year
+					$options['stime'] = date(TIMESTAMP_FORMAT, $time - $options['period']);
+					$usertime = date(TIMESTAMP_FORMAT, $time);
+					$stimeNow = date(TIMESTAMP_FORMAT, zbxAddSecondsToUnixtime(SEC_PER_YEAR, $options['stime']));
 
 					if ($options['updateProfile']) {
 						CProfile::update($options['profileIdx'].'.stime', $options['stime'], PROFILE_TYPE_STR, $options['profileIdx2']);
@@ -384,19 +384,19 @@ class CScreenBase {
 				}
 				else {
 					$options['stime'] = CProfile::get($options['profileIdx'].'.stime', null, $options['profileIdx2']);
-					$usertime = date('YmdHis', zbxAddSecondsToUnixtime($options['period'], $options['stime']));
+					$usertime = date(TIMESTAMP_FORMAT, zbxAddSecondsToUnixtime($options['period'], $options['stime']));
 				}
 			}
 
 			if (empty($options['stime'])) {
-				$options['stime'] = date('YmdHis', $time - $options['period']);
-				$usertime = date('YmdHis', $time);
-				$stimeNow = date('YmdHis', zbxAddSecondsToUnixtime(31536000, $options['stime'])); // 31536000 = 86400 * 365 = 1 year
-				$isNow = true;
+				$options['stime'] = date(TIMESTAMP_FORMAT, $time - $options['period']);
+				$usertime = date(TIMESTAMP_FORMAT, $time);
+				$stimeNow = date(TIMESTAMP_FORMAT, zbxAddSecondsToUnixtime(SEC_PER_YEAR, $options['stime']));
+				$isNow = 1;
 
 				if ($options['updateProfile'] && !empty($options['profileIdx'])) {
 					CProfile::update($options['profileIdx'].'.stime', $options['stime'], PROFILE_TYPE_STR, $options['profileIdx2']);
-					CProfile::update($options['profileIdx'].'.isnow', $isNow, PROFILE_TYPE_STR, $options['profileIdx2']);
+					CProfile::update($options['profileIdx'].'.isnow', $isNow, PROFILE_TYPE_INT, $options['profileIdx2']);
 				}
 			}
 		}
@@ -405,7 +405,7 @@ class CScreenBase {
 			'period' => $options['period'],
 			'stime' => $options['stime'],
 			'stimeNow' => !empty($stimeNow) ? $stimeNow : $options['stime'],
-			'starttime' => date('YmdHis', $time - ZBX_MAX_PERIOD),
+			'starttime' => date(TIMESTAMP_FORMAT, $time - ZBX_MAX_PERIOD),
 			'usertime' => $usertime,
 			'isNow' => $isNow
 		);
@@ -422,7 +422,7 @@ class CScreenBase {
 	 * @param string	$options['stimeNow']
 	 * @param string	$options['starttime']
 	 * @param string	$options['usertime']
-	 * @param boolean	$options['isNow']
+	 * @param int		$options['isNow']
 	 */
 	public static function debugTime(array $time = array()) {
 		return 'period='.zbx_date2age(0, $time['period']).', ('.$time['period'].')<br/>'.
