@@ -19,39 +19,19 @@
 **/
 
 
-if (!isset($DB)) {
-	$DB = array();
-	if (isset($DB_TYPE)) {
-		$DB['TYPE'] = $DB_TYPE;
-	}
-	if (isset($DB_SERVER)) {
-		$DB['SERVER'] = $DB_SERVER;
-	}
-	if (isset($DB_PORT)) {
-		$DB['PORT'] = $DB_PORT;
-	}
-	if (isset($DB_DATABASE)) {
-		$DB['DATABASE'] = $DB_DATABASE;
-	}
-	if (isset($DB_USER)) {
-		$DB['USER'] = $DB_USER;
-	}
-	if (isset($DB_PASSWORD)) {
-		$DB['PASSWORD'] = $DB_PASSWORD;
-	}
-}
-
 /**
- * Creates global database connection
+ * Creates global database connection.
  *
  * @param string $error returns a message in case of an error
+ * @param bool   $debug turns On or Off trace calls when making connections. Suggested debug mode Off during Zabbix setup
+ *
  * @return bool
  */
 function DBconnect(&$error) {
 	global $DB;
 
 	if (isset($DB['DB'])) {
-		$error=_('Cannot create another database connection.');
+		$error = _('Cannot create another database connection.');
 		return false;
 	}
 
@@ -72,20 +52,17 @@ function DBconnect(&$error) {
 
 		switch ($DB['TYPE']) {
 			case ZBX_DB_MYSQL:
-				$mysql_server = $DB['SERVER'].(!empty($DB['PORT']) ? ':'.$DB['PORT'] : '');
-
-				if (!$DB['DB'] = mysql_connect($mysql_server, $DB['USER'], $DB['PASSWORD'])) {
-					$error = 'Error connecting to database ['.mysql_error().']';
+				$DB['DB'] = @mysqli_connect($DB['SERVER'], $DB['USER'], $DB['PASSWORD'], $DB['DATABASE'], $DB['PORT']);
+				if (!$DB['DB']) {
+					$error = 'Error connecting to database ['.trim(mysqli_connect_error()).']';
 					$result = false;
 				}
 				else {
-					if (!mysql_select_db($DB['DATABASE'])) {
-						$error = 'Error database in selection ['.mysql_error().']';
-						$result = false;
-					}
-					else {
-						DBexecute('SET NAMES utf8');
-					}
+					DBexecute('SET NAMES utf8');
+				}
+
+				if ($result) {
+					$dbBackend = new MysqlDbBackend();
 				}
 				break;
 			case ZBX_DB_POSTGRESQL:
@@ -96,7 +73,7 @@ function DBconnect(&$error) {
 					(!empty($DB['PASSWORD']) ? 'password=\''.pg_connect_escape($DB['PASSWORD']).'\' ' : '').
 					(!empty($DB['PORT']) ? 'port='.pg_connect_escape($DB['PORT']) : '');
 
-				$DB['DB']= pg_connect($pg_connection_string);
+				$DB['DB']= @pg_connect($pg_connection_string);
 				if (!$DB['DB']) {
 					$error = 'Error connecting to database';
 					$result = false;
@@ -104,9 +81,13 @@ function DBconnect(&$error) {
 				elseif (false !== ($pgsql_version = pg_parameter_status('server_version'))) {
 					if ((int) $pgsql_version >= 9) {
 						// change the output format for values of type bytea from hex (the default) to escape
-						DBexecute('set bytea_output = escape');
+						DBexecute('SET bytea_output = escape');
 					}
 				}
+
+				if ($result) {
+					$dbBackend = new PostgresqlDbBackend();
+				}
 				break;
 			case ZBX_DB_ORACLE:
 				$connect = '';
@@ -121,7 +102,7 @@ function DBconnect(&$error) {
 					}
 				}
 
-				$DB['DB'] = ociplogon($DB['USER'], $DB['PASSWORD'], $connect);
+				$DB['DB'] = @oci_connect($DB['USER'], $DB['PASSWORD'], $connect);
 				if ($DB['DB']) {
 					DBexecute('ALTER SESSION SET NLS_NUMERIC_CHARACTERS='.zbx_dbstr('. '));
 				}
@@ -129,6 +110,10 @@ function DBconnect(&$error) {
 					$error = 'Error connecting to database';
 					$result = false;
 				}
+
+				if ($result) {
+					$dbBackend = new OracleDbBackend();
+				}
 				break;
 			case ZBX_DB_DB2:
 				$connect = '';
@@ -139,7 +124,7 @@ function DBconnect(&$error) {
 				$connect .= 'UID='.$DB['USER'].';';
 				$connect .= 'PWD='.$DB['PASSWORD'].';';
 
-				$DB['DB'] = db2_connect($connect, $DB['USER'], $DB['PASSWORD']);
+				$DB['DB'] = @db2_connect($connect, $DB['USER'], $DB['PASSWORD']);
 				if (!$DB['DB']) {
 					$error = 'Error connecting to database';
 					$result = false;
@@ -149,17 +134,21 @@ function DBconnect(&$error) {
 						'db2_attr_case' => DB2_CASE_LOWER,
 					);
 					db2_set_option($DB['DB'], $options, 1);
-					if (isset($DB['SCHEMA']) && ($DB['SCHEMA'] != '')) {
-						DBexecute("SET CURRENT SCHEMA='".$DB['SCHEMA']."'");
+					if (isset($DB['SCHEMA']) && $DB['SCHEMA'] != '') {
+						DBexecute('SET CURRENT SCHEMA='.zbx_dbstr($DB['SCHEMA']));
 					}
 				}
+
+				if ($result) {
+					$dbBackend = new Db2DbBackend();
+				}
 				break;
 			case ZBX_DB_SQLITE3:
 				if (file_exists($DB['DATABASE'])) {
 					init_sqlite3_access();
 					lock_sqlite3_access();
 					try{
-						$DB['DB'] = new SQLite3($DB['DATABASE'], SQLITE3_OPEN_READWRITE);
+						$DB['DB'] = @new SQLite3($DB['DATABASE'], SQLITE3_OPEN_READWRITE);
 					}
 					catch (Exception $e) {
 						$error = 'Error connecting to database';
@@ -171,15 +160,26 @@ function DBconnect(&$error) {
 					$error = 'Missing database';
 					$result = false;
 				}
+
+				if ($result) {
+					$dbBackend = new SqliteDbBackend();
+				}
 				break;
 			default:
 				$error = 'Unsupported database';
 				$result = false;
 		}
 	}
+
+	if ($result && !$dbBackend->checkDbVersion()) {
+		$error = $dbBackend->getError();
+		$result = false;
+	}
+
 	if (false == $result) {
 		$DB['DB'] = null;
 	}
+
 	return $result;
 }
 
@@ -191,13 +191,13 @@ function DBclose() {
 	if (isset($DB['DB']) && !empty($DB['DB'])) {
 		switch ($DB['TYPE']) {
 			case ZBX_DB_MYSQL:
-				$result = mysql_close($DB['DB']);
+				$result = mysqli_close($DB['DB']);
 				break;
 			case ZBX_DB_POSTGRESQL:
 				$result = pg_close($DB['DB']);
 				break;
 			case ZBX_DB_ORACLE:
-				$result = ocilogoff($DB['DB']);
+				$result = oci_close($DB['DB']);
 				break;
 			case ZBX_DB_DB2:
 				$result = db2_close($DB['DB']);
@@ -261,10 +261,10 @@ function DBstart() {
 
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
-			$result = DBexecute('begin');
+			$result = DBexecute('BEGIN');
 			break;
 		case ZBX_DB_POSTGRESQL:
-			$result = DBexecute('begin');
+			$result = DBexecute('BEGIN');
 			break;
 		case ZBX_DB_ORACLE:
 			$result = true;
@@ -274,16 +274,17 @@ function DBstart() {
 			break;
 		case ZBX_DB_SQLITE3:
 			lock_sqlite3_access();
-			$result = DBexecute('begin');
+			$result = DBexecute('BEGIN');
 			break;
 	}
 	return $result;
 }
 
 /**
- * Closes transaction
+ * Closes transaction.
  *
  * @param string $doCommit True - do commit, rollback otherwise. Rollback is also always performed if a sql failed within this transaction.
+ *
  * @return bool True - successful commit, False - otherwise
  */
 function DBend($doCommit = true) {
@@ -321,13 +322,13 @@ function DBcommit() {
 
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
-			$result = DBexecute('commit');
+			$result = DBexecute('COMMIT');
 			break;
 		case ZBX_DB_POSTGRESQL:
-			$result = DBexecute('commit');
+			$result = DBexecute('COMMIT');
 			break;
 		case ZBX_DB_ORACLE:
-			$result = ocicommit($DB['DB']);
+			$result = oci_commit($DB['DB']);
 			break;
 		case ZBX_DB_DB2:
 			$result = db2_commit($DB['DB']);
@@ -336,7 +337,7 @@ function DBcommit() {
 			}
 			break;
 		case ZBX_DB_SQLITE3:
-			$result = DBexecute('commit');
+			$result = DBexecute('COMMIT');
 			unlock_sqlite3_access();
 			break;
 	}
@@ -350,40 +351,26 @@ function DBrollback() {
 
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
-			$result = DBexecute('rollback');
+			$result = DBexecute('ROLLBACK');
 			break;
 		case ZBX_DB_POSTGRESQL:
-			$result = DBexecute('rollback');
+			$result = DBexecute('ROLLBACK');
 			break;
 		case ZBX_DB_ORACLE:
-			$result = ocirollback($DB['DB']);
+			$result = oci_rollback($DB['DB']);
 			break;
 		case ZBX_DB_DB2:
 			$result = db2_rollback($DB['DB']);
 			db2_autocommit($DB['DB'], DB2_AUTOCOMMIT_ON);
 			break;
 		case ZBX_DB_SQLITE3:
-			$result = DBexecute('rollback');
+			$result = DBexecute('ROLLBACK');
 			unlock_sqlite3_access();
 			break;
 	}
 	return $result;
 }
 
-/* NOTE:
-	LIMIT and OFFSET records
-
-	Example: select 6-15 row.
-
-	MySQL:
-		SELECT a FROM tbl LIMIT 5,10
-		SELECT a FROM tbl LIMIT 10 OFFSET 5
-	PostgreSQL:
-		SELECT a FROM tbl LIMIT 10 OFFSET 5
-	Oracle:
-		SELECT a FROM tbe WHERE ROWNUM < 15 // ONLY < 15
-		SELECT * FROM (SELECT ROWNUM as RN, * FROM tbl) WHERE RN BETWEEN 6 AND 15
-//*/
 /**
  * Select data from DB. Use function DBexecute for non-selects.
  *
@@ -392,11 +379,12 @@ function DBrollback() {
  * DBselect('select * from users',50,200)
  *
  * @param string $query
- * @param integer $limit max number of record to return
- * @param integer $offset return starting from $offset record
+ * @param int $limit    max number of record to return
+ * @param int $offset   return starting from $offset record
+ *
  * @return resource or object, False if failed
  */
-function &DBselect($query, $limit = null, $offset = 0) {
+function DBselect($query, $limit = null, $offset = 0) {
 	global $DB;
 
 	$result = false;
@@ -405,35 +393,18 @@ function &DBselect($query, $limit = null, $offset = 0) {
 		return $result;
 	}
 
-	if ((isset($limit) && ($limit < 0 || !zbx_ctype_digit($limit))) || $offset < 0 || !zbx_ctype_digit($offset)) {
-		$moreDetails = isset($limit) ? ' Limit ['.$limit.'] Offset ['.$offset.']' : ' Offset ['.$offset.']';
-		error('Incorrect parameters for limit and/or offset. Query ['.$query.']'.$moreDetails);
-		return $result;
+	// add the LIMIT clause
+	if(!$query = DBaddLimit($query, $limit, $offset)) {
+		return false;
 	}
 
 	$time_start = microtime(true);
 	$DB['SELECT_COUNT']++;
 
-	// Process limit and offset
-	if (isset($limit)) {
-		switch ($DB['TYPE']) {
-			case ZBX_DB_MYSQL:
-			case ZBX_DB_POSTGRESQL:
-			case ZBX_DB_SQLITE3:
-				$query .= ' LIMIT '.intval($limit).' OFFSET '.intval($offset);
-				break;
-			case ZBX_DB_ORACLE:
-			case ZBX_DB_DB2:
-				$till = $offset + $limit;
-				$query = 'SELECT * FROM ('.$query.') WHERE rownum BETWEEN '.intval($offset).' AND '.intval($till);
-				break;
-		}
-	}
-
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
-			if (!$result = mysql_query($query, $DB['DB'])) {
-				error('Error in query ['.$query.'] ['.mysql_error().']');
+			if (!$result = mysqli_query($DB['DB'], $query)) {
+				error('Error in query ['.$query.'] ['.mysqli_error($DB['DB']).']');
 			}
 			break;
 		case ZBX_DB_POSTGRESQL:
@@ -442,12 +413,12 @@ function &DBselect($query, $limit = null, $offset = 0) {
 			}
 			break;
 		case ZBX_DB_ORACLE:
-			if (!$result = OCIParse($DB['DB'], $query)) {
-				$e = @ocierror();
+			if (!$result = oci_parse($DB['DB'], $query)) {
+				$e = @oci_error();
 				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']');
 			}
-			elseif (!@OCIExecute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
-				$e = ocierror($result);
+			elseif (!@oci_execute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
+				$e = oci_error($result);
 				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']');
 			}
 			break;
@@ -489,6 +460,60 @@ function &DBselect($query, $limit = null, $offset = 0) {
 	return $result;
 }
 
+/**
+ * Add the LIMIT clause to the given query.
+ *
+ * NOTE:
+ * LIMIT and OFFSET records
+ *
+ * Example: select 6-15 row.
+ *
+ * MySQL:
+ * SELECT a FROM tbl LIMIT 5,10
+ * SELECT a FROM tbl LIMIT 10 OFFSET 5
+ *
+ * PostgreSQL:
+ * SELECT a FROM tbl LIMIT 10 OFFSET 5
+ *
+ * Oracle, DB2:
+ * SELECT a FROM tbe WHERE rownum < 15 // ONLY < 15
+ * SELECT * FROM (SELECT * FROM tbl) WHERE rownum BETWEEN 6 AND 15
+ *
+ * @param $query
+ * @param int $limit    max number of record to return
+ * @param int $offset   return starting from $offset record
+ *
+ * @return bool|string
+ */
+function DBaddLimit($query, $limit = 0, $offset = 0) {
+	global $DB;
+
+	if ((isset($limit) && ($limit < 0 || !zbx_ctype_digit($limit))) || $offset < 0 || !zbx_ctype_digit($offset)) {
+		$moreDetails = isset($limit) ? ' Limit ['.$limit.'] Offset ['.$offset.']' : ' Offset ['.$offset.']';
+		error('Incorrect parameters for limit and/or offset. Query ['.$query.']'.$moreDetails);
+
+		return false;
+	}
+
+	// Process limit and offset
+	if (isset($limit)) {
+		switch ($DB['TYPE']) {
+			case ZBX_DB_MYSQL:
+			case ZBX_DB_POSTGRESQL:
+			case ZBX_DB_SQLITE3:
+				$query .= ' LIMIT '.intval($limit).' OFFSET '.intval($offset);
+				break;
+			case ZBX_DB_ORACLE:
+			case ZBX_DB_DB2:
+				$till = $offset + $limit;
+				$query = 'SELECT * FROM ('.$query.') WHERE rownum BETWEEN '.intval($offset).' AND '.intval($till);
+				break;
+		}
+	}
+
+	return $query;
+}
+
 function DBexecute($query, $skip_error_messages = 0) {
 	global $DB;
 
@@ -503,8 +528,8 @@ function DBexecute($query, $skip_error_messages = 0) {
 
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
-			if (!$result = mysql_query($query, $DB['DB'])) {
-				error('Error in query ['.$query.'] ['.mysql_error().']');
+			if (!$result = mysqli_query($DB['DB'], $query)) {
+				error('Error in query ['.$query.'] ['.mysqli_error($DB['DB']).']');
 			}
 			break;
 		case ZBX_DB_POSTGRESQL:
@@ -513,12 +538,12 @@ function DBexecute($query, $skip_error_messages = 0) {
 			}
 			break;
 		case ZBX_DB_ORACLE:
-			if (!$result = OCIParse($DB['DB'], $query)) {
-				$e = @ocierror();
+			if (!$result = oci_parse($DB['DB'], $query)) {
+				$e = @oci_error();
 				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']');
 			}
-			elseif (!@OCIExecute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
-				$e = ocierror($result);
+			elseif (!@oci_execute($result, ($DB['TRANSACTIONS'] ? OCI_DEFAULT : OCI_COMMIT_ON_SUCCESS))) {
+				$e = oci_error($result);
 				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']');
 			}
 			else {
@@ -558,19 +583,28 @@ function DBexecute($query, $skip_error_messages = 0) {
 	return (bool) $result;
 }
 
-function DBfetch(&$cursor, $convertNulls = true) {
+/**
+ * Returns the next data set from a DB resource or false if there are no more results.
+ *
+ * @param resource $cursor
+ * @param bool $convertNulls	convert all null values to string zeroes
+ *
+ * @return array|bool
+ */
+function DBfetch($cursor, $convertNulls = true) {
 	global $DB;
 
 	$result = false;
 
-	if (!isset($DB['DB']) || empty($DB['DB'])) {
+	if (!isset($DB['DB']) || empty($DB['DB']) || is_bool($cursor)) {
 		return $result;
 	}
 
 	switch ($DB['TYPE']) {
 		case ZBX_DB_MYSQL:
-			if (!$result = mysql_fetch_assoc($cursor)) {
-				mysql_free_result($cursor);
+			$result = mysqli_fetch_assoc($cursor);
+			if (!$result) {
+				mysqli_free_result($cursor);
 			}
 			break;
 		case ZBX_DB_POSTGRESQL:
@@ -579,10 +613,12 @@ function DBfetch(&$cursor, $convertNulls = true) {
 			}
 			break;
 		case ZBX_DB_ORACLE:
-			if (ocifetchinto($cursor, $row, (OCI_ASSOC+OCI_RETURN_NULLS))) {
+			if ($row = oci_fetch_assoc($cursor)) {
 				$result = array();
 				foreach ($row as $key => $value) {
 					$field_type = zbx_strtolower(oci_field_type($cursor, $key));
+					// Oracle does not support NULL values for string fields, so if the string is empty, it will return NULL
+					// convert it to an empty string to be consistent with other databases
 					$value = (str_in_array($field_type, array('varchar', 'varchar2', 'blob', 'clob')) && is_null($value)) ? '' : $value;
 
 					if (is_object($value) && (zbx_stristr($field_type, 'lob') !== false)) {
@@ -628,123 +664,19 @@ function DBfetch(&$cursor, $convertNulls = true) {
 			break;
 	}
 
-	if ($convertNulls && $result) {
-		foreach ($result as $key => $val) {
-			if (is_null($val)) {
-				$result[$key] = '0';
+	if ($result) {
+		if ($convertNulls) {
+			foreach ($result as $key => $val) {
+				if (is_null($val)) {
+					$result[$key] = '0';
+				}
 			}
 		}
-	}
-	return $result;
-}
 
-// string value prepearing
-if (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_MYSQL) {
-	function zbx_dbstr($var) {
-		if (is_array($var)) {
-			foreach ($var as $vnum => $value) {
-				$var[$vnum] = "'".mysql_real_escape_string($value)."'";
-			}
-			return $var;
-		}
-		return "'".mysql_real_escape_string($var)."'";
+		return $result;
 	}
 
-	function zbx_dbcast_2bigint($field) {
-		return ' CAST('.$field.' AS UNSIGNED) ';
-	}
-
-	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
-		return !empty($max) ? 'LIMIT '.$min.','.$max : 'LIMIT '.$min;
-	}
-}
-elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_POSTGRESQL) {
-	function zbx_dbstr($var) {
-		if (is_array($var)) {
-			foreach ($var as $vnum => $value) {
-				$var[$vnum] = "'".pg_escape_string($value)."'";
-			}
-			return $var;
-		}
-		return "'".pg_escape_string($var)."'";
-	}
-
-	function zbx_dbcast_2bigint($field) {
-		return ' CAST('.$field.' AS BIGINT) ';
-	}
-
-	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
-		return !empty($max) ? 'LIMIT '.$min.','.$max : 'LIMIT '.$min;
-	}
-}
-elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_ORACLE) {
-	function zbx_dbstr($var) {
-		if (is_array($var)) {
-			foreach ($var as $vnum => $value) {
-				$var[$vnum] = "'".preg_replace('/\'/', '\'\'', $value)."'";
-			}
-			return $var;
-		}
-		return "'".preg_replace('/\'/','\'\'',$var)."'";
-	}
-
-	function zbx_dbcast_2bigint($field) {
-		return ' CAST('.$field.' AS NUMBER(20)) ';
-	}
-
-	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
-		if ($afterWhere) {
-			return !empty($max) ? ' AND ROWNUM BETWEEN '.$min.' AND '.$max : ' AND ROWNUM <='.$min;
-		}
-		else {
-			return !empty($max) ? ' WHERE ROWNUM BETWEEN '.$min.' AND '.$max : ' WHERE ROWNUM <='.$min;
-		}
-	}
-}
-elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_DB2) {
-	function zbx_dbstr($var) {
-		if (is_array($var)) {
-			foreach ($var as $vnum => $value) {
-				$var[$vnum] = "'".db2_escape_string($value)."'";
-			}
-			return $var;
-		}
-		return "'".db2_escape_string($var)."'";
-	}
-
-	function zbx_dbcast_2bigint($field) {
-		return ' CAST('.$field.' AS BIGINT) ';
-	}
-
-	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
-		if ($afterWhere) {
-			return !empty($max) ? ' AND ROWNUM BETWEEN '.$min.' AND '.$max : ' AND ROWNUM <='.$min;
-		}
-		else {
-			return !empty($max) ? ' WHERE ROWNUM BETWEEN '.$min.' AND '.$max : ' WHERE ROWNUM <='.$min;
-		}
-	}
-}
-elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_SQLITE3) {
-	function zbx_dbstr($var) {
-		global $DB;
-
-		if (is_array($var)) {
-			foreach ($var as $vnum => $value) {
-				$var[$vnum] = "'".$DB['DB']->escapeString($value)."'";
-			}
-			return $var;
-		}
-		return "'".$DB['DB']->escapeString($var)."'";
-	}
-
-	function zbx_dbcast_2bigint($field) {
-		return ' CAST('.$field.' AS BIGINT) ';
-	}
-
-	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
-		return !empty($max) ? 'LIMIT '.$min.','.$max : 'LIMIT '.$min;
-	}
+	return false;
 }
 
 function zbx_dbconcat($params) {
@@ -785,11 +717,26 @@ function DBid2nodeid($id_name) {
 	return $result;
 }
 
-function id2nodeid($id_var) {
-	return (int)bcdiv("$id_var", '100000000000000');
+function id2nodeid($id) {
+	return (int) bcdiv("$id", '100000000000000');
 }
 
-function DBin_node($id_name, $nodes = null) {
+/**
+ * Generates the filter by a node for the SQL statement.
+ * For a standalone setup the function will return an empty string.
+ *
+ * For example, function will return " AND h.hostid BETWEEN 500000000000000 AND 599999999999999"
+ *   for $fieldName = 'h.hostid', $nodes = 5, $operator = 'AND'
+ *
+ * Don't call this function directly. Use wrapper functions whereDbNode(), andDbNode() and sqlPartDbNode()
+ *
+ * @param string $fieldName
+ * @param mixed  $nodes
+ * @param string $operator		SQL operator ('AND', 'WHERE')
+ *
+ * @return string
+ */
+function dbNode($fieldName, $nodes = null, $operator = '') {
 	if (is_null($nodes)) {
 		$nodes = get_current_nodeid();
 	}
@@ -803,29 +750,82 @@ function DBin_node($id_name, $nodes = null) {
 	elseif (!is_array($nodes)) {
 		if (is_string($nodes)) {
 			if (!preg_match('/^([0-9,]+)$/', $nodes)) {
-				fatal_error('Incorrect "nodes" for "DBin_node". Passed ['.$nodes.']');
+				fatal_error('Incorrect "nodes" for "dbNode". Passed ['.$nodes.']');
 			}
 		}
 		elseif (!zbx_ctype_digit($nodes)) {
-			fatal_error('Incorrect type of "nodes" for "DBin_node". Passed ['.gettype($nodes).']');
+			fatal_error('Incorrect type of "nodes" for "dbNode". Passed ['.gettype($nodes).']');
 		}
 		$nodes = zbx_toArray($nodes);
 	}
 
+	$sql = '';
 	if (count($nodes) == 1) {
 		$nodeid = reset($nodes);
-		$sql = $id_name.' BETWEEN '.$nodeid.'00000000000000 AND '.$nodeid.'99999999999999';
+		if ($nodeid != 0) {
+			$sql = $fieldName.' BETWEEN '.$nodeid.'00000000000000 AND '.$nodeid.'99999999999999';
+		}
 	}
 	else {
-		$sql = '';
 		foreach ($nodes as $nodeid) {
-			$sql .= '('.$id_name.' BETWEEN '.$nodeid.'00000000000000 AND '.$nodeid.'99999999999999) OR ';
+			$sql .= '('.$fieldName.' BETWEEN '.$nodeid.'00000000000000 AND '.$nodeid.'99999999999999) OR ';
 		}
 		$sql = '('.rtrim($sql, ' OR ').')';
 	}
+
+	if ($sql != '' && $operator != '') {
+		$sql = ' '.$operator.' '.$sql;
+	}
+
 	return $sql;
 }
 
+/**
+ * Wrapper function to generate condition like " WHERE h.hostid BETWEEN 500000000000000 AND 599999999999999"
+ * For a standalone setup the function will return an empty string.
+ *
+ * @param string $fieldName
+ * @param mixed  $nodes
+ *
+ * @return string
+ */
+function whereDbNode($fieldName, $nodes = null) {
+	return dbNode($fieldName, $nodes, 'WHERE');
+}
+
+/**
+ * Wrapper function to generate condition like " AND h.hostid BETWEEN 500000000000000 AND 599999999999999"
+ * For a standalone setup the function will return an empty string.
+ *
+ * @param string $fieldName
+ * @param mixed nodes
+ *
+ * @return string
+ */
+function andDbNode($fieldName, $nodes = null) {
+	return dbNode($fieldName, $nodes, 'AND');
+}
+
+/**
+ * Wrapper function to add condition like "h.hostid BETWEEN 500000000000000 AND 599999999999999" to an array $sqlPartWhere.
+ * For a standalone setup the function will make nothing and will return $sqlPartWhere array without any changes.
+ *
+ * @param array  $sqlPartWhere
+ * @param string $fieldName
+ * @param mixed  $nodes
+ *
+ * @return array
+ */
+function sqlPartDbNode($sqlPartWhere, $fieldName, $nodes = null) {
+	$sql = dbNode($fieldName, $nodes);
+
+	if ($sql != '') {
+		$sqlPartWhere[] = $sql;
+	}
+
+	return $sqlPartWhere;
+}
+
 function in_node($id_var, $nodes = null) {
 	if (is_null($nodes)) {
 		$nodes = get_current_nodeid();
@@ -988,63 +988,26 @@ function zbx_db_search($table, $options, &$sql_parts) {
 	return false;
 }
 
-function zbx_db_sorting(&$sql_parts, $options, $sort_columns, $alias) {
-	if (!zbx_empty($options['sortfield'])) {
-		if (!is_array($options['sortfield'])) {
-			$options['sortfield'] = array($options['sortfield']);
-		}
-		else {
-			$options['sortfield'] = array_unique($options['sortfield']);
-		}
-
-		foreach ($options['sortfield'] as $i => $sortfield) {
-			// validate sortfield
-			if (!str_in_array($sortfield, $sort_columns)) {
-				throw new APIException(ZBX_API_ERROR_INTERNAL, _s('Sorting by field "%1$s" not allowed.', $sortfield));
-			}
-
-			// add sort field to order
-			$sortorder = '';
-			if (is_array($options['sortorder'])) {
-				if (!empty($options['sortorder'][$i])) {
-					$sortorder = ($options['sortorder'][$i] == ZBX_SORT_DOWN) ? ' '.ZBX_SORT_DOWN : '';
-				}
-			}
-			else {
-				$sortorder = ($options['sortorder'] == ZBX_SORT_DOWN) ? ' '.ZBX_SORT_DOWN : '';
-			}
-			$sql_parts['order'][] = $alias.'.'.$sortfield.$sortorder;
-
-			// add sort field to select if distinct is used
-			if (count($sql_parts['from']) > 1) {
-				if (!str_in_array($alias.'.'.$sortfield, $sql_parts['select']) && !str_in_array($alias.'.*', $sql_parts['select'])) {
-					$sql_parts['select'][$sortfield] = $alias.'.'.$sortfield;
-				}
-			}
-		}
-	}
-}
-
 function remove_nodes_from_id($id) {
 	return bcmod($id, '100000000000');
 }
 
 /**
- * Cheks whether all $db_fields keys exists as $args keys.
+ * Checks whether all $db_fields keys exists as $args keys.
  *
  * If $db_fields element value is given and corresponding $args is not then it is assigned to $args element.
  *
- * @param $db_fields
+ * @param $dbFields
  * @param $args
  *
- * @return boolean
+ * @return bool
  */
-function check_db_fields($db_fields, &$args) {
+function check_db_fields($dbFields, &$args) {
 	if (!is_array($args)) {
 		return false;
 	}
 
-	foreach ($db_fields as $field => $def) {
+	foreach ($dbFields as $field => $def) {
 		if (!isset($args[$field])) {
 			if (is_null($def)) {
 				return false;
@@ -1054,122 +1017,121 @@ function check_db_fields($db_fields, &$args) {
 			}
 		}
 	}
+
 	return true;
 }
 
 /**
  * Takes an initial part of SQL query and appends a generated WHERE condition.
- * The WHERE condidion is generated from the given list of values as a mix of
+ * The WHERE condition is generated from the given list of values as a mix of
  * <fieldname> BETWEEN <id1> AND <idN>" and "<fieldname> IN (<id1>,<id2>,...,<idN>)" elements.
  *
- * @param string $fieldName  field name to be used in SQL WHERE condition
- * @param array  $values     array of numerical values sorted in ascending order to be included in WHERE
- * @param bool   $notIn      builds inverted condition
+ * In some frontend places we can get array with bool as input values parameter. This is fail!
+ * Therefore we need check it and return 1=0 as temporary solution to not break the frontend.
+ *
+ * @param string $fieldName		field name to be used in SQL WHERE condition
+ * @param array  $values		array of numerical values sorted in ascending order to be included in WHERE
+ * @param bool   $notIn			builds inverted condition
+ * @param bool   $sort			values mandatory must be sorted
  *
  * @return string
  */
-function dbConditionInt($fieldName, array $values, $notIn = false) {
+function dbConditionInt($fieldName, array $values, $notIn = false, $sort = true) {
 	$MAX_EXPRESSIONS = 950; // maximum  number of values for using "IN (id1>,<id2>,...,<idN>)"
-	$MIN_NUM_BETWEEN = 5; // minimum number of consecutive values for using "BETWEEN <id1> AND <idN>"
+	$MIN_NUM_BETWEEN = 4; // minimum number of consecutive values for using "BETWEEN <id1> AND <idN>"
 
-	if (count($values) == 0) {
+	if (is_bool(reset($values))) {
 		return '1=0';
 	}
 
-	$condition = '';
+	$values = array_keys(array_flip($values));
+
+	if ($sort) {
+		natsort($values);
+
+		$values = array_values($values);
+	}
 
 	$betweens = array();
-	$ins = array();
+	$data = array();
 
-	$pos = 1;
-	$len = 1;
-	$valueL = reset($values);
-	while (false !== ($valueR = next($values))) {
-		$valueL = bcadd($valueL, 1, 0);
+	for ($i = 0, $size = count($values); $i < $size; $i++) {
+		$between = array();
 
-		if (bccomp($valueR, $valueL) != 0) {
-			if ($len >= $MIN_NUM_BETWEEN) {
-				$betweens[] = array(bcsub($valueL, $len, 0), bcsub($valueL, 1, 0));
-			}
-			else {
-				$ins = array_merge($ins, array_slice($values, $pos - $len, $len));
+		// analyze by chunk
+		if (isset($values[$i + $MIN_NUM_BETWEEN])
+				&& bccomp(bcadd($values[$i], $MIN_NUM_BETWEEN), $values[$i + $MIN_NUM_BETWEEN]) == 0) {
+			for ($sizeMinBetween = $i + $MIN_NUM_BETWEEN; $i < $sizeMinBetween; $i++) {
+				$between[] = $values[$i];
 			}
 
-			$len = 1;
-			$valueL = $valueR;
+			$i--; // shift 1 back
+
+			// analyze by one
+			for (; $i < $size; $i++) {
+				if (isset($values[$i + 1]) && bccomp(bcadd($values[$i], 1), $values[$i + 1]) == 0) {
+					$between[] = $values[$i + 1];
+				}
+				else {
+					break;
+				}
+			}
+
+			$betweens[] = $between;
 		}
 		else {
-			$len++;
+			$data[] = $values[$i];
 		}
-		$pos++;
 	}
 
-	if ($len >= $MIN_NUM_BETWEEN) {
-		$betweens[] = array(bcadd(bcsub($valueL, $len, 0), 1, 0), $valueL);
+	// concatenate conditions
+	$dataSize = count($data);
+	$betweenSize = count($betweens);
+
+	$condition = '';
+	$operatorAnd = $notIn ? ' AND ' : ' OR ';
+
+	if ($betweens) {
+		$operatorNot = $notIn ? 'NOT ' : '';
+
+		foreach ($betweens as $between) {
+			$between = $operatorNot.$fieldName.' BETWEEN '.zbx_dbstr($between[0]).' AND '.zbx_dbstr(end($between));
+
+			$condition .= $condition ? $operatorAnd.$between : $between;
+		}
+	}
+
+	if ($dataSize == 1) {
+		$operator = $notIn ? '!=' : '=';
+
+		$condition .= ($condition ? $operatorAnd : '').$fieldName.$operator.zbx_dbstr($data[0]);
 	}
 	else {
-		$ins = array_merge($ins, array_slice($values, $pos - $len, $len));
-	}
+		$operatorNot = $notIn ? ' NOT' : '';
+		$data = array_chunk($data, $MAX_EXPRESSIONS);
 
-	$operand = $notIn ? 'AND' : 'OR';
-	$not = $notIn ? 'NOT ' : '';
-	$inNum = count($ins);
-	$betweenNum = count($betweens);
+		foreach ($data as $chunk) {
+			$chunkIns = '';
 
-	if ($MAX_EXPRESSIONS < $inNum || 1 < $betweenNum || (0 < $inNum && 0 < $betweenNum)) {
-		$condition .= '(';
-	}
-
-	// compose "BETWEEN"s
-	$first = true;
-	foreach ($betweens as $between) {
-		if (!$first) {
-			$condition .= ' '.$operand.' ';
-		}
-		$condition .= $not.$fieldName.' BETWEEN '.zbx_dbstr($between[0]).' AND '.zbx_dbstr($between[1]);
-		$first = false;
-	}
-
-	if (0 < $inNum && 0 < $betweenNum) {
-		$condition .= ' '.$operand.' ';
-	}
-
-	if ($inNum == 1) {
-		foreach ($ins as $insValue) {
-			$condition .= $notIn
-				? $fieldName.'!='.zbx_dbstr($insValue)
-				: $fieldName.'='.zbx_dbstr($insValue);
-			break;
-		}
-	}
-
-	// compose "IN"s
-	else {
-		$first = true;
-		foreach (array_chunk($ins, $MAX_EXPRESSIONS) as $in) {
-			if (!$first) {
-				$condition .= ' '.$operand.' ';
+			foreach ($chunk as $value) {
+				$chunkIns .= ','.zbx_dbstr($value);
 			}
 
-			$in = array_map('zbx_dbstr', $in);
-			$condition .= $fieldName.' '.$not.'IN ('.implode(',', $in).')';
-			$first = false;
+			$chunkIns = $fieldName.$operatorNot.' IN ('.substr($chunkIns, 1).')';
+
+			$condition .= $condition ? $operatorAnd.$chunkIns : $chunkIns;
 		}
 	}
 
-	if ($MAX_EXPRESSIONS < $inNum || 1 < $betweenNum || (0 < $inNum && 0 < $betweenNum)) {
-		$condition .= ')';
-	}
-
-	return $condition;
+	return (($dataSize && $betweenSize) || $betweenSize > 1 || $dataSize > $MAX_EXPRESSIONS) ? '('.$condition.')' : $condition;
 }
 
 /**
  * Takes an initial part of SQL query and appends a generated WHERE condition.
  *
- * @param string $fieldName  field name to be used in SQL WHERE condition
- * @param array  $values     array of string values sorted in ascending order to be included in WHERE
- * @param bool   $notIn      builds inverted condition
+ * @param string $fieldName		field name to be used in SQL WHERE condition
+ * @param array  $values		array of string values sorted in ascending order to be included in WHERE
+ * @param bool   $notIn			builds inverted condition
  *
  * @return string
  */
@@ -1193,15 +1155,11 @@ function dbConditionString($fieldName, array $values, $notIn = false) {
 		$condition .= implode(',', zbx_dbstr($values));
 	}
 
-	return $fieldName.$in.'('.$condition.')';
-}
-
-function zero2null($val) {
-	return ($val == '0') ? 'NULL' : $val; // string 0 because ('any string' == 0) = true
+	return '('.$fieldName.$in.'('.$condition.'))';
 }
 
 /**
- * Transform DB cursor to array
+ * Transform DB cursor to array.
  *
  * @return array
  */
@@ -1214,7 +1172,7 @@ function DBfetchArray($cursor) {
 }
 
 /**
- * Transform DB cursor to array
+ * Transform DB cursor to array.
  *
  * @return array
  */
@@ -1227,7 +1185,31 @@ function DBfetchArrayAssoc($cursor, $field) {
 }
 
 /**
- * Initialize access to SQLite3 database
+ * Fetch only values from one column to array.
+ *
+ * @param resource $cursor
+ * @param string   $column
+ * @param bool     $asHash
+ *
+ * @return array
+ */
+function DBfetchColumn($cursor, $column, $asHash = false) {
+	$result = array();
+
+	while ($dbResult = DBfetch($cursor)) {
+		if ($asHash) {
+			$result[$dbResult[$column]] = $dbResult[$column];
+		}
+		else {
+			$result[] = $dbResult[$column];
+		}
+	}
+
+	return $result;
+}
+
+/**
+ * Initialize access to SQLite3 database.
  *
  * The function creates a semaphore for exclusive SQLite3 access. It is
  * shared between Zabbix front-end and Zabbix Server.
@@ -1241,7 +1223,7 @@ function init_sqlite3_access() {
 }
 
 /**
- * Get exclusive lock on SQLite3 database
+ * Get exclusive lock on SQLite3 database.
  *
  * @return bool
  */
@@ -1252,7 +1234,7 @@ function lock_sqlite3_access() {
 }
 
 /**
- * Release exclusive lock on SQLite3 database
+ * Release exclusive lock on SQLite3 database.
  *
  * @return bool
  */
@@ -1284,3 +1266,131 @@ function idcmp($id1, $id2) {
 function pg_connect_escape($string) {
 	return addcslashes($string, "'\\");
 }
+
+/**
+ * Escape string for safe usage in SQL queries.
+ * Works for ibmdb2, mysql, oracle, postgresql, sqlite.
+ *
+ * @param array|string $var
+ *
+ * @return array|bool|string
+ */
+function zbx_dbstr($var) {
+	global $DB;
+
+	if (!isset($DB['TYPE'])) {
+		return false;
+	}
+
+	switch ($DB['TYPE']) {
+		case ZBX_DB_DB2:
+			if (is_array($var)) {
+				foreach ($var as $vnum => $value) {
+					$var[$vnum] = "'".db2_escape_string($value)."'";
+				}
+				return $var;
+			}
+			return "'".db2_escape_string($var)."'";
+
+		case ZBX_DB_MYSQL:
+			if (is_array($var)) {
+				foreach ($var as $vnum => $value) {
+					$var[$vnum] = "'".mysqli_real_escape_string($DB['DB'], $value)."'";
+				}
+				return $var;
+			}
+			return "'".mysqli_real_escape_string($DB['DB'], $var)."'";
+
+		case ZBX_DB_ORACLE:
+			if (is_array($var)) {
+				foreach ($var as $vnum => $value) {
+					$var[$vnum] = "'".preg_replace('/\'/', '\'\'', $value)."'";
+				}
+				return $var;
+			}
+			return "'".preg_replace('/\'/','\'\'',$var)."'";
+
+		case ZBX_DB_POSTGRESQL:
+			if (is_array($var)) {
+				foreach ($var as $vnum => $value) {
+					$var[$vnum] = "'".pg_escape_string($value)."'";
+				}
+				return $var;
+			}
+			return "'".pg_escape_string($var)."'";
+
+		case ZBX_DB_SQLITE3:
+			if (is_array($var)) {
+				foreach ($var as $vnum => $value) {
+					$var[$vnum] = "'".$DB['DB']->escapeString($value)."'";
+				}
+				return $var;
+			}
+			return "'".$DB['DB']->escapeString($var)."'";
+
+		default:
+			return false;
+	}
+}
+
+/**
+ * Creates db dependent string with sql expression that casts passed value to bigint.
+ * Works for ibmdb2, mysql, oracle, postgresql, sqlite.
+ *
+ * @param int $field
+ *
+ * @return bool|string
+ */
+function zbx_dbcast_2bigint($field) {
+	global $DB;
+
+	if (!isset($DB['TYPE'])) {
+		return false;
+	}
+
+	switch ($DB['TYPE']) {
+		case ZBX_DB_DB2:
+		case ZBX_DB_POSTGRESQL:
+		case ZBX_DB_SQLITE3:
+			return 'CAST('.$field.' AS BIGINT)';
+
+		case ZBX_DB_MYSQL:
+			return 'CAST('.$field.' AS UNSIGNED)';
+
+		case ZBX_DB_ORACLE:
+			return 'CAST('.$field.' AS NUMBER(20))';
+
+		default:
+			return false;
+	}
+}
+
+/**
+ * Creates db dependent string with sql limit.
+ * Works for ibmdb2, mysql, oracle, postgresql, sqlite.
+ *
+ * @param int $limit
+ *
+ * @return bool|string
+ */
+function zbx_limit($limit) {
+	global $DB;
+
+	if (!isset($DB['TYPE'])) {
+		return false;
+	}
+
+	switch ($DB['TYPE']) {
+		case ZBX_DB_DB2:
+		case ZBX_DB_ORACLE:
+			return 'AND rownum<='.$limit;
+
+		case ZBX_DB_MYSQL:
+		case ZBX_DB_POSTGRESQL:
+		case ZBX_DB_SQLITE3:
+			return 'LIMIT '.$limit;
+
+		default:
+			return false;
+	}
+}
