@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,15 +10,14 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
+
 
 function init_mbstrings() {
 	$res = true;
@@ -37,35 +36,46 @@ function mbstrings_available() {
 	return function_exists('mb_strlen') && function_exists('mb_strtoupper') && function_exists('mb_strpos') && function_exists('mb_substr');
 }
 
-function set_zbx_locales() {
-	global $ZBX_LOCALES;
-
-	$ZBX_LOCALES = array(
-		'en_GB' => _('English (en_GB)'),
-		'en_US' => _('English (en_US)'),
-		'zh_CN' => _('Chinese (zh_CN)'),
-		'zh_TW' => _('Chinese (zh_TW)'),
-		'cs_CZ' => _('Czech (cs_CZ)'),
-		'nl_NL' => _('Dutch (nl_NL)'),
-		'fi_FI' => _('Finnish (fi_FI)'),
-		'fr_FR' => _('French (fr_FR)'),
-		'de_DE' => _('German (de_DE)'),
-		'el_GR' => _('Greek (el_GR)'),
-		'hu_HU' => _('Hungarian (hu_HU)'),
-		'id_ID' => _('Indonesian (id_ID)'),
-		'it_IT' => _('Italian (it_IT)'),
-		'ko_KR' => _('Korean (ko_KR)'),
-		'ja_JP' => _('Japanese (ja_JP)'),
-		'lv_LV' => _('Latvian (lv_LV)'),
-		'fa_IR' => _('Persian (fa_IR)'),
-		'pl_PL' => _('Polish (pl_PL)'),
-		'pt_BR' => _('Portuguese (pt_BR)'),
-		'ru_RU' => _('Russian (ru_RU)'),
-		'sk_SK' => _('Slovak (sk_SK)'),
-		'es_ES' => _('Spanish (es_ES)'),
-		'sv_SE' => _('Swedish (sv_SE)'),
-//		'tr_TR' => _('Turkish (tr_TR)'),
-		'uk_UA' => _('Ukrainian (uk_UA)')
+/**
+ * Returns a list of all used locales.
+ *
+ * Each locale has the following properties:
+ * - name       - the full name of the locale
+ * - display    - whether to display the locale in the frontend
+ *
+ * @return array    an array of locales with locale codes as keys and arrays as values
+ */
+function getLocales() {
+	return array(
+		'en_GB' => array('name' => _('English (en_GB)'),	'display' => true),
+		'en_US' => array('name' => _('English (en_US)'),	'display' => true),
+		'bg_BG' => array('name' => _('Bulgarian (bg_BG)'),	'display' => true),
+		'zh_CN' => array('name' => _('Chinese (zh_CN)'),	'display' => true),
+		'zh_TW' => array('name' => _('Chinese (zh_TW)'),	'display' => true),
+		'cs_CZ' => array('name' => _('Czech (cs_CZ)'),		'display' => false),
+		'nl_NL' => array('name' => _('Dutch (nl_NL)'),		'display' => false),
+		'fi_FI' => array('name' => _('Finnish (fi_FI)'),	'display' => false),
+		'fr_FR' => array('name' => _('French (fr_FR)'),		'display' => true),
+		'de_DE' => array('name' => _('German (de_DE)'),		'display' => true),
+		'el_GR' => array('name' => _('Greek (el_GR)'),		'display' => false),
+		'hu_HU' => array('name' => _('Hungarian (hu_HU)'),	'display' => false),
+		'id_ID' => array('name' => _('Indonesian (id_ID)'),	'display' => true),
+		'it_IT' => array('name' => _('Italian (it_IT)'),	'display' => true),
+		'ko_KR' => array('name' => _('Korean (ko_KR)'),		'display' => false),
+		'ja_JP' => array('name' => _('Japanese (ja_JP)'),	'display' => true),
+		'lv_LV' => array('name' => _('Latvian (lv_LV)'),	'display' => false),
+		'lt_LT' => array('name' => _('Lithuanian (lt_LT)'),	'display' => false),
+		'fa_IR' => array('name' => _('Persian (fa_IR)'),	'display' => false),
+		'pl_PL' => array('name' => _('Polish (pl_PL)'),		'display' => false),
+		'pt_BR' => array('name' => _('Portuguese (pt_BR)'),	'display' => true),
+		'pt_PT' => array('name' => _('Portuguese (pt_PT)'),	'display' => true),
+		'ro_RO' => array('name' => _('Romanian (ro_RO)'),	'display' => false),
+		'ru_RU' => array('name' => _('Russian (ru_RU)'),	'display' => true),
+		'sk_SK' => array('name' => _('Slovak (sk_SK)'),		'display' => true),
+		'es_ES' => array('name' => _('Spanish (es_ES)'),	'display' => false),
+		'sv_SE' => array('name' => _('Swedish (sv_SE)'),	'display' => false),
+		'tr_TR' => array('name' => _('Turkish (tr_TR)'),	'display' => false),
+		'uk_UA' => array('name' => _('Ukrainian (uk_UA)'),	'display' => true)
 	);
 }
 
@@ -136,6 +146,7 @@ function zbx_locale_variants_win($language) {
 	$winLanguageName = array(
 		'en_gb' => array('English_United Kingdom.1252', 'english-uk'),
 		'en_us' => array('English_United States.1252', 'english-usa'),
+		'bg_bg' => array('Bulgarian_Bulgaria.1251'),
 		'zh_cn' => array('Chinese (Simplified)_People\'s Republic of China.936', 'chinese'),
 		'zh_tw' => array('Chinese_Taiwan.950', 'chinese'),
 		'cs_cz' => array('Czech_Czech Republic.1250', 'czech'),
@@ -143,22 +154,25 @@ function zbx_locale_variants_win($language) {
 		'fi_fi' => array('Finnish_Finland.1252', 'finnish'),
 		'fr_fr' => array('French_France.1252', 'french'),
 		'de_de' => array('German_Germany.1252', 'german'),
+		'el_gr' => array('Greek_Greece.1253', 'greek'),
 		'hu_hu' => array('Hungarian_Hungary.1250', 'hungarian'),
 		'id_id' => array('Indonesian_indonesia.1252', 'indonesian'),
 		'it_it' => array('Italian_Italy.1252', 'italian'),
 		'ko_kr' => array('Korean_Korea.949', 'korean'),
 		'ja_jp' => array('Japanese_Japan.932', 'japanese'),
 		'lv_lv' => array('Latvian_Latvia.1257', 'latvian'),
+		'lt_lt' => array('Lithuanian_Lithuania.1257', 'lithuanian'),
 		'fa_ir' => array('Farsi_Iran.1256', 'farsi'),
 		'pl_pl' => array('Polish_Poland.1250', 'polish'),
-		'pt_br' => array('Portuguese_Brazil.1252', 'portuguese'),
+		'pt_br' => array('Portuguese_Brazil.1252', 'portuguese-brazil'),
+		'pt_pt' => array('Portuguese_Portugal.1252', 'portuguese'),
+		'ro_ro' => array('Romanian_Romania.1250', 'romanian'),
 		'ru_ru' => array('Russian_Russia.1251', 'russian'),
 		'sk_sk' => array('Slovak_Slovakia.1250', 'slovak'),
 		'es_es' => array('Spanish_Spain.1252', 'spanish'),
 		'sv_se' => array('Swedish_Sweden.1252', 'swedish'),
-		'uk_ua' => array('Ukrainian_Ukraine.1251', 'ukrainian'),
-		'el_gr' => array('Greek_Greece.1253', 'greek')
+		'tr_tr' => array('Turkish_Turkey.1254', 'turkish'),
+		'uk_ua' => array('Ukrainian_Ukraine.1251', 'ukrainian')
 	);
 	return $winLanguageName[strtolower($language)];
 }
-?>
