@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -40,10 +40,6 @@ function is_int_range($value) {
 	return true;
 }
 
-function is_hex_color($value) {
-	return preg_match('/^([0-9,A-F]{6})$/i', $value);
-}
-
 function BETWEEN($min, $max, $var = null) {
 	return '({'.$var.'}>='.$min.'&&{'.$var.'}<='.$max.')&&';
 }
@@ -75,11 +71,13 @@ function validate_ipv4($str, &$arr) {
 	if (!preg_match('/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})$/', $str, $arr)) {
 		return false;
 	}
+
 	for ($i = 1; $i <= 4; $i++) {
 		if (!is_numeric($arr[$i]) || $arr[$i] > 255 || $arr[$i] < 0 ) {
 			return false;
 		}
 	}
+
 	return true;
 }
 
@@ -107,9 +105,11 @@ function validate_ip($str, &$arr) {
 	if (validate_ipv4($str, $arr)) {
 		return true;
 	}
-	if (defined('ZBX_HAVE_IPV6')) {
+
+	if (ZBX_HAVE_IPV6) {
 		return validate_ipv6($str);
 	}
+
 	return false;
 }
 
@@ -134,7 +134,7 @@ function validate_ip_range_mask($ip_range) {
 	if (validate_ipv4($ip, $arr)) {
 		return preg_match('/^\d{1,2}$/', $bits) && $bits >= 16 && $bits <= 32;
 	}
-	elseif (defined('ZBX_HAVE_IPV6') && validate_ipv6($ip, $arr)) {
+	elseif (ZBX_HAVE_IPV6 && validate_ipv6($ip, $arr)) {
 		return preg_match('/^\d{1,3}$/', $bits) && $bits >= 112 && $bits <= 128;
 	}
 	else {
@@ -166,7 +166,7 @@ function validate_ip_range_range($ip_range) {
 			}
 		}
 	}
-	elseif (defined('ZBX_HAVE_IPV6') && validate_ipv6($parts[0])) {
+	elseif (ZBX_HAVE_IPV6 && validate_ipv6($parts[0])) {
 		$ip_parts = explode(':', $parts[0]);
 		$ip_parts_count = count($ip_parts);
 
@@ -287,111 +287,76 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 	if (is_null($caption)) {
 		$caption = $field;
 	}
+
 	if (is_array($var) && $type != T_ZBX_IP) {
 		$err = ZBX_VALID_OK;
-		foreach ($var as $el) {
-			$err |= check_type($field, $flags, $el, $type);
+
+		foreach ($var as $v) {
+			$err |= check_type($field, $flags, $v, $type);
 		}
+
 		return $err;
 	}
 
 	if ($type == T_ZBX_IP) {
 		if (!validate_ip($var, $arr)) {
-			if ($flags&P_SYS) {
-				info(_s('Critical error. Field "%1$s" is not IP.', $field));
-				return ZBX_VALID_ERROR;
-			}
-			else {
-				info(_s('Warning. Field "%1$s" is not IP.', $field));
-				return ZBX_VALID_WARNING;
-			}
+			info(_s('Field "%1$s" is not IP.', $caption));
+
+			return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 		}
+
 		return ZBX_VALID_OK;
 	}
 
 	if ($type == T_ZBX_IP_RANGE) {
 		if (!validate_ip_range($var)) {
-			if ($flags&P_SYS) {
-				info(_s('Critical error. Field "%1$s" is not IP range.', $field));
-				return ZBX_VALID_ERROR;
-			}
-			else{
-				info(_s('Warning. Field "%1$s" is not IP range.', $field));
-				return ZBX_VALID_WARNING;
-			}
+			info(_s('Field "%1$s" is not IP range.', $caption));
+
+			return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 		}
+
 		return ZBX_VALID_OK;
 	}
 
 	if ($type == T_ZBX_INT_RANGE) {
 		if (!is_int_range($var)) {
-			if ($flags&P_SYS) {
-				info(_s('Critical error. Field "%1$s" is not integer list or range.', $field));
-				return ZBX_VALID_ERROR;
-			}
-			else {
-				info(_s('Warning. Field "%1$s" is not integer list or range.', $field));
-				return ZBX_VALID_WARNING;
-			}
+			info(_s('Field "%1$s" is not integer list or range.', $caption));
+
+			return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 		}
+
 		return ZBX_VALID_OK;
 	}
 
 	if ($type == T_ZBX_INT && !zbx_is_int($var)) {
-		if ($flags&P_SYS) {
-			info(_s('Critical error. Field "%1$s" is not integer.', $field));
-			return ZBX_VALID_ERROR;
-		}
-		else {
-			info(_s('Warning. Field "%1$s" is not integer.', $field));
-			return ZBX_VALID_WARNING;
-		}
+		info(_s('Field "%1$s" is not integer.', $caption));
+
+		return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 	}
 
 	if ($type == T_ZBX_DBL && !is_numeric($var)) {
-		if ($flags&P_SYS) {
-			info(_s('Critical error. Field "%1$s" is not decimal number.', $field));
-			return ZBX_VALID_ERROR;
-		}
-		else {
-			info(_s('Warning. Field "%1$s" is not decimal number.', $field));
-			return ZBX_VALID_WARNING;
-		}
+		info(_s('Field "%1$s" is not decimal number.', $caption));
+
+		return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 	}
 
 	if ($type == T_ZBX_STR && !is_string($var)) {
-		if ($flags&P_SYS) {
-			info(_s('Critical error. Field "%1$s" is not string.', $field));
-			return ZBX_VALID_ERROR;
-		}
-		else {
-			info(_s('Warning. Field "%1$s" is not string.', $field));
-			return ZBX_VALID_WARNING;
+		info(_s('Field "%1$s" is not string.', $caption));
+
+		return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
+	}
+
+	if ($type == T_ZBX_CLR) {
+		$colorValidator = new CColorValidator();
+
+		if (!$colorValidator->validate($var)) {
+			$var = 'FFFFFF';
+			info(_s('Colour "%1$s" is not correct: expecting hexadecimal colour code (6 symbols).', $caption));
+
+			return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 		}
 	}
 
-	if ($type == T_ZBX_STR && !defined('ZBX_ALLOW_UNICODE') && zbx_strlen($var) != zbx_strlen($var)) {
-		if ($flags&P_SYS) {
-			info(_s('Critical error. Field "%1$s" contains Multibyte chars.', $field));
-			return ZBX_VALID_ERROR;
-		}
-		else {
-			info(_s('Warning. Field "%1$s" multibyte chars are restricted.', $field));
-			return ZBX_VALID_ERROR;
-		}
-	}
-
-	if ($type == T_ZBX_CLR && !is_hex_color($var)) {
-		$var = 'FFFFFF';
-		if ($flags&P_SYS) {
-			info(_s('Critical error. Field "%1$s" is not a colour.', $field));
-			return ZBX_VALID_ERROR;
-		}
-		else {
-			info(_s('Warning. Field "%1$s" is not a colour.', $caption));
-			return ZBX_VALID_WARNING;
-		}
-	}
 	return ZBX_VALID_OK;
 }
 
@@ -432,14 +397,9 @@ function check_field(&$fields, &$field, $checks) {
 
 	if ($opt == O_MAND) {
 		if (!isset($_REQUEST[$field])) {
-			if ($flags&P_SYS) {
-				info(_s('Critical error. Field "%1$s" is mandatory.', $caption));
-				return ZBX_VALID_ERROR;
-			}
-			else {
-				info(_s('Warning. Field "%1$s" is mandatory.', $caption));
-				return ZBX_VALID_WARNING;
-			}
+			info(_s('Field "%1$s" is mandatory.', $caption));
+
+			return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 		}
 	}
 	elseif ($opt == O_NO) {
@@ -449,23 +409,18 @@ function check_field(&$fields, &$field, $checks) {
 
 		unset_request($field);
 
-		if ($flags&P_SYS) {
-			info(_s('Critical error. Field "%1$s" must be missing.', $caption));
-			return ZBX_VALID_ERROR;
-		}
-		else {
-			info(_s('Warning. Field "%1$s" must be missing.', $caption));
-			return ZBX_VALID_WARNING;
-		}
+		info(_s('Field "%1$s" must be missing.', $caption));
+
+		return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 	}
 	elseif ($opt == O_OPT) {
 		if (!isset($_REQUEST[$field])) {
 			return ZBX_VALID_OK;
 		}
-		elseif ($flags&P_ACT) {
+		elseif ($flags & P_ACT) {
 			if (!isset($_REQUEST['sid'])
 					|| (isset($_COOKIE['zbx_sessionid'])
-					&& ($_REQUEST['sid'] != substr($_COOKIE['zbx_sessionid'], 16, 16)))) {
+							&& $_REQUEST['sid'] != substr($_COOKIE['zbx_sessionid'], 16, 16))) {
 				info(_('Operation cannot be performed due to unauthorized request.'));
 				return ZBX_VALID_ERROR;
 			}
@@ -482,37 +437,23 @@ function check_field(&$fields, &$field, $checks) {
 		return $err;
 	}
 
-	if ((is_null($exception) || $except) && $validation && !calc_exp($fields, $field, $validation)){
+	if ((is_null($exception) || $except) && $validation && !calc_exp($fields, $field, $validation)) {
 		if ($validation == NOT_EMPTY) {
-			if ($flags&P_SYS) {
-				info(_s('Critical error. Incorrect value for field "%1$s": cannot be empty.', $caption));
-			}
-			else {
-				info(_s('Warning. Incorrect value for field "%1$s": cannot be empty.', $caption));
-			}
-		}
-		// check for BETWEEN() function pattern and extract numbers e.g. ({}>=0&&{}<=999)&&
-		elseif (preg_match('/\(\{\}\>=([0-9]*)\&\&\{\}\<=([0-9]*)\)\&\&/', $validation, $result)) {
-			if ($flags&P_SYS) {
-				info(_s('Critical error. Incorrect value "%1$s" for "%2$s" field: must be between %3$s and %4$s.',
-					$_REQUEST[$field], $caption, $result[1], $result[2]));
-			}
-			else {
-				info(_s('Warning. Incorrect value for field "%1$s": must be between %2$s and %3$s.',
-					$caption, $result[1], $result[2]));
-			}
-		}
-		else {
-			if ($flags&P_SYS) {
-				info(_s('Critical error. Incorrect value "%1$s" for "%2$s" field.', $_REQUEST[$field], $caption));
-			}
-			else {
-				info(_s('Warning. Incorrect value for field "%1$s".', $caption));
-			}
+			info(_s('Incorrect value for field "%1$s": cannot be empty.', $caption));
 		}
 
-		return ($flags&P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
+		// check for BETWEEN() function pattern and extract numbers e.g. ({}>=0&&{}<=999)&&
+		elseif (preg_match('/\(\{\}\>=([0-9]*)\&\&\{\}\<=([0-9]*)\)\&\&/', $validation, $result)) {
+			info(_s('Incorrect value "%1$s" for "%2$s" field: must be between %3$s and %4$s.',
+				$_REQUEST[$field], $caption, $result[1], $result[2]));
+		}
+		else {
+			info(_s('Incorrect value "%1$s" for "%2$s" field.', $_REQUEST[$field], $caption));
+		}
+
+		return ($flags & P_SYS) ? ZBX_VALID_ERROR : ZBX_VALID_WARNING;
 	}
+
 	return ZBX_VALID_OK;
 }
 
@@ -520,7 +461,19 @@ function invalid_url($msg = null) {
 	if (empty($msg)) {
 		$msg = _('Zabbix has received an incorrect request.');
 	}
+
+	// required global parameters for correct including page_header.php
+	global $DB, $ZBX_MESSAGES;
+
+	// backup messages before including page_header.php
+	$temp = $ZBX_MESSAGES;
+	$ZBX_MESSAGES = null;
+
 	require_once dirname(__FILE__).'/page_header.php';
+
+	// rollback reseted messages
+	$ZBX_MESSAGES = $temp;
+
 	unset_all();
 	show_error_message($msg);
 	require_once dirname(__FILE__).'/page_footer.php';
@@ -529,13 +482,13 @@ function invalid_url($msg = null) {
 function check_fields(&$fields, $show_messages = true) {
 	// VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 	$system_fields = array(
-		'sid' =>		array(T_ZBX_STR, O_OPT, P_SYS, HEX(),		null),
+		'sid' =>			array(T_ZBX_STR, O_OPT, P_SYS, HEX(),		null),
 		'switch_node' =>	array(T_ZBX_INT, O_OPT, P_SYS, DB_ID,		null),
 		'triggers_hash' =>	array(T_ZBX_STR, O_OPT, P_SYS, NOT_EMPTY,	null),
-		'print' =>		array(T_ZBX_INT, O_OPT, P_SYS, IN('1'),		null),
-		'sort' =>		array(T_ZBX_STR, O_OPT, P_SYS, null,		null),
+		'print' =>			array(T_ZBX_INT, O_OPT, P_SYS, IN('1'),		null),
+		'sort' =>			array(T_ZBX_STR, O_OPT, P_SYS, null,		null),
 		'sortorder' =>		array(T_ZBX_STR, O_OPT, P_SYS, null,		null),
-		'page' =>		array(T_ZBX_INT, O_OPT, P_SYS, null,		null), // paging
+		'page' =>			array(T_ZBX_INT, O_OPT, P_SYS, null,		null), // paging
 		'ddreset' =>		array(T_ZBX_INT, O_OPT, P_SYS, null,		null)
 	);
 	$fields = zbx_array_merge($system_fields, $fields);
@@ -547,6 +500,7 @@ function check_fields(&$fields, $show_messages = true) {
 
 	unset_not_in_list($fields);
 	unset_if_zero($fields);
+
 	if ($err != ZBX_VALID_OK) {
 		unset_action_vars($fields);
 	}
@@ -558,10 +512,10 @@ function check_fields(&$fields, $show_messages = true) {
 	}
 
 	if ($show_messages && $err != ZBX_VALID_OK) {
-		show_messages($err == ZBX_VALID_OK, null, _('Page received incorrect data'));
+		show_messages(($err == ZBX_VALID_OK), null, _('Page received incorrect data'));
 	}
 
-	return $err == ZBX_VALID_OK;
+	return ($err == ZBX_VALID_OK);
 }
 
 function validatePortNumberOrMacro($port) {
@@ -592,6 +546,46 @@ function validateUserMacro($value) {
 	return preg_match('/^'.ZBX_PREG_EXPRESSION_USER_MACROS.'$/', $value);
 }
 
-function validateMaxTime($time) {
-	return $time <= 2147464800; // 2038.01.19 00:00
+/**
+ * Validate, if unix time in (1970.01.01 00:00:01 - 2038.01.19 00:00:00).
+ *
+ * @param int $time
+ *
+ * @return bool
+ */
+function validateUnixTime($time) {
+	return (is_numeric($time) && $time > 0 && $time <= 2147464800);
+}
+
+/**
+ * Validate if date and time are in correct range, e.g. month is not greater than 12 etc.
+ *
+ * @param int $year
+ * @param int $month
+ * @param int $day
+ * @param int $minutes
+ * @param int $seconds
+ *
+ * @return bool
+ */
+function validateDateTime($year, $month, $day, $hours, $minutes, $seconds = null) {
+	return !($month < 1 || $month > 12
+			|| $day < 1  || $day > 31 || (($month == 4 || $month == 6 || $month == 9 || $month == 11) && $day > 30)
+			|| ($month == 2 && ((($year % 4) == 0 && $day > 29) || (($year % 4) != 0 && $day > 28)))
+			|| $hours < 0 || $hours > 23
+			|| $minutes < 0 || $minutes > 59
+			|| (!is_null($seconds) && ($seconds < 0 || $seconds > 59)));
+}
+
+/**
+ * Validate allowed date interval (1970.01.01-2038.01.18).
+ *
+ * @param int $year
+ * @param int $month
+ * @param int $day
+ *
+ * @return bool
+ */
+function validateDateInterval($year, $month, $day) {
+	return !($year < 1970 || $year > 2038 || ($year == 2038 && (($month > 1) || ($month == 1 && $day > 18))));
 }
