@@ -17,22 +17,25 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
+
+
 $imageForm = new CForm('post', null, 'multipart/form-data');
 $imageForm->setName('imageForm');
 $imageForm->addVar('form', $this->data['form']);
 $imageForm->addVar('imageid', $this->data['imageid']);
 
-$imageCb = new CComboBox('imagetype', $this->data['imagetype']);
-$imageCb->addItem(IMAGE_TYPE_ICON, _('Icon'));
-$imageCb->addItem(IMAGE_TYPE_BACKGROUND, _('Background'));
+$imageComboBox = new CComboBox('imagetype', $this->data['imagetype']);
+$imageComboBox->addItem(IMAGE_TYPE_ICON, _('Icon'));
+$imageComboBox->addItem(IMAGE_TYPE_BACKGROUND, _('Background'));
 
 // append form list
 $imageFormList = new CFormList('imageFormList');
-$imageFormList->addRow(_('Name'), new CTextBox('name', $this->data['imagename'], 64, 'no', 64));
-$imageFormList->addRow(_('Type'), $imageCb);
+$nameTextBox = new CTextBox('name', $this->data['imagename'], 64, 'no', 64);
+$nameTextBox->attr('autofocus', 'autofocus');
+$imageFormList->addRow(_('Name'), $nameTextBox);
+$imageFormList->addRow(_('Type'), $imageComboBox);
 $imageFormList->addRow(_('Upload'), new CFile('image'));
+
 if (!empty($this->data['imageid'])) {
 	if ($data['imagetype'] == IMAGE_TYPE_BACKGROUND) {
 		$imageFormList->addRow(_('Image'), new CLink(new CImg('imgstore.php?width=200&height=200&iconid='.$this->data['imageid'], 'no image'), 'image.php?imageid='.$this->data['imageid']));
@@ -45,16 +48,20 @@ if (!empty($this->data['imageid'])) {
 // append tab
 $imageTab = new CTabView();
 $imageTab->addTab('imageTab', _('Image'), $imageFormList);
+$imageForm->addItem($imageTab);
 
 // append buttons
-$imageForm->addItem($imageTab);
-if (!empty($this->data['imageid'])) {
-	$imageForm->addItem(makeFormFooter(array(new CSubmit('save', _('Save'))), array(new CButtonDelete(_('Delete selected image?'), url_param('form').url_param('imageid')), new CButtonCancel())));
+if (empty($this->data['imageid'])) {
+	$imageForm->addItem(makeFormFooter(new CSubmit('save', _('Save')), new CButtonCancel()));
 }
 else {
-	$imageForm->addItem(makeFormFooter(array(new CSubmit('save', _('Save'))), new CButtonCancel()));
+	$imageForm->addItem(makeFormFooter(
+		new CSubmit('save', _('Save')),
+		array(
+			new CButtonDelete(_('Delete selected image?'), url_param('form').url_param('imageid')),
+			new CButtonCancel()
+		)
+	));
 }
 
 return $imageForm;
-?>
-
