@@ -17,8 +17,8 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
+
+
 $itemsWidget = new CWidget();
 
 // create new item button
@@ -29,18 +29,18 @@ $createForm->addItem(new CSubmit('form', _('Create item prototype')));
 $itemsWidget->addPageHeader(_('CONFIGURATION OF ITEM PROTOTYPES'), $createForm);
 
 // header
-$itemsWidget->addHeader(array(_('Item prototypes of').SPACE, new CSpan($this->data['discovery_rule']['name'], 'gold')));
+$itemsWidget->addHeader(array(_('Item prototypes of').SPACE, new CSpan($this->data['discovery_rule']['name'], 'parent-discovery')));
 $itemsWidget->addHeaderRowNumber();
 $itemsWidget->addItem(get_header_host_table('items', $this->data['hostid'], $this->data['parent_discoveryid']));
 
 // create form
-$itemForm = new CForm('get');
+$itemForm = new CForm();
 $itemForm->setName('items');
 $itemForm->addVar('hostid', $this->data['hostid']);
 $itemForm->addVar('parent_discoveryid', $this->data['parent_discoveryid']);
 
 // create table
-$itemTable = new CTableInfo(_('No items prototypes defined.'));
+$itemTable = new CTableInfo(_('No item prototypes found.'));
 
 $sortLink = new CUrl();
 $sortLink->setArgument('parent_discoveryid', $this->data['parent_discoveryid']);
@@ -54,8 +54,8 @@ $itemTable->setHeader(array(
 	make_sorting_header(_('History'), 'history', $sortLink),
 	make_sorting_header(_('Trends'), 'trends', $sortLink),
 	make_sorting_header(_('Type'), 'type', $sortLink),
-	make_sorting_header(_('Status'), 'status', $sortLink),
-	_('Applications')
+	_('Applications'),
+	make_sorting_header(_('Status'), 'status', $sortLink)
 ));
 
 foreach ($this->data['items'] as $item) {
@@ -65,12 +65,19 @@ foreach ($this->data['items'] as $item) {
 		$templateDiscoveryRuleId = get_realrule_by_itemid_and_hostid($this->data['parent_discoveryid'], $template_host['hostid']);
 
 		$description[] = new CLink($template_host['name'], '?parent_discoveryid='.$templateDiscoveryRuleId, 'unknown');
-		$description[] = ': ';
+		$description[] = NAME_DELIMITER;
 	}
-	$description[] = new CLink(itemName($item), '?form=update&itemid='.$item['itemid'].'&parent_discoveryid='.$this->data['parent_discoveryid']);
+	$description[] = new CLink(
+		itemName($item),
+		'?form=update&itemid='.$item['itemid'].'&parent_discoveryid='.$this->data['parent_discoveryid']
+	);
 
-	$status = new CLink(item_status2str($item['status']), '?group_itemid='.$item['itemid'].'&parent_discoveryid='.$this->data['parent_discoveryid'].
-		'&go='.($item['status'] ? 'activate' : 'disable'), item_status2style($item['status'])
+	$status = new CLink(
+		itemIndicator($item['status']),
+		'?group_itemid='.$item['itemid'].
+			'&parent_discoveryid='.$this->data['parent_discoveryid'].
+			'&go='.($item['status'] ? 'activate' : 'disable'),
+		itemIndicatorStyle($item['status'])
 	);
 
 	if (!empty($item['applications'])) {
@@ -94,33 +101,36 @@ foreach ($this->data['items'] as $item) {
 		$item['history'],
 		in_array($item['value_type'], array(ITEM_VALUE_TYPE_STR, ITEM_VALUE_TYPE_LOG, ITEM_VALUE_TYPE_TEXT)) ? '' : $item['trends'],
 		item_type2str($item['type']),
-		$status,
 		new CCol($applications, 'wraptext'),
+		$status
 	));
 }
 
 // create go buttons
 $goComboBox = new CComboBox('go');
 $goOption = new CComboItem('activate', _('Activate selected'));
-$goOption->setAttribute('confirm', _('Enable selected items?'));
+$goOption->setAttribute('confirm', _('Enable selected item prototypes?'));
 $goComboBox->addItem($goOption);
 
 $goOption = new CComboItem('disable', _('Disable selected'));
-$goOption->setAttribute('confirm', _('Disable selected items?'));
+$goOption->setAttribute('confirm', _('Disable selected item prototypes?'));
 $goComboBox->addItem($goOption);
 
 $goOption = new CComboItem('delete', _('Delete selected'));
-$goOption->setAttribute('confirm', _('Delete selected items?'));
+$goOption->setAttribute('confirm', _('Delete selected item prototypes?'));
 $goComboBox->addItem($goOption);
 
 $goButton = new CSubmit('goButton', _('Go').' (0)');
 $goButton->setAttribute('id', 'goButton');
+
 zbx_add_post_js('chkbxRange.pageGoName = "group_itemid";');
+zbx_add_post_js('chkbxRange.prefix = "'.$this->data['parent_discoveryid'].'";');
+zbx_add_post_js('cookie.prefix = "'.$this->data['parent_discoveryid'].'";');
 
 // append table to form
 $itemForm->addItem(array($this->data['paging'], $itemTable, $this->data['paging'], get_table_header(array($goComboBox, $goButton))));
 
 // append form to widget
 $itemsWidget->addItem($itemForm);
+
 return $itemsWidget;
-?>
