@@ -43,8 +43,8 @@ else {
 
 if (isset($_REQUEST['screenitemid']) && !isset($_REQUEST['form_refresh'])) {
 	$screenItem		= $screenItems[$_REQUEST['screenitemid']];
-	$resourcetype	= $screenItem['resourcetype'];
-	$resourceid		= $screenItem['resourceid'];
+	$resourceType	= $screenItem['resourcetype'];
+	$resourceId		= $screenItem['resourceid'];
 	$width			= $screenItem['width'];
 	$height			= $screenItem['height'];
 	$colspan		= $screenItem['colspan'];
@@ -55,11 +55,12 @@ if (isset($_REQUEST['screenitemid']) && !isset($_REQUEST['form_refresh'])) {
 	$style			= $screenItem['style'];
 	$url			= $screenItem['url'];
 	$dynamic		= $screenItem['dynamic'];
-	$sort_triggers	= $screenItem['sort_triggers'];
+	$sortTriggers	= $screenItem['sort_triggers'];
+	$application	= $screenItem['application'];
 }
 else {
-	$resourcetype	= get_request('resourcetype', 0);
-	$resourceid		= get_request('resourceid', 0);
+	$resourceType	= get_request('resourcetype', 0);
+	$resourceId		= get_request('resourceid', 0);
 	$width			= get_request('width', 500);
 	$height			= get_request('height', 100);
 	$colspan		= get_request('colspan', 1);
@@ -70,11 +71,12 @@ else {
 	$style			= get_request('style', 0);
 	$url			= get_request('url', '');
 	$dynamic		= get_request('dynamic', SCREEN_SIMPLE_ITEM);
-	$sort_triggers	= get_request('sort_triggers', SCREEN_SORT_TRIGGERS_DATE_DESC);
+	$sortTriggers	= get_request('sort_triggers', SCREEN_SORT_TRIGGERS_DATE_DESC);
+	$application	= get_request('application', '');
 }
 
 // append resource types to form list
-$resourceTypeComboBox = new CComboBox('resourcetype', $resourcetype, 'submit()');
+$resourceTypeComboBox = new CComboBox('resourcetype', $resourceType, 'submit()');
 $screenResources = screen_resources();
 if ($this->data['screen']['templateid']) {
 	unset(
@@ -90,27 +92,25 @@ $resourceTypeComboBox->addItems($screenResources);
 $screenFormList->addRow(_('Resource'), $resourceTypeComboBox);
 
 /*
- * Graph
+ * Screen item: Graph
  */
-if ($resourcetype == SCREEN_RESOURCE_GRAPH) {
+if ($resourceType == SCREEN_RESOURCE_GRAPH) {
 	$caption = '';
 	$id = 0;
 
 	$graphs = API::Graph()->get(array(
-		'graphids' => $resourceid,
+		'graphids' => $resourceId,
 		'selectHosts' => array('hostid', 'name', 'status'),
 		'output' => API_OUTPUT_EXTEND
 	));
 	if (!empty($graphs)) {
-		$id = $resourceid;
+		$id = $resourceId;
 		$graph = reset($graphs);
 
 		order_result($graph['hosts'], 'name');
 		$graph['host'] = reset($graph['hosts']);
 
-		$caption = ($graph['host']['status'] != HOST_STATUS_TEMPLATE)
-			? $graph['host']['name'].':'.$graph['name']
-			: $graph['name'];
+		$caption = $graph['host']['name'].NAME_DELIMITER.$graph['name'];
 
 		$nodeName = get_node_name_by_elid($graph['host']['hostid']);
 		if (!zbx_empty($nodeName)) {
@@ -123,7 +123,7 @@ if ($resourcetype == SCREEN_RESOURCE_GRAPH) {
 			'javascript: return PopUp("popup.php?srctbl=graphs&srcfld1=graphid&srcfld2=name'.
 				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].
-				'&writeonly=1&simpleName=1", 800, 450);',
+				'&writeonly=1", 800, 450);',
 			'formlist'
 		);
 	}
@@ -144,25 +144,23 @@ if ($resourcetype == SCREEN_RESOURCE_GRAPH) {
 }
 
 /*
- * Simple graph
+ * Screen item: Simple graph
  */
-elseif ($resourcetype == SCREEN_RESOURCE_SIMPLE_GRAPH) {
+elseif ($resourceType == SCREEN_RESOURCE_SIMPLE_GRAPH) {
 	$caption = '';
 	$id = 0;
 
 	$items = API::Item()->get(array(
-		'itemids' => $resourceid,
-		'selectHosts' => array('hostid', 'name', 'status'),
-		'output' => API_OUTPUT_EXTEND
+		'itemids' => $resourceId,
+		'selectHosts' => array('name'),
+		'output' => array('itemid', 'key_', 'name')
 	));
 	if (!empty($items)) {
-		$id = $resourceid;
+		$id = $resourceId;
 		$item = reset($items);
 		$item['host'] = reset($item['hosts']);
 
-		$caption = $item['host']['status'] != HOST_STATUS_TEMPLATE
-			? $item['host']['name'].':'.itemName($item)
-			: itemName($item);
+		$caption = $item['host']['name'].NAME_DELIMITER.itemName($item);
 
 		$nodeName = get_node_name_by_elid($item['itemid']);
 		if (!zbx_empty($nodeName)) {
@@ -172,17 +170,17 @@ elseif ($resourcetype == SCREEN_RESOURCE_SIMPLE_GRAPH) {
 
 	if ($this->data['screen']['templateid']) {
 		$selectButton = new CButton('select', _('Select'),
-			'javascript: return PopUp("popup.php?srctbl=simple_graph&srcfld1=itemid&srcfld2=name'.
+			'javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
 				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].
-				'&simpleName=1&writeonly=1", 800, 450);', 'formlist'
+				'&templated=1&writeonly=1&numeric=1", 800, 450);', 'formlist'
 		);
 	}
 	else {
 		$selectButton = new CButton('select', _('Select'),
-			'javascript: return PopUp("popup.php?srctbl=simple_graph&srcfld1=itemid&srcfld2=name'.
+			'javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
 				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&real_hosts=1&with_simple_graph_items=1&writeonly=1", 800, 450);',
+				'&real_hosts=1&with_simple_graph_items=1&writeonly=1&templated=0&numeric=1", 800, 450);',
 			'formlist'
 		);
 	}
@@ -195,18 +193,18 @@ elseif ($resourcetype == SCREEN_RESOURCE_SIMPLE_GRAPH) {
 }
 
 /*
- * Map
+ * Screen item: Map
  */
-elseif ($resourcetype == SCREEN_RESOURCE_MAP) {
+elseif ($resourceType == SCREEN_RESOURCE_MAP) {
 	$caption = '';
 	$id = 0;
 
 	$maps = API::Map()->get(array(
-		'sysmapids' => $resourceid,
+		'sysmapids' => $resourceId,
 		'output' => API_OUTPUT_EXTEND
 	));
 	if (!empty($maps)) {
-		$id = $resourceid;
+		$id = $resourceId;
 		$map = reset($maps);
 		$caption = $map['name'];
 		$nodeName = get_node_name_by_elid($map['sysmapid']);
@@ -228,22 +226,22 @@ elseif ($resourcetype == SCREEN_RESOURCE_MAP) {
 }
 
 /*
- * Plain text
+ * Screen item: Plain text
  */
-elseif ($resourcetype == SCREEN_RESOURCE_PLAIN_TEXT) {
+elseif ($resourceType == SCREEN_RESOURCE_PLAIN_TEXT) {
 	$caption = '';
 	$id = 0;
 
 	$items = API::Item()->get(array(
-		'itemids' => $resourceid,
-		'selectHosts' => array('hostid', 'name'),
-		'output' => API_OUTPUT_EXTEND
+		'itemids' => $resourceId,
+		'selectHosts' => array('name'),
+		'output' => array('itemid', 'key_', 'name')
 	));
 	if (!empty($items)) {
-		$id = $resourceid;
+		$id = $resourceId;
 		$item = reset($items);
 		$item['host'] = reset($item['hosts']);
-		$caption = $item['host']['name'].':'.itemName($item);
+		$caption = $item['host']['name'].NAME_DELIMITER.itemName($item);
 
 		$nodeName = get_node_name_by_elid($item['itemid']);
 		if (!zbx_empty($nodeName)) {
@@ -253,7 +251,7 @@ elseif ($resourcetype == SCREEN_RESOURCE_PLAIN_TEXT) {
 
 	if ($this->data['screen']['templateid']) {
 		$selectButton = new CButton('select', _('Select'),
-			'javascript: return PopUp("popup.php?srctbl=plain_text&srcfld1=itemid&srcfld2=name'.
+			'javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
 				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].
 				'&writeonly=1", 800, 450);',
@@ -262,9 +260,9 @@ elseif ($resourcetype == SCREEN_RESOURCE_PLAIN_TEXT) {
 	}
 	else {
 		$selectButton = new CButton('select', _('Select'),
-			'javascript: return PopUp("popup.php?srctbl=plain_text&srcfld1=itemid&srcfld2=name'.
+			'javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
 				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&real_hosts=1&writeonly=1", 800, 450);',
+				'&real_hosts=1&writeonly=1&templated=0", 800, 450);',
 			'formlist'
 		);
 	}
@@ -274,70 +272,69 @@ elseif ($resourcetype == SCREEN_RESOURCE_PLAIN_TEXT) {
 		new CTextBox('caption', $caption, ZBX_TEXTBOX_STANDARD_SIZE, 'yes'),
 		$selectButton
 	));
-	$screenFormList->addRow(_('Show lines'), new CNumericBox('elements', $elements, 2));
+	$screenFormList->addRow(_('Show lines'), new CNumericBox('elements', $elements, 3));
 	$screenFormList->addRow(_('Show text as HTML'), new CCheckBox('style', $style, null, 1));
 }
 
 /*
- * Status of triggers
+ * Screen item: Status of triggers
  */
-else if(in_array($resourcetype, array(SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESOURCE_HOST_TRIGGERS))) {
-	$caption = '';
-	$id = 0;
+elseif (in_array($resourceType, array(SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESOURCE_HOST_TRIGGERS))) {
+	$data = array();
 
-	if (SCREEN_RESOURCE_HOSTGROUP_TRIGGERS == $resourcetype) {
-		if ($resourceid > 0) {
-			$groups = API::HostGroup()->get(array(
-				'groupids' => $resourceid,
-				'output' => API_OUTPUT_EXTEND,
+	if ($resourceType == SCREEN_RESOURCE_HOSTGROUP_TRIGGERS) {
+		if ($resourceId > 0) {
+			$data = API::HostGroup()->get(array(
+				'groupids' => $resourceId,
+				'output' => array('groupid', 'name'),
 				'editable' => true
 			));
-			foreach ($groups as $group) {
-				$caption = get_node_name_by_elid($group['groupid'], true, ':').$group['name'];
-				$id = $resourceid;
+
+			if ($data) {
+				$data = reset($data);
+
+				$data['prefix'] = get_node_name_by_elid($data['groupid'], true, NAME_DELIMITER);
 			}
 		}
 
-		$screenFormList->addVar('resourceid', $id);
-		$screenFormList->addRow(_('Group'), array(
-			new CTextBox('caption', $caption, ZBX_TEXTBOX_STANDARD_SIZE, 'yes'),
-			new CButton('select', _('Select'),
-				'javascript: return PopUp("popup.php?srctbl=host_group&srcfld1=groupid&srcfld2=name'.
-					'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
-					'&writeonly=1", 800, 450);',
-				'formlist'
-			)
-		));
+		$screenFormList->addRow(_('Group'), new CMultiSelect(array(
+			'name' => 'resourceid',
+			'objectName' => 'hostGroup',
+			'objectOptions' => array('editable' => true),
+			'data' => $data ? array(array('id' => $data['groupid'], 'name' => $data['name'], 'prefix' => $data['prefix'])) : null,
+			'defaultValue' => 0,
+			'selectedLimit' => 1
+		)));
 	}
 	else {
-		if ($resourceid > 0) {
-			$hosts = API::Host()->get(array(
-				'hostids' => $resourceid,
-				'output' => API_OUTPUT_EXTEND,
+		if ($resourceId > 0) {
+			$data = API::Host()->get(array(
+				'hostids' => $resourceId,
+				'output' => array('hostid', 'name'),
 				'editable' => true
 			));
-			foreach ($hosts as $host) {
-				$caption = get_node_name_by_elid($host['hostid'], true, ':').$host['name'];
-				$id = $resourceid;
+
+			if ($data) {
+				$data = reset($data);
+
+				$data['prefix'] = get_node_name_by_elid($data['hostid'], true, NAME_DELIMITER);
 			}
 		}
 
-		$screenFormList->addVar('resourceid', $id);
-		$screenFormList->addRow(_('Host'), array(
-			new CTextBox('caption', $caption, ZBX_TEXTBOX_STANDARD_SIZE, 'yes'),
-			new CButton('select', _('Select'),
-				'javascript: return PopUp("popup.php?srctbl=hosts&srcfld1=hostid&srcfld2=name'.
-					'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
-					'&real_hosts=1&writeonly=1", 800, 450);',
-				'formlist'
-			)
-		));
+		$screenFormList->addRow(_('Host'), new CMultiSelect(array(
+			'name' => 'resourceid',
+			'objectName' => 'hosts',
+			'objectOptions' => array('editable' => true),
+			'data' => $data ? array(array('id' => $data['hostid'], 'name' => $data['name'], 'prefix' => $data['prefix'])) : null,
+			'defaultValue' => 0,
+			'selectedLimit' => 1
+		)));
 	}
 
-	$screenFormList->addRow(_('Show lines'), new CNumericBox('elements', $elements, 2));
+	$screenFormList->addRow(_('Show lines'), new CNumericBox('elements', $elements, 3));
 	$screenFormList->addRow(
 		_('Sort triggers by'),
-		new CComboBox('sort_triggers', $sort_triggers, null, array(
+		new CComboBox('sort_triggers', $sortTriggers, null, array(
 			SCREEN_SORT_TRIGGERS_DATE_DESC => _('Last change (descending)'),
 			SCREEN_SORT_TRIGGERS_SEVERITY_DESC => _('Severity (descending)'),
 			SCREEN_SORT_TRIGGERS_HOST_NAME_ASC => _('Host (ascending)')
@@ -346,13 +343,13 @@ else if(in_array($resourcetype, array(SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN
 }
 
 /*
- * History of actions
+ * Screen item: History of actions
  */
-elseif ($resourcetype == SCREEN_RESOURCE_ACTIONS) {
-	$screenFormList->addRow(_('Show lines'), new CNumericBox('elements', $elements, 2));
+elseif ($resourceType == SCREEN_RESOURCE_ACTIONS) {
+	$screenFormList->addRow(_('Show lines'), new CNumericBox('elements', $elements, 3));
 	$screenFormList->addRow(
 		_('Sort triggers by'),
-		new CComboBox('sort_triggers', $sort_triggers, null, array(
+		new CComboBox('sort_triggers', $sortTriggers, null, array(
 			SCREEN_SORT_TRIGGERS_TIME_DESC => _('Time (descending)'),
 			SCREEN_SORT_TRIGGERS_TIME_ASC => _('Time (ascending)'),
 			SCREEN_SORT_TRIGGERS_TYPE_DESC => _('Type (descending)'),
@@ -369,62 +366,61 @@ elseif ($resourcetype == SCREEN_RESOURCE_ACTIONS) {
 }
 
 /*
- * History of events
+ * Screen item: History of events
  */
-elseif ($resourcetype == SCREEN_RESOURCE_EVENTS) {
-	$screenFormList->addRow(_('Show lines'), new CNumericBox('elements', $elements, 2));
+elseif ($resourceType == SCREEN_RESOURCE_EVENTS) {
+	$screenFormList->addRow(_('Show lines'), new CNumericBox('elements', $elements, 3));
 	$screenFormList->addVar('resourceid', 0);
 }
 
 /*
- * Overviews
+ * Screen item: Overviews
  */
-elseif (in_array($resourcetype, array(SCREEN_RESOURCE_TRIGGERS_OVERVIEW, SCREEN_RESOURCE_DATA_OVERVIEW))) {
-	$caption = '';
-	$id = 0;
+elseif (in_array($resourceType, array(SCREEN_RESOURCE_TRIGGERS_OVERVIEW, SCREEN_RESOURCE_DATA_OVERVIEW))) {
+	$data = array();
 
-	if ($resourceid > 0) {
-		$groups = API::HostGroup()->get(array(
-			'groupids' => $resourceid,
-			'output' => API_OUTPUT_EXTEND,
+	if ($resourceId > 0) {
+		$data = API::HostGroup()->get(array(
+			'groupids' => $resourceId,
+			'output' => array('groupid', 'name'),
 			'editable' => true
 		));
-		foreach ($groups as $group) {
-			$caption = get_node_name_by_elid($group['groupid'], true, ':').$group['name'];
-			$id = $resourceid;
+
+		if ($data) {
+			$data = reset($data);
+
+			$data['prefix'] = get_node_name_by_elid($data['groupid'], true, NAME_DELIMITER);
 		}
 	}
 
-	$screenFormList->addVar('resourceid', $id);
-	$screenFormList->addRow(_('Group'), array(
-		new CTextBox('caption', $caption, ZBX_TEXTBOX_STANDARD_SIZE, 'yes'),
-		new CButton('select', _('Select'),
-			'javascript: return PopUp("popup.php?srctbl=overview&srcfld1=groupid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&writeonly=1", 800, 450);',
-			'formlist'
-		)
-	));
+	$screenFormList->addRow(_('Group'), new CMultiSelect(array(
+		'name' => 'resourceid',
+		'objectName' => 'hostGroup',
+		'objectOptions' => array('editable' => true),
+		'data' => $data ? array(array('id' => $data['groupid'], 'name' => $data['name'], 'prefix' => $data['prefix'])) : null,
+		'selectedLimit' => 1
+	)));
+	$screenFormList->addRow(_('Application'), new CTextBox('application', $application, ZBX_TEXTBOX_STANDARD_SIZE, false, 255));
 }
 
 /*
- * Screens
+ * Screen item: Screens
  */
-elseif ($resourcetype == SCREEN_RESOURCE_SCREEN) {
+elseif ($resourceType == SCREEN_RESOURCE_SCREEN) {
 	$caption = '';
 	$id = 0;
 
-	if ($resourceid > 0) {
+	if ($resourceId > 0) {
 		$db_screens = DBselect(
 			'SELECT DISTINCT n.name AS node_name,s.screenid,s.name'.
 			' FROM screens s'.
 				' LEFT JOIN nodes n ON n.nodeid='.DBid2nodeid('s.screenid').
-			' WHERE s.screenid='.$resourceid
+			' WHERE s.screenid='.zbx_dbstr($resourceId)
 		);
 		while ($row = DBfetch($db_screens)) {
 			$screen = API::Screen()->get(array(
 				'screenids' => $row['screenid'],
-				'output' => API_OUTPUT_SHORTEN
+				'output' => array('screenid')
 			));
 			if (empty($screen)) {
 				continue;
@@ -435,7 +431,7 @@ elseif ($resourcetype == SCREEN_RESOURCE_SCREEN) {
 
 			$row['node_name'] = !empty($row['node_name']) ? '('.$row['node_name'].') ' : '';
 			$caption = $row['node_name'].$row['name'];
-			$id = $resourceid;
+			$id = $resourceId;
 		}
 	}
 
@@ -452,60 +448,56 @@ elseif ($resourcetype == SCREEN_RESOURCE_SCREEN) {
 }
 
 /*
- * Hosts info
+ * Screen item: Hosts info
  */
-elseif ($resourcetype == SCREEN_RESOURCE_HOSTS_INFO || $resourcetype == SCREEN_RESOURCE_TRIGGERS_INFO) {
-	$caption = '';
-	$id = 0;
+elseif ($resourceType == SCREEN_RESOURCE_HOSTS_INFO || $resourceType == SCREEN_RESOURCE_TRIGGERS_INFO) {
+	$data = array();
 
-	if (remove_nodes_from_id($resourceid) > 0) {
-		$groups = API::HostGroup()->get(array(
-			'groupids' => $resourceid,
+	if ($resourceId > 0) {
+		$data = API::HostGroup()->get(array(
+			'groupids' => $resourceId,
 			'nodeids' => get_current_nodeid(true),
-			'output' => array('name'),
-			'preservekeys' => true
+			'output' => array('groupid', 'name'),
+			'editable' => true
 		));
-		if ($group = reset($groups)) {
-			$caption = get_node_name_by_elid($resourceid, true, ': ').$group['name'];
-			$id = $resourceid;
-		}
-	}
-	elseif (remove_nodes_from_id($resourceid) == 0) {
-		if ($nodeName = get_node_name_by_elid($resourceid, true, ': ')) {
-			$caption = $nodeName._('- all groups -');
-			$id = $resourceid;
+
+		if ($data) {
+			$data = reset($data);
+
+			$data['prefix'] = get_node_name_by_elid($data['groupid'], true, NAME_DELIMITER);
 		}
 	}
 
-	$screenFormList->addVar('resourceid', $id);
-	$screenFormList->addRow(_('Group'), array(
-		new CTextBox('caption', $caption, ZBX_TEXTBOX_STANDARD_SIZE, 'yes'),
-		new CButton('select', _('Select'),
-			'javascript: return PopUp("popup.php?srctbl=host_group_scr&srcfld1=groupid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&writeonly=1", 480, 450);',
-			'formlist'
-		)
-	));
+	$screenFormList->addRow(_('Group'), new CMultiSelect(array(
+		'name' => 'resourceid',
+		'objectName' => 'hostGroup',
+		'objectOptions' => array('editable' => true),
+		'data' => $data ? array(array('id' => $data['groupid'], 'name' => $data['name'], 'prefix' => $data['prefix'])) : null,
+		'defaultValue' => 0,
+		'selectedLimit' => 1
+	)));
 }
 
 /*
- * Clock
+ * Screen item: Clock
  */
-elseif ($resourcetype == SCREEN_RESOURCE_CLOCK) {
+elseif ($resourceType == SCREEN_RESOURCE_CLOCK) {
 	$caption = get_request('caption', '');
-	if (zbx_empty($caption) && TIME_TYPE_HOST == $style && $resourceid > 0) {
+
+	if (zbx_empty($caption) && TIME_TYPE_HOST == $style && $resourceId > 0) {
 		$items = API::Item()->get(array(
-			'itemids' => $resourceid,
+			'itemids' => $resourceId,
 			'selectHosts' => array('name'),
-			'output' => API_OUTPUT_EXTEND
+			'output' => array('itemid', 'key_', 'name')
 		));
-		$item = reset($items);
-		$host = reset($item['hosts']);
-		$caption = $host['name'].':'.$item['name'];
+		if ($items) {
+			$item = reset($items);
+			$host = reset($item['hosts']);
+			$caption = $host['name'].NAME_DELIMITER.itemName($item);
+		}
 	}
 
-	$screenFormList->addVar('resourceid', $resourceid);
+	$screenFormList->addVar('resourceid', $resourceId);
 
 	$styleComboBox = new CComboBox('style', $style, 'javascript: submit();');
 	$styleComboBox->addItem(TIME_TYPE_LOCAL, _('Local time'));
@@ -540,7 +532,10 @@ else {
 	$screenFormList->addVar('resourceid', 0);
 }
 
-if (in_array($resourcetype, array(SCREEN_RESOURCE_HOSTS_INFO, SCREEN_RESOURCE_TRIGGERS_INFO))) {
+/*
+ * Append common fields
+ */
+if (in_array($resourceType, array(SCREEN_RESOURCE_HOSTS_INFO, SCREEN_RESOURCE_TRIGGERS_INFO))) {
 	$styleRadioButton = array(
 		new CRadioButton('style', STYLE_HORISONTAL, null, 'style_'.STYLE_HORISONTAL, $style == STYLE_HORISONTAL),
 		new CLabel(_('Horizontal'), 'style_'.STYLE_HORISONTAL),
@@ -549,7 +544,7 @@ if (in_array($resourcetype, array(SCREEN_RESOURCE_HOSTS_INFO, SCREEN_RESOURCE_TR
 	);
 	$screenFormList->addRow(_('Style'), new CDiv($styleRadioButton, 'jqueryinputset'));
 }
-elseif (in_array($resourcetype, array(SCREEN_RESOURCE_TRIGGERS_OVERVIEW,SCREEN_RESOURCE_DATA_OVERVIEW))) {
+elseif (in_array($resourceType, array(SCREEN_RESOURCE_TRIGGERS_OVERVIEW,SCREEN_RESOURCE_DATA_OVERVIEW))) {
 	$styleRadioButton = array(
 		new CRadioButton('style', STYLE_LEFT, null, 'style_'.STYLE_LEFT, $style == STYLE_LEFT),
 		new CLabel(_('Left'), 'style_'.STYLE_LEFT),
@@ -562,14 +557,14 @@ else {
 	$screenFormList->addVar('style', 0);
 }
 
-if (in_array($resourcetype, array(SCREEN_RESOURCE_URL))) {
+if (in_array($resourceType, array(SCREEN_RESOURCE_URL))) {
 	$screenFormList->addRow(_('Url'), new CTextBox('url', $url, ZBX_TEXTBOX_STANDARD_SIZE));
 }
 else {
 	$screenFormList->addVar('url', '');
 }
 
-if (in_array($resourcetype, array(SCREEN_RESOURCE_GRAPH, SCREEN_RESOURCE_SIMPLE_GRAPH, SCREEN_RESOURCE_CLOCK, SCREEN_RESOURCE_URL))) {
+if (in_array($resourceType, array(SCREEN_RESOURCE_GRAPH, SCREEN_RESOURCE_SIMPLE_GRAPH, SCREEN_RESOURCE_CLOCK, SCREEN_RESOURCE_URL))) {
 	$screenFormList->addRow(_('Width'), new CNumericBox('width', $width, 5));
 	$screenFormList->addRow(_('Height'), new CNumericBox('height', $height, 5));
 }
@@ -578,7 +573,7 @@ else {
 	$screenFormList->addVar('height', 100);
 }
 
-if (in_array($resourcetype, array(SCREEN_RESOURCE_GRAPH, SCREEN_RESOURCE_SIMPLE_GRAPH, SCREEN_RESOURCE_MAP, SCREEN_RESOURCE_CLOCK, SCREEN_RESOURCE_URL))) {
+if (in_array($resourceType, array(SCREEN_RESOURCE_GRAPH, SCREEN_RESOURCE_SIMPLE_GRAPH, SCREEN_RESOURCE_MAP, SCREEN_RESOURCE_CLOCK, SCREEN_RESOURCE_URL))) {
 	$hightAlignRadioButton = array(
 		new CRadioButton('halign', HALIGN_LEFT, null, 'halign_'.HALIGN_LEFT, $halign == HALIGN_LEFT),
 		new CLabel(_('Left'), 'halign_'.HALIGN_LEFT),
@@ -606,7 +601,7 @@ $screenFormList->addRow(_('Column span'), new CNumericBox('colspan', $colspan, 3
 $screenFormList->addRow(_('Row span'), new CNumericBox('rowspan', $rowspan, 3));
 
 // dynamic addon
-if ($this->data['screen']['templateid'] == 0 && in_array($resourcetype, array(SCREEN_RESOURCE_GRAPH, SCREEN_RESOURCE_SIMPLE_GRAPH, SCREEN_RESOURCE_PLAIN_TEXT))) {
+if ($this->data['screen']['templateid'] == 0 && in_array($resourceType, array(SCREEN_RESOURCE_GRAPH, SCREEN_RESOURCE_SIMPLE_GRAPH, SCREEN_RESOURCE_PLAIN_TEXT))) {
 	$screenFormList->addRow(_('Dynamic item'), new CCheckBox('dynamic', $dynamic, null, 1));
 }
 
