@@ -19,10 +19,8 @@
 **/
 
 
-require_once dirname(__FILE__).'/js/general.script.confirm.js.php';
-
 $screenWidget = new CWidget();
-$screenWidget->addPageHeader(_('CONFIGURATION OF SCREEN'));
+$screenWidget->addPageHeader(_('CONFIGURATION OF SCREENS'));
 $screenWidget->addHeader($this->data['screen']['name']);
 if (!empty($this->data['screen']['templateid'])) {
 	$screenWidget->addItem(get_header_host_table('screens', $this->data['screen']['templateid']));
