@@ -19,11 +19,8 @@
 **/
 
 
-require_once dirname(__FILE__).'/js/general.script.confirm.js.php';
-require_once dirname(__FILE__).'/js/monitoring.maps.js.php';
-
 $mapWidget = new CWidget('hat_maps');
-$mapTable = new CTable(_('No maps defined.'), 'map map-container');
+$mapTable = new CTable(_('No maps found.'), 'map map-container');
 $mapTable->setAttribute('style', 'margin-top: 4px;');
 
 $icon = $fsIcon = null;
@@ -31,14 +28,18 @@ $icon = $fsIcon = null;
 if (!empty($this->data['maps'])) {
 	$mapComboBox = new CComboBox('sysmapid', get_request('sysmapid', 0), 'submit()');
 	foreach ($this->data['maps'] as $sysmapId => $map) {
-		$mapComboBox->addItem($sysmapId, get_node_name_by_elid($sysmapId, null, ': ').$map['name']);
+		$mapComboBox->addItem($sysmapId, get_node_name_by_elid($sysmapId, null, NAME_DELIMITER).$map['name']);
 	}
 
-	$headerForm = new CForm('get');
-	$headerForm->addVar('fullscreen', $this->data['fullscreen']);
-	$headerForm->addItem($mapComboBox);
+	$headerMapForm = new CForm('get');
+	$headerMapForm->addVar('fullscreen', $this->data['fullscreen']);
+	$headerMapForm->addItem(array(_('Maps'), SPACE, $mapComboBox));
 
-	$mapWidget->addHeader($this->data['map']['name'], $headerForm);
+	$headerSeverityMinForm = new CForm('get');
+	$headerSeverityMinForm->addVar('fullscreen', $this->data['fullscreen']);
+	$headerSeverityMinForm->addItem(array(SPACE, _('Minimum severity'), SPACE, $this->data['pageFilter']->getSeveritiesMinCB()));
+
+	$mapWidget->addHeader($this->data['map']['name'], array($headerMapForm, $headerSeverityMinForm));
 
 	// get map parent maps
 	$parentMaps = array();
@@ -46,7 +47,7 @@ if (!empty($this->data['maps'])) {
 		// check for permissions
 		if (isset($this->data['maps'][$parent['sysmapid']])) {
 			$parentMaps[] = SPACE.SPACE;
-			$parentMaps[] = new Clink($parent['name'], 'maps.php?sysmapid='.$parent['sysmapid'].'&fullscreen='.$this->data['fullscreen']);
+			$parentMaps[] = new Clink($parent['name'], 'maps.php?sysmapid='.$parent['sysmapid'].'&fullscreen='.$this->data['fullscreen'].'&severity_min='.$this->data['severity_min']);
 		}
 	}
 	if (!empty($parentMaps)) {
@@ -54,11 +55,11 @@ if (!empty($this->data['maps'])) {
 		$mapWidget->addHeader($parentMaps);
 	}
 
-	$actionMap = getActionMapBySysmap($this->data['map']);
+	$actionMap = getActionMapBySysmap($this->data['map'], array('severity_min' => $this->data['severity_min']));
 
 	$mapTable->addRow($actionMap);
 
-	$imgMap = new CImg('map.php?sysmapid='.$this->data['sysmapid']);
+	$imgMap = new CImg('map.php?sysmapid='.$this->data['sysmapid'].'&severity_min='.$this->data['severity_min']);
 	$imgMap->setMap($actionMap->getName());
 	$mapTable->addRow($imgMap);
 
