@@ -56,14 +56,42 @@ $fields = array(
 );
 check_fields($fields);
 
+/*
+ * Permissions
+ */
+// validate group IDs
+$validateGroupIds = array_filter(array(
+	get_request('groupid'),
+	get_request('tr_groupid')
+));
+if ($validateGroupIds && !API::HostGroup()->isReadable($validateGroupIds)) {
+	access_deny();
+}
+
+// validate host IDs
+$validateHostIds = array_filter(array(
+	get_request('hostid'),
+	get_request('tr_hostid')
+));
+if ($validateHostIds && !API::Host()->isReadable($validateHostIds)) {
+	access_deny();
+}
+
+if (get_request('elementid')) {
+	$screens = API::Screen()->get(array(
+		'screenids' => array($_REQUEST['elementid']),
+		'output' => array('screenid')
+	));
+	if (!$screens) {
+		access_deny();
+	}
+}
+
+
 /*
  * Filter
  */
 if (isset($_REQUEST['favobj'])) {
-	if ($_REQUEST['favobj'] == 'hat') {
-		CProfile::update('web.screens.hats.'.$_REQUEST['favref'].'.state', $_REQUEST['favstate'], PROFILE_TYPE_INT);
-	}
-
 	if ($_REQUEST['favobj'] == 'filter') {
 		CProfile::update('web.screens.filter.state', $_REQUEST['favstate'], PROFILE_TYPE_INT);
 	}
@@ -77,14 +105,14 @@ if (isset($_REQUEST['favobj'])) {
 	if (str_in_array($_REQUEST['favobj'], array('screenid', 'slideshowid'))) {
 		$result = false;
 		if ($_REQUEST['favaction'] == 'add') {
-			$result = add2favorites('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
+			$result = CFavorite::add('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
 			if ($result) {
 				echo '$("addrm_fav").title = "'._('Remove from favourites').'";'."\n".
 					'$("addrm_fav").onclick = function() { rm4favorites("'.$_REQUEST['favobj'].'", "'.$_REQUEST['favid'].'", 0); }'."\n";
 			}
 		}
 		elseif ($_REQUEST['favaction'] == 'remove') {
-			$result = rm4favorites('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
+			$result = CFavorite::remove('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
 			if ($result) {
 				echo '$("addrm_fav").title = "'._('Add to favourites').'";'."\n".
 					'$("addrm_fav").onclick = function() { add2favorites("'.$_REQUEST['favobj'].'", "'.$_REQUEST['favid'].'"); }'."\n";
@@ -113,7 +141,7 @@ if ($page['type'] == PAGE_TYPE_JS || $page['type'] == PAGE_TYPE_HTML_BLOCK) {
  * Display
  */
 $data = array(
-	'fullscreen' => get_request('fullscreen'),
+	'fullscreen' => $_REQUEST['fullscreen'],
 	'period' => get_request('period'),
 	'stime' => get_request('stime'),
 	'elementid' => get_request('elementid', false),
@@ -126,12 +154,6 @@ $data = array(
 if (empty($data['elementid']) && !$data['use_screen_name']) {
 	// get element id saved in profile from the last visit
 	$data['elementid'] = CProfile::get('web.screens.elementid', null);
-
-	// this flag will be used in case this element does not exist
-	$data['id_has_been_fetched_from_profile'] = true;
-}
-else {
-	$data['id_has_been_fetched_from_profile'] = false;
 }
 
 $data['screens'] = API::Screen()->get(array(
