@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -38,8 +38,6 @@ require_once dirname(__FILE__).'/include/page_header.php';
 $fields = array(
 	'groupid' =>		array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,	null),
 	'hostid' =>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,	null),
-	'tr_groupid' =>		array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,	null),
-	'tr_hostid' =>		array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,	null),
 	'elementid' =>		array(T_ZBX_INT, O_OPT, P_SYS|P_NZERO, DB_ID, null),
 	'step' =>			array(T_ZBX_INT, O_OPT, P_SYS,	BETWEEN(0, 65535), null),
 	'period' =>			array(T_ZBX_INT, O_OPT, P_SYS,	null,	null),
@@ -54,11 +52,25 @@ $fields = array(
 	'pmasterid' =>		array(T_ZBX_STR, O_OPT, P_SYS,	null,	null),
 	// actions
 	'favaction' =>		array(T_ZBX_STR, O_OPT, P_ACT,	IN("'add','remove','refresh','flop'"), null),
-	'favstate' =>		array(T_ZBX_INT, O_OPT, P_ACT,	NOT_EMPTY, 'isset({favaction})&&("flop"=={favaction})'),
+	'favstate' =>		array(T_ZBX_INT, O_OPT, P_ACT,	NOT_EMPTY, 'isset({favaction})&&"flop"=={favaction}'),
 	'upd_counter' =>	array(T_ZBX_INT, O_OPT, P_ACT,	null,	null)
 );
 check_fields($fields);
 
+/*
+ * Permissions
+ */
+if (get_request('groupid') && !API::HostGroup()->isReadable(array($_REQUEST['groupid']))
+		|| get_request('hostid') && !API::Host()->isReadable(array($_REQUEST['hostid']))) {
+	access_deny();
+}
+if (get_request('elementid')) {
+	$slideshow = get_slideshow_by_slideshowid($_REQUEST['elementid']);
+	if (!$slideshow) {
+		access_deny();
+	}
+}
+
 /*
  * Actions
  */
@@ -71,17 +83,17 @@ if (isset($_REQUEST['favobj'])) {
 	elseif (str_in_array($_REQUEST['favobj'], array('screenid', 'slideshowid'))) {
 		$result = false;
 		if ($_REQUEST['favaction'] == 'add') {
-			$result = add2favorites('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
+			$result = CFavorite::add('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
 			if ($result) {
-				echo 'jQuery("#addrm_fav").title = "'._('Remove from').' '._('Favourites').'";'."\n".
-					'jQuery("#addrm_fav").click(function() { rm4favorites("'.$_REQUEST['favobj'].'", "'.$_REQUEST['favid'].'", 0); });'."\n";
+				echo '$("addrm_fav").title = "'._('Remove from').' '._('Favourites').'";'."\n".
+					'$("addrm_fav").onclick = function() { rm4favorites("'.$_REQUEST['favobj'].'", "'.$_REQUEST['favid'].'", 0); };'."\n";
 			}
 		}
 		elseif ($_REQUEST['favaction'] == 'remove') {
-			$result = rm4favorites('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
+			$result = CFavorite::remove('web.favorite.screenids', $_REQUEST['favid'], $_REQUEST['favobj']);
 			if ($result) {
-				echo 'jQuery("#addrm_fav").title = "'._('Add to').' '._('Favourites').'";'."\n".
-					'jQuery("#addrm_fav").click(function() { add2favorites("'.$_REQUEST['favobj'].'", "'.$_REQUEST['favid'].'"); });'."\n";
+				echo '$("addrm_fav").title = "'._('Add to').' '._('Favourites').'";'."\n".
+					'$("addrm_fav").onclick = function() { add2favorites("'.$_REQUEST['favobj'].'", "'.$_REQUEST['favid'].'"); };'."\n";
 			}
 		}
 		if ($page['type'] == PAGE_TYPE_JS && $result) {
@@ -130,6 +142,7 @@ if (isset($_REQUEST['favobj'])) {
 						'timeline' => $screenBuilder->timeline,
 						'profileIdx' => $screenBuilder->profileIdx
 					));
+
 					insertPagePostJs();
 
 					// insert slide show refresh js
@@ -176,6 +189,7 @@ if ($page['type'] == PAGE_TYPE_JS || $page['type'] == PAGE_TYPE_HTML_BLOCK) {
  * Display
  */
 $data = array(
+	'fullscreen' => $_REQUEST['fullscreen'],
 	'slideshows' => array()
 );
 
@@ -183,10 +197,10 @@ $data = array(
 $db_slideshows = DBselect(
 	'SELECT s.slideshowid,s.name'.
 	' FROM slideshows s'.
-	' WHERE '.DBin_node('s.slideshowid')
+	whereDbNode('s.slideshowid')
 );
 while ($slideshow = DBfetch($db_slideshows)) {
-	if (slideshow_accessible($slideshow['slideshowid'], PERM_READ_ONLY)) {
+	if (slideshow_accessible($slideshow['slideshowid'], PERM_READ)) {
 		$data['slideshows'][$slideshow['slideshowid']] = $slideshow;
 	}
 };
@@ -194,7 +208,6 @@ order_result($data['slideshows'], 'name');
 
 // get element id
 $data['elementid'] = get_request('elementid', CProfile::get('web.slides.elementid', null));
-$data['fullscreen'] = get_request('fullscreen', null);
 
 CProfile::update('web.slides.elementid', $data['elementid'], PROFILE_TYPE_ID);
 
@@ -204,25 +217,25 @@ if (!isset($data['slideshows'][$data['elementid']])) {
 }
 
 // get screen
-$data['screen'] = !empty($data['elementid']) ? get_slideshow($data['elementid'], 0) : array();
+$data['screen'] = empty($data['elementid']) ? array() : get_slideshow($data['elementid'], 0);
 if (!empty($data['screen'])) {
 	// get groups and hosts
 	if (check_dynamic_items($data['elementid'], 1)) {
-		$data['hostid'] = get_request('hostid', 0);
+		$data['isDynamicItems'] = true;
 
-		$options = array('allow_all_hosts', 'monitored_hosts', 'with_items');
-		if (!$ZBX_WITH_ALL_NODES) {
-			array_push($options, 'only_current_node');
-		}
-		$params = array();
-		foreach ($options as $option) {
-			$params[$option] = 1;
-		}
-
-		$data['page_groups'] = get_viewed_groups(PERM_READ_ONLY, $params);
-		$data['page_hosts'] = get_viewed_hosts(PERM_READ_ONLY, $data['page_groups']['selected'], $params);
-
-		validate_group_with_host($data['page_groups'], $data['page_hosts']);
+		$data['pageFilter'] = new CPageFilter(array(
+			'groups' => array(
+				'monitored_hosts' => true,
+				'with_items' => true
+			),
+			'hosts' => array(
+				'monitored_hosts' => true,
+				'with_items' => true,
+				'DDFirstLabel' => _('Default')
+			),
+			'hostid' => get_request('hostid', null),
+			'groupid' => get_request('groupid', null)
+		));
 	}
 
 	// get element
