@@ -26,7 +26,6 @@ require_once dirname(__FILE__).'/include/items.inc.php';
 
 $page['title'] = _('Trigger form');
 $page['file'] = 'tr_logform.php';
-$page['scripts'] = array();
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 
 define('ZBX_PAGE_NO_MENU', 1);
@@ -34,35 +33,37 @@ define('ZBX_PAGE_NO_MENU', 1);
 require_once dirname(__FILE__).'/include/page_header.php';
 
 //		VAR			TYPE	OPTIONAL FLAGS	VALIDATION	EXCEPTION
+$fields = array(
+	'description' =>	array(T_ZBX_STR, O_OPT,  NULL,			NOT_EMPTY,			'isset({save_trigger})'),
+	'itemid' =>			array(T_ZBX_INT, O_OPT,	 P_SYS,			DB_ID,				'isset({save_trigger})'),
+	'sform' =>			array(T_ZBX_INT, O_OPT,  NULL,			IN('0,1'),			null),
+	'sitems' =>			array(T_ZBX_INT, O_OPT,  NULL,			IN('0,1'),			null),
+	'triggerid' =>		array(T_ZBX_INT, O_OPT,  P_SYS,			DB_ID,				null),
+	'type' =>			array(T_ZBX_INT, O_OPT,  NULL,			IN('0,1'),			null),
+	'priority' =>		array(T_ZBX_INT, O_OPT,  NULL,			IN('0,1,2,3,4,5'),	'isset({save_trigger})'),
+	'expressions' =>	array(T_ZBX_STR, O_OPT,	 NULL,			NOT_EMPTY,			'isset({save_trigger})'),
+	'expr_type' =>		array(T_ZBX_INT, O_OPT,  NULL,			IN('0,1'),			null),
+	'comments' =>		array(T_ZBX_STR, O_OPT,  null,			null,				null),
+	'url' =>			array(T_ZBX_STR, O_OPT,  null,			null,				null),
+	'status' =>			array(T_ZBX_INT, O_OPT,  NULL,			IN('0,1'),			null),
+	'form_refresh' =>	array(T_ZBX_INT, O_OPT,	 NULL,			NULL,				NULL),
+	'save_trigger' =>	array(T_ZBX_STR, O_OPT,	 P_SYS|P_ACT,	NULL,				null),
+	'keys '=> 			array(T_ZBX_STR, O_OPT,  NULL,			NULL,				NULL),
+);
+check_fields($fields);
 
-	$fields=array(
-		'description'=>		array(T_ZBX_STR, O_OPT,  NULL,		NOT_EMPTY,	'isset({save_trigger})'),
-		'itemid'=>			array(T_ZBX_INT, O_OPT,	 P_SYS,		DB_ID,	'isset({save_trigger})'),
-		'sform'=>			array(T_ZBX_INT, O_OPT,  NULL,	  	IN('0,1'),	null),
-		'sitems'=>			array(T_ZBX_INT, O_OPT,  NULL, 		IN('0,1'),	null),
+/*
+ * Permissions
+ */
+if (get_request('itemid') && !API::Item()->isWritable(array($_REQUEST['itemid']))
+		|| get_request('triggerid') && !API::Trigger()->isWritable(array($_REQUEST['triggerid']))) {
+	access_deny();
+}
 
-		'groupid'=>			array(T_ZBX_INT, O_OPT,	 P_SYS,		DB_ID,	null),
-		'hostid'=>			array(T_ZBX_INT, O_OPT,  P_SYS,		DB_ID,	null),
-		'triggerid'=>		array(T_ZBX_INT, O_OPT,  P_SYS,		DB_ID,	null),
-
-		'type'=>			array(T_ZBX_INT, O_OPT,  NULL, 		IN('0,1'),	null),
-		'priority'=>		array(T_ZBX_INT, O_OPT,  NULL, 		IN('0,1,2,3,4,5'),	'isset({save_trigger})'),
-		'expressions'=>		array(T_ZBX_STR, O_OPT,	 NULL,		NOT_EMPTY,	'isset({save_trigger})'),
-		'expr_type'=>		array(T_ZBX_INT, O_OPT,  NULL, 		IN('0,1'),	null),
-		'comments'=>		array(T_ZBX_STR, O_OPT,  null,  	null, null),
-		'url'=>				array(T_ZBX_STR, O_OPT,  null,  	null, null),
-		'status'=>			array(T_ZBX_INT, O_OPT,  NULL, 		IN('0,1'),	null),
-		'form_refresh'=>	array(T_ZBX_INT, O_OPT,	 NULL,		NULL,	NULL),
-		'save_trigger'=>	array(T_ZBX_STR, O_OPT,	 P_SYS|P_ACT,	NULL,	null),
-		'keys'=> 			array(T_ZBX_STR, O_OPT,  NULL,		NULL,	NULL),
-	);
-
-	check_fields($fields);
-
-	$itemid = get_request('itemid',0);
+$itemid = get_request('itemid', 0);
 
 //------------------------ <ACTIONS> ---------------------------
-if(isset($_REQUEST['save_trigger'])){
+if (isset($_REQUEST['save_trigger'])) {
 	show_messages();
 
 	$exprs = get_request('expressions', false);
@@ -174,7 +175,7 @@ if(isset($_REQUEST['sform'])){
 		$sql = 'SELECT DISTINCT f.functionid, f.function, f.parameter, t.expression, '.
 								' t.description, t.priority, t.comments, t.url, t.status, t.type'.
 					' FROM functions f, triggers t, items i '.
-					' WHERE t.triggerid='.$_REQUEST['triggerid'].
+					' WHERE t.triggerid='.zbx_dbstr($_REQUEST['triggerid']).
 						' AND i.itemid=f.itemid '.
 						' AND f.triggerid = t.triggerid '.
 						' AND i.value_type IN ('.ITEM_VALUE_TYPE_LOG.' , '.ITEM_VALUE_TYPE_TEXT.', '.ITEM_VALUE_TYPE_STR.')';
@@ -203,16 +204,12 @@ if(isset($_REQUEST['sform'])){
 		$expr_v = preg_replace('/\(\(\((.+?)\)\) &/i', '(($1) &', $expr_v);
 		$expr_v = preg_replace('/\(\(\((.+?)\)\)$/i', '(($1)', $expr_v);
 
-		$expression = preg_split('/ [&|] /',$expression);
-		$expr_v = preg_split('/ [&|] /',$expr_v);
+		$expression = splitByFirstLevel($expression);
+		$expr_v = splitByFirstLevel($expr_v);
 
 		foreach($expression as $id => $expr){
 			$expr = preg_replace('/^\((.*)\)$/u','$1',$expr);
 
-			if(preg_match('/\([regexp|iregexp].+\)[=|#]0/U',$expr, $rr)){
-				$value = preg_replace('/(\(([regexp|iregexp].*)\)[=|#]0)/U','$2',$expr);
-			}
-
 			$value = preg_replace('/([=|#]0)/','',$expr);
 			$value = preg_replace('/^\((.*)\)$/u','$1',$value); // removing wrapping parentheses
 
@@ -256,11 +253,11 @@ if(isset($_REQUEST['sform'])){
 	$frmTRLog->addRow(_('Description'), new CTextBox('description', $description, 80));
 
 	$item = '';
-	$db_items = DBselect('SELECT DISTINCT * FROM items WHERE itemid='.$itemid);
+	$db_items = DBselect('SELECT DISTINCT * FROM items WHERE itemid='.zbx_dbstr($itemid));
 	while($db_item = DBfetch($db_items)){
 		if($db_item['templateid']){
 			$template_host = get_realhost_by_itemid($db_item['templateid']);
-			$item = $template_host['host'].':';
+			$item = $template_host['host'].NAME_DELIMITER;
 		}
 
 		$item .= itemName($db_item,$db_item['key_']);
